@@ -14,6 +14,8 @@
 #include <string.h>
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
+#include <winsock2.h>
+#include <ole2.h>
 #include <io.h>
 
 #include "../../misc/coding/coding.h"
@@ -37,7 +39,7 @@ GSM_Error bluetooth_connect(GSM_StateMachine *s, int port, char *device)
 	if (d->hPhone == INVALID_SOCKET) {
 		i = GetLastError();
 		GSM_OSErrorInfo(s, "Socket in bluetooth_open");
-		if (i == 10041) return ERR_DEVICENODRIVER;//unknown socket type
+		if (i == 10041) return ERR_DEVICENODRIVER;/* unknown socket type */
 		return ERR_UNKNOWN;
 	}
 
@@ -65,9 +67,9 @@ GSM_Error bluetooth_connect(GSM_StateMachine *s, int port, char *device)
 	if (connect (d->hPhone, (struct sockaddr *)&sab, sizeof(sab)) != 0) {
 		i = GetLastError();
 		GSM_OSErrorInfo(s, "Connect in bluetooth_open");
-		if (i == 10060) return ERR_TIMEOUT;	 //remote device failed to respond
-		if (i == 10050) return ERR_DEVICENOTWORK; //socket operation connected with dead network
-		//noauth
+		if (i == 10060) return ERR_TIMEOUT;	 /* remote device failed to respond */
+		if (i == 10050) return ERR_DEVICENOTWORK; /* socket operation connected with dead network */
+		/* noauth */
 		close(d->hPhone);
 		return ERR_UNKNOWN;
 	}
@@ -80,7 +82,6 @@ GSM_Error bluetooth_connect(GSM_StateMachine *s, int port, char *device)
 DEFINE_GUID(L2CAP_PROTOCOL_UUID,  0x00000100, 0x0000, 0x1000, 0x80, 0x00, 0x00, 0x80, 0x5F, 0x9B, 0x34, 0xFB);
 
 #ifndef __GNUC__
-#pragma comment(lib, "irprops.lib")
 #pragma comment(lib, "ws2_32.lib")
 #endif
 
@@ -162,7 +163,7 @@ GSM_Error bluetooth_findchannel(GSM_StateMachine *s)
 	if (d->hPhone == INVALID_SOCKET) {
 		i = GetLastError();
 		GSM_OSErrorInfo(s, "Socket in bluetooth_open");
-		if (i == 10041) return ERR_DEVICENODRIVER;//unknown socket type
+		if (i == 10041) return ERR_DEVICENODRIVER;/* unknown socket type */
 		return ERR_UNKNOWN;
 	}
 
