@@ -73,9 +73,9 @@ static GSM_Error serial_close(GSM_StateMachine *s)
 }
 
 #if defined(GSM_ENABLE_DKU2PHONET) || defined(GSM_ENABLE_DKU2AT)
-  DEFINE_GUID(DKU2AT,    0x4F919104, 0x4adf, 0x11d5, 0x88, 0x2d, 0x0, 0xb0, 0xd0, 0x2f, 0xe3, 0x81); //"4F919104-4ADF-11D5-882D-00B0D02FE381"
-  DEFINE_GUID(DKU2FBUS2, 0x4F919102, 0x4adf, 0x11d5, 0x88, 0x2d, 0x0, 0xb0, 0xd0, 0x2f, 0xe3, 0x81); //"4F919102-4ADF-11D5-882D-00B0D02FE381"
-  DEFINE_GUID(DKU2OBEX,  0x4F919100, 0x4adf, 0x11d5, 0x88, 0x2d, 0x0, 0xb0, 0xd0, 0x2f, 0xe3, 0x81); //"4F919100-4ADF-11D5-882D-00B0D02FE381"
+  DEFINE_GUID(DKU2AT,    0x4F919104, 0x4adf, 0x11d5, 0x88, 0x2d, 0x0, 0xb0, 0xd0, 0x2f, 0xe3, 0x81); /* "4F919104-4ADF-11D5-882D-00B0D02FE381" */
+  DEFINE_GUID(DKU2FBUS2, 0x4F919102, 0x4adf, 0x11d5, 0x88, 0x2d, 0x0, 0xb0, 0xd0, 0x2f, 0xe3, 0x81); /* "4F919102-4ADF-11D5-882D-00B0D02FE381" */
+  DEFINE_GUID(DKU2OBEX,  0x4F919100, 0x4adf, 0x11d5, 0x88, 0x2d, 0x0, 0xb0, 0xd0, 0x2f, 0xe3, 0x81); /* "4F919100-4ADF-11D5-882D-00B0D02FE381" */
 #endif
 
 static GSM_Error serial_open (GSM_StateMachine *s)
@@ -146,7 +146,7 @@ static GSM_Error serial_open (GSM_StateMachine *s)
 #ifdef GSM_ENABLE_DKU5FBUS2
 	if (s->ConnectionType == GCT_DKU5FBUS2) {
 		smprintf(s,"Reading DKU5 device\n");
-		//win xp
+		/* win xp */
 		DeviceName2[0] = 0;
 		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\SERIALCOMM", 0, KEY_QUERY_VALUE, &hKey) != ERROR_SUCCESS) {
 			smprintf(s,"Error opening key\n");
@@ -163,7 +163,7 @@ static GSM_Error serial_open (GSM_StateMachine *s)
 		}
 		RegCloseKey(hKey);
 		if (strlen(DeviceName2) == 0) {
-			// win 98
+			/*  win 98 */
 			if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Enum\\USBVSP\\ATMELPORT\\0000", 0, KEY_QUERY_VALUE, &hKey) != ERROR_SUCCESS) {
 				smprintf(s,"Error opening key\n");
 				return ERR_DEVICENOTWORK;
@@ -205,9 +205,9 @@ static GSM_Error serial_open (GSM_StateMachine *s)
 	if (d->hPhone == INVALID_HANDLE_VALUE) {
 		i = GetLastError();
 		GSM_OSErrorInfo(s, "CreateFile in serial_open");
-		if (i == 2)   return ERR_DEVICENOTWORK; //can't find specified file
-		if (i == 5)   return ERR_DEVICEBUSY;    //access denied
-		if (i == 31)  return ERR_DEVICENOTWORK; //attached device not working
+		if (i == 2)   return ERR_DEVICENOTWORK; /* can't find specified file */
+		if (i == 5)   return ERR_DEVICEBUSY;    /* access denied */
+		if (i == 31)  return ERR_DEVICENOTWORK; /* attached device not working */
 		if (i == 123) return ERR_DEVICENOTEXIST;
 		return ERR_DEVICEOPENERROR;
 	}
@@ -238,14 +238,14 @@ static GSM_Error serial_open (GSM_StateMachine *s)
 		dcb.StopBits 	 = ONESTOPBIT;
 
 		/* No Xon/Xof flow control */
-	//	dcb.fOutX 	 = false;
-	//	dcb.fInX 	 = false;
+	/* 	dcb.fOutX 	 = false; */
+	/* 	dcb.fInX 	 = false; */
 
 		/* Hardware flow control */
-	//	dcb.fOutxDsrFlow = true;
-	//	dcb.fOutxCtsFlow = true;
-	//	dcb.fDtrControl  = DTR_CONTROL_HANDSHAKE;
-	//	dcb.fRtsControl  = RTS_CONTROL_HANDSHAKE;
+	/* 	dcb.fOutxDsrFlow = true; */
+	/* 	dcb.fOutxCtsFlow = true; */
+	/* 	dcb.fDtrControl  = DTR_CONTROL_HANDSHAKE; */
+	/* 	dcb.fRtsControl  = RTS_CONTROL_HANDSHAKE; */
 
 	    	/* Initialise the port settings */
 		if (SetCommState(d->hPhone, &dcb)==0) {
@@ -287,6 +287,8 @@ static GSM_Error serial_setdtrrts(GSM_StateMachine *s, bool dtr, bool rts)
 	DCB        		dcb;
 	GSM_Device_SerialData 	*d = &s->Device.Data.Serial;
 
+	if (s->SkipDtrRts) return ERR_NONE;
+
 	dcb.DCBlength = sizeof(DCB);
 	if (GetCommState(d->hPhone, &dcb)==0) {
 		GSM_OSErrorInfo(s, "ReadDevice in serial_setdtrrts");
@@ -314,21 +316,21 @@ static GSM_Error serial_setdtrrts(GSM_StateMachine *s, bool dtr, bool rts)
 	dcb.DCBlength = sizeof(DCB);
 	GetCommState(d->hPhone, &dcb);
 
-	dbgprintf("Serial device:");
-	dbgprintf(" DTR is ");
+	smprintf(s, "Serial device:");
+	smprintf(s, " DTR is ");
 	switch (dcb.fDtrControl) {
-		case DTR_CONTROL_ENABLE    : dbgprintf("up");        break;
-		case DTR_CONTROL_DISABLE   : dbgprintf("down");      break;
-		case DTR_CONTROL_HANDSHAKE : dbgprintf("handshake"); break;
+		case DTR_CONTROL_ENABLE    : smprintf(s, "up");        break;
+		case DTR_CONTROL_DISABLE   : smprintf(s, "down");      break;
+		case DTR_CONTROL_HANDSHAKE : smprintf(s, "handshake"); break;
 	}
-	dbgprintf(", RTS is ");
+	smprintf(s, ", RTS is ");
 	switch (dcb.fRtsControl) {
-		case RTS_CONTROL_ENABLE    : dbgprintf("up");        break;
-		case RTS_CONTROL_DISABLE   : dbgprintf("down");      break;
-		case RTS_CONTROL_HANDSHAKE : dbgprintf("handshake"); break;
-		case RTS_CONTROL_TOGGLE    : dbgprintf("toggle");    break;
+		case RTS_CONTROL_ENABLE    : smprintf(s, "up");        break;
+		case RTS_CONTROL_DISABLE   : smprintf(s, "down");      break;
+		case RTS_CONTROL_HANDSHAKE : smprintf(s, "handshake"); break;
+		case RTS_CONTROL_TOGGLE    : smprintf(s, "toggle");    break;
 	}
-	dbgprintf("\n");
+	smprintf(s, "\n");
 	if ( dtr && dcb.fDtrControl != DTR_CONTROL_ENABLE ) return ERR_DEVICEDTRRTSERROR;
 	if (!dtr && dcb.fDtrControl != DTR_CONTROL_DISABLE) return ERR_DEVICEDTRRTSERROR;
 	if ( rts && dcb.fRtsControl != RTS_CONTROL_ENABLE ) return ERR_DEVICEDTRRTSERROR;
@@ -342,6 +344,8 @@ static GSM_Error serial_setspeed(GSM_StateMachine *s, int speed)
 	DCB			dcb;
 	GSM_Device_SerialData 	*d = &s->Device.Data.Serial;
 
+	if (s->SkipDtrRts) return ERR_NONE;
+
 	dcb.DCBlength = sizeof(DCB);
 	if (GetCommState(d->hPhone, &dcb)==0) {
 		GSM_OSErrorInfo(s, "ReadDevice in serial_setspeed");
@@ -355,49 +359,57 @@ static GSM_Error serial_setspeed(GSM_StateMachine *s, int speed)
 		return ERR_DEVICECHANGESPEEDERROR;
 	}
 
+    	smprintf(s, "Setting speed to %d\n", speed);
+
 	return ERR_NONE;
 }
 
 static int serial_read(GSM_StateMachine *s, void *buf, size_t nbytes)
 {
 	COMSTAT			ComStat;
-	DWORD			ErrorFlags, Length;
+	DWORD			ErrorFlags, Length, Error;
 	GSM_Device_SerialData 	*d = &s->Device.Data.Serial;
+	BOOL			ReadStatus;
 
-	if (s->ConnectionType != GCT_DKU2PHONET && s->ConnectionType != GCT_DKU2AT) {
-		/* Gets information about a communications error and
-		 * current status of device
-		 */
-		ClearCommError(d->hPhone, &ErrorFlags, &ComStat);
-		Length = ComStat.cbInQue;
+	/* Gets information about a communications error and
+	 * current status of device
+	 */
+	ClearCommError(d->hPhone, &ErrorFlags, &ComStat);
 
-		/* Nothing to read */
-		if (Length <= 0) return Length;
+	/* How much we can read? */
+	Length = min(nbytes, ComStat.cbInQue);
 
-		if (Length > nbytes) Length = nbytes;
-	} else {
-		Length = 5;
-	}
+	/* Nothing to read */
+	if (Length <= 0) goto end;
 
 	/* Read without problems */
-	if (ReadFile(d->hPhone, buf, Length, &Length, &d->osRead)) return Length;
+	ReadStatus = ReadFile(d->hPhone, buf, Length, &Length, &d->osRead);
 
-	if (GetLastError() != ERROR_IO_PENDING) {
-		GSM_OSErrorInfo(s, "serial_read1");
-		Length = 0;
-		ClearCommError(d->hPhone, &ErrorFlags, &ComStat);
-		return Length;
-	}
-
-	while(1) {
-		if (GetOverlappedResult(d->hPhone,&d->osRead, &Length, TRUE)) break;
-		if (GetLastError() != ERROR_IO_INCOMPLETE) {
-			GSM_OSErrorInfo(s, "serial_read2");
-			/* an error occurred, try to recover */
+	if (!ReadStatus) {
+		if (GetLastError() != ERROR_IO_PENDING) {
+			Length = 0;
 			ClearCommError(d->hPhone, &ErrorFlags, &ComStat);
-			break;
+			goto end;
+		}
+
+		/* Operation did not complete */
+		GSM_OSErrorInfo(s, "win_serial_read-pending");
+
+		/* Wait for completing */
+		while (!GetOverlappedResult(d->hPhone, &d->osRead, &Length, TRUE)) {
+			Error = GetLastError();
+			if (Error == ERROR_IO_INCOMPLETE) {
+				/* Just go on */
+				continue;
+			} else {
+				/* Something went wrong, bail out */
+				GSM_OSErrorInfo(s, "win_serial_read-overlapped");
+				ClearCommError(d->hPhone, &ErrorFlags, &ComStat);
+				break;
+			}
 		}
 	}
+end:
 	return Length;
 }
 
