@@ -2,11 +2,16 @@
 /* Phones ID (c) partially by Walek */
 
 #include <stdarg.h>
+#define _GNU_SOURCE /* For strcasestr */
 #include <string.h>
 #include <errno.h>
+#include <limits.h>
 
 #include <gammu-call.h>
 #include <gammu-settings.h>
+#include <gammu-unicode.h>
+#include <gammu-config.h>
+#include <gammu-misc.h>
 
 #include "gsmcomon.h"
 #include "gsmphones.h"
@@ -15,11 +20,20 @@
 #include "device/devfunc.h"
 
 #if defined(WIN32) || defined(DJGPP)
+/* Needed for SHGFP_TYPE_CURRENT */
+#define _WIN32_IE 0x0500
 #include <shlobj.h>
 
 #define FALLBACK_GAMMURC "gammurc"
+#define GAMMURC_NAME "\\gammurc"
 #else
 #define FALLBACK_GAMMURC "/etc/gammurc"
+#define GAMMURC_NAME "/.gammurc"
+#endif
+
+/* Win32 compatibility */
+#ifndef PATH_MAX
+#define PATH_MAX (MAX_PATH)
 #endif
 
 static void GSM_RegisterConnection(GSM_StateMachine *s, unsigned int connection,
@@ -34,85 +48,115 @@ static void GSM_RegisterConnection(GSM_StateMachine *s, unsigned int connection,
 typedef struct {
 	const char *Name;
 	const GSM_ConnectionType Connection;
+	bool SkipDtrRts;
 } GSM_ConnectionInfo;
 
 /**
  * Mapping of connection names to internal identifications.
  */
 static const GSM_ConnectionInfo GSM_Connections[] = {
-	{"at", GCT_AT},
+	{"at", GCT_AT, false},
 
-	// cables
-	{"mbus", GCT_MBUS2},
-	{"fbus", GCT_FBUS2},
-	{"fbus-nodtr", GCT_FBUS2NODTR},
-	{"fbuspl2303", GCT_FBUS2PL2303},
-	{"dlr3", GCT_FBUS2DLR3},
-	{"dlr3-nodtr", GCT_FBUS2DLR3NODTR},
-	{"fbusdlr3", GCT_FBUS2DLR3},
-	{"dku5", GCT_DKU5FBUS2},
-	{"dku5fbus", GCT_DKU5FBUS2},
-	{"dku5fbus-nodtr", GCT_DKU5FBUS2NODTR},
-	{"ark3116fbus", GCT_ARK3116FBUS2},
-	{"dku2", GCT_DKU2PHONET},
-	{"dku2phonet", GCT_DKU2PHONET},
-	{"dku2at", GCT_DKU2AT},
+	/* cables */
+	{"mbus", GCT_MBUS2, false},
+	{"fbus", GCT_FBUS2, false},
+	{"fbuspl2303", GCT_FBUS2PL2303, false},
+	{"dlr3", GCT_FBUS2DLR3, false},
+	{"fbusdlr3", GCT_FBUS2DLR3, false},
+	{"dku5", GCT_DKU5FBUS2, false},
+	{"dku5fbus", GCT_DKU5FBUS2, false},
+	{"ark3116fbus", GCT_DKU5FBUS2, true},
+	{"dku2", GCT_DKU2PHONET, false},
+	{"dku2phonet", GCT_DKU2PHONET, false},
+	{"dku2at", GCT_DKU2AT, false},
 
-        // for serial ports assigned by bt stack
-	{"fbusblue", GCT_FBUS2BLUE},
-	{"phonetblue", GCT_PHONETBLUE},
+        /* for serial ports assigned by bt stack */
+	{"fbusblue", GCT_FBUS2BLUE, false},
+	{"phonetblue", GCT_PHONETBLUE, false},
 
-	// bt
-	{"blueobex", GCT_BLUEOBEX},
-	{"bluephonet", GCT_BLUEPHONET},
-	{"blueat", GCT_BLUEAT},
-	{"bluerfobex", GCT_BLUEOBEX},
-	{"bluerffbus", GCT_BLUEFBUS2},
-	{"bluerfphonet", GCT_BLUEPHONET},
-	{"bluerfat", GCT_BLUEAT},
-	{"bluerfgnapbus", GCT_BLUEGNAPBUS},
+	/* bt */
+	{"blueobex", GCT_BLUEOBEX, false},
+	{"bluephonet", GCT_BLUEPHONET, false},
+	{"blueat", GCT_BLUEAT, false},
+	{"bluerfobex", GCT_BLUEOBEX, false},
+	{"bluerffbus", GCT_BLUEFBUS2, false},
+	{"bluerfphonet", GCT_BLUEPHONET, false},
+	{"bluerfat", GCT_BLUEAT, false},
+	{"bluerfgnapbus", GCT_BLUEGNAPBUS, false},
 
-	// old "serial" irda
-	{"infrared", GCT_FBUS2IRDA},
-	{"fbusirda", GCT_FBUS2IRDA},
+	/* old "serial" irda */
+	{"infrared", GCT_FBUS2IRDA, false},
+	{"fbusirda", GCT_FBUS2IRDA, false},
 
-	// socket irda
-	{"irda", GCT_IRDAPHONET},
-	{"irdaphonet", GCT_IRDAPHONET},
-	{"irdaat", GCT_IRDAAT},
-	{"irdaobex", GCT_IRDAOBEX},
-	{"irdagnapbus", GCT_IRDAGNAPBUS},
+	/* socket irda */
+	{"irda", GCT_IRDAPHONET, false},
+	{"irdaphonet", GCT_IRDAPHONET, false},
+	{"irdaat", GCT_IRDAAT, false},
+	{"irdaobex", GCT_IRDAOBEX, false},
+	{"irdagnapbus", GCT_IRDAGNAPBUS, false},
 };
 
-static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, char *connection)
+static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *connection)
 {
 	size_t i;
+	char *buff, *nodtr_pos, *nopower_pos;
+
+	/* Copy connection name, so that we can play with it */
+	buff = strdup(connection);
+	if (buff == NULL) {
+		return ERR_MOREMEMORY;
+	}
+
 	/* We check here is used connection string type is correct for ANY
 	 * OS. If not, we return with error, that string is incorrect at all
 	 */
 	s->ConnectionType = 0;
+	s->SkipDtrRts = false;
+	s->NoPowerCable = false;
 
+	/* Are we asked for connection using stupid cable? */
+	nodtr_pos = strcasestr(buff, "-nodtr");
+	if (nodtr_pos != NULL) {
+		*nodtr_pos = 0;
+	}
+
+	/* Are we asked for connection using cable which does not
+	 * use DTR/RTS as power supply? */
+	nopower_pos = strcasestr(buff, "-nopower");
+	if (nopower_pos != NULL) {
+		*nopower_pos = 0;
+		s->NoPowerCable = true;
+	}
+
+	/* Compare known connections to what we got */
 	for (i = 0; i < sizeof(GSM_Connections) / sizeof(GSM_Connections[0]); i++) {
-		if (strcasecmp(GSM_Connections[i].Name, connection) == 0) {
+		/* Check connection name */
+		if (strcasecmp(GSM_Connections[i].Name, buff) == 0) {
 			s->ConnectionType = GSM_Connections[i].Connection;
+			s->SkipDtrRts = GSM_Connections[i].SkipDtrRts;
 			break;
 		}
 	}
 
+	/* If we were forced, set this flag */
+	if (nodtr_pos != NULL) {
+		s->SkipDtrRts = true;
+	}
+
 	/* Special case - at can contains speed */
-	if (s->ConnectionType == 0 && strncasecmp("at", connection, 2) == 0) {
-		/* Use some resonable default, when no speed defined */
-		if (strlen(connection) == 2) {
-			s->Speed = 19200;
-		} else {
-			s->Speed = FindSerialSpeed(connection + 2);
-		}
+	if (s->ConnectionType == 0 && strncasecmp("at", buff, 2) == 0) {
+		s->Speed = FindSerialSpeed(buff + 2);
 		if (s->Speed != 0) {
 			s->ConnectionType = GCT_AT;
 		}
 	}
 
-	if (s->ConnectionType==0) return ERR_UNKNOWNCONNECTIONTYPESTRING;
+	/* Free allocated memory */
+	free(buff);
+
+	if (s->ConnectionType==0) {
+		return ERR_UNKNOWNCONNECTIONTYPESTRING;
+	}
 
 	/* We check now if user gave connection type compiled & available
 	 * for used OS (if not, we return, that source not available)
@@ -130,7 +174,6 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, char *connectio
 #endif
 #ifdef GSM_ENABLE_DKU5FBUS2
 	GSM_RegisterConnection(s, GCT_DKU5FBUS2,  &SerialDevice,  &FBUS2Protocol);
-	GSM_RegisterConnection(s, GCT_ARK3116FBUS2,  &SerialDevice,  &FBUS2Protocol);
 #endif
 #ifdef GSM_ENABLE_FBUS2PL2303
 	GSM_RegisterConnection(s, GCT_FBUS2PL2303,&SerialDevice,  &FBUS2Protocol);
@@ -189,7 +232,7 @@ static void GSM_RegisterModule(GSM_StateMachine *s,GSM_Phone_Functions *phone)
 {
 	/* Auto model */
 	if (s->CurrentConfig->Model[0] == 0) {
-		if (strstr(phone->models,GetModelData(NULL,s->Phone.Data.Model,NULL)->model) != NULL) {
+		if (strstr(phone->models,GetModelData(s, NULL, s->Phone.Data.Model, NULL)->model) != NULL) {
 			smprintf(s,"[Module           - \"%s\"]\n",phone->models);
 			s->Phone.Functions = phone;
 		}
@@ -201,13 +244,20 @@ static void GSM_RegisterModule(GSM_StateMachine *s,GSM_Phone_Functions *phone)
 	}
 }
 
+/**
+ * Tries to register all modules to find one matching current configuration.
+ *
+ * \param s State machine pointer.
+ *
+ * \return Error code, ERR_NONE on success.
+ */
 GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 {
 	OnePhoneModel *model;
 
 	/* Auto model */
 	if (s->CurrentConfig->Model[0] == 0) {
-		model = GetModelData(NULL,s->Phone.Data.Model,NULL);
+		model = GetModelData(s, NULL, s->Phone.Data.Model, NULL);
 #ifdef GSM_ENABLE_ATGEN
 		/* With ATgen and auto model we can work with unknown models too */
 		if (s->ConnectionType==GCT_AT || s->ConnectionType==GCT_BLUEAT || s->ConnectionType==GCT_IRDAAT || s->ConnectionType==GCT_DKU2AT) {
@@ -283,22 +333,127 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 #ifdef GSM_ENABLE_ATOBEX
 	GSM_RegisterModule(s,&ATOBEXPhone);
 #endif
-	if (s->Phone.Functions==NULL) return ERR_UNKNOWNMODELSTRING;
+	if (s->Phone.Functions == NULL) {
+		return ERR_UNKNOWNMODELSTRING;
+	}
+	return ERR_NONE;
+}
+
+
+/**
+ * Opens connection to device and initiates protocol layer.
+ */
+GSM_Error GSM_OpenConnection(GSM_StateMachine *s)
+{
+	GSM_Error error;
+
+	if (strcasecmp(s->CurrentConfig->LockDevice,"yes") == 0) {
+		error = lock_device(s->CurrentConfig->Device, &(s->LockFile));
+		if (error != ERR_NONE) return error;
+	}
+
+	/* Irda devices can set now model to some specific and
+	 * we don't have to make auto detection later */
+	error=s->Device.Functions->OpenDevice(s);
+	if (error!=ERR_NONE) {
+		if (s->LockFile != NULL)
+			unlock_device(&(s->LockFile));
+		return error;
+	}
+
+	s->opened = true;
+
+	error=s->Protocol.Functions->Initialise(s);
+	if (error!=ERR_NONE) return error;
+
+	return ERR_NONE;
+}
+
+/**
+ * Tries to read model using configured phone connection.
+ */
+GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
+{
+	GSM_Error error;
+
+	error = GSM_OpenConnection(s);
+	if (error != ERR_NONE) return error;
+
+	/* If still auto model, try to get model by asking phone for it */
+	if (s->Phone.Data.Model[0]==0) {
+		smprintf(s,"[Module           - \"auto\"]\n");
+		switch (s->ConnectionType) {
+#ifdef GSM_ENABLE_ATGEN
+			case GCT_AT:
+			case GCT_BLUEAT:
+			case GCT_IRDAAT:
+			case GCT_DKU2AT:
+				s->Phone.Functions = &ATGENPhone;
+				break;
+#endif
+#ifdef GSM_ENABLE_OBEXGEN
+			case GCT_IRDAOBEX:
+			case GCT_BLUEOBEX:
+				s->Phone.Functions = &OBEXGENPhone;
+				break;
+#endif
+#ifdef GSM_ENABLE_GNAPGEN
+			case GCT_BLUEGNAPBUS:
+			case GCT_IRDAGNAPBUS:
+				s->Phone.Functions = &GNAPGENPhone;
+				break;
+#endif
+#if defined(GSM_ENABLE_NOKIA_DCT3) || defined(GSM_ENABLE_NOKIA_DCT4)
+			case GCT_MBUS2:
+			case GCT_FBUS2:
+			case GCT_FBUS2DLR3:
+			case GCT_FBUS2PL2303:
+			case GCT_FBUS2BLUE:
+			case GCT_FBUS2IRDA:
+			case GCT_DKU5FBUS2:
+			case GCT_DKU2PHONET:
+			case GCT_PHONETBLUE:
+			case GCT_IRDAPHONET:
+			case GCT_BLUEFBUS2:
+			case GCT_BLUEPHONET:
+				s->Phone.Functions = &NAUTOPhone;
+				break;
+#endif
+			default:
+				s->Phone.Functions = NULL;
+		}
+		if (s->Phone.Functions == NULL) return ERR_UNKNOWN;
+
+		/* Please note, that AT module need to send first
+		 * command for enabling echo
+		 */
+		error=s->Phone.Functions->Initialise(s);
+		if (error != ERR_NONE) return error;
+
+		error=s->Phone.Functions->GetModel(s);
+		if (error != ERR_NONE) return error;
+	}
 	return ERR_NONE;
 }
 
 GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 {
 	GSM_Error	error;
-	GSM_DateTime	time;
+	GSM_DateTime	current_time;
 	int		i;
 
 	for (i=0;i<s->ConfigNum;i++) {
 		s->CurrentConfig		  = &s->Config[i];
 
+		/* Skip non configured sections */
+		if (s->CurrentConfig->Connection == NULL) {
+			smprintf_level(s, D_ERROR, "[Empty section    - %d]\n", i);
+			continue;
+		}
+
 		s->Speed			  = 0;
 		s->ReplyNum			  = ReplyNum;
-		s->Phone.Data.ModelInfo		  = GetModelData("unknown",NULL,NULL);
+		s->Phone.Data.ModelInfo		  = GetModelData(s, "unknown", NULL, NULL);
 		s->Phone.Data.Manufacturer[0]	  = 0;
 		s->Phone.Data.Model[0]		  = 0;
 		s->Phone.Data.Version[0]	  = 0;
@@ -329,26 +484,24 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 		error=GSM_SetDebugFile(s->CurrentConfig->DebugFile, &s->di);
 		if (error != ERR_NONE) return error;
 
-		if (s->di.dl == DL_TEXTALL || s->di.dl == DL_TEXT || s->di.dl == DL_TEXTERROR ||
-	    	    s->di.dl == DL_TEXTALLDATE || s->di.dl == DL_TEXTDATE || s->di.dl == DL_TEXTERRORDATE) {
-			smprintf(s, "[Gammu            - %s built %s %s using %s]\n",
-					VERSION,
-					__TIME__,
-					__DATE__,
-					GetCompiler()
-					);
-			smprintf(s, "[Connection       - \"%s\"]\n",
-					s->CurrentConfig->Connection);
-			smprintf(s, "[Connection index - %d]\n", i);
-			smprintf(s, "[Model type       - \"%s\"]\n",
-					s->CurrentConfig->Model);
-			smprintf(s, "[Device           - \"%s\"]\n",
-					s->CurrentConfig->Device);
-			if (strlen(GetOS()) != 0) {
-				smprintf(s, "[Runing on        - %s]\n",
-						GetOS());
-			}
+		smprintf_level(s, D_ERROR, "[Gammu            - %s built %s %s using %s]\n",
+				VERSION,
+				__TIME__,
+				__DATE__,
+				GetCompiler()
+				);
+		smprintf_level(s, D_ERROR, "[Connection       - \"%s\"]\n",
+				s->CurrentConfig->Connection);
+		smprintf_level(s, D_ERROR, "[Connection index - %d]\n", i);
+		smprintf_level(s, D_ERROR, "[Model type       - \"%s\"]\n",
+				s->CurrentConfig->Model);
+		smprintf_level(s, D_ERROR, "[Device           - \"%s\"]\n",
+				s->CurrentConfig->Device);
+		if (strlen(GetOS()) != 0) {
+			smprintf_level(s, D_ERROR, "[Runing on        - %s]\n",
+					GetOS());
 		}
+
 		if (s->di.dl==DL_BINARY) {
 			smprintf(s,"%c",((unsigned char)strlen(VERSION)));
 			smprintf(s,"%s",VERSION);
@@ -357,16 +510,10 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 		error=GSM_RegisterAllConnections(s, s->CurrentConfig->Connection);
 		if (error!=ERR_NONE) return error;
 
+autodetect:
 		/* Model auto */
-		if (s->CurrentConfig->Model[0]==0) {
-			if (strcasecmp(s->CurrentConfig->LockDevice,"yes") == 0) {
-				error = lock_device(s->CurrentConfig->Device, &(s->LockFile));
-				if (error != ERR_NONE) return error;
-			}
-
-			/* Irda devices can set now model to some specific and
-			 * we don't have to make auto detection later */
-			error=s->Device.Functions->OpenDevice(s);
+		if (s->CurrentConfig->Model[0] == 0) {
+			error = GSM_TryGetModel(s);
 			if (i != s->ConfigNum - 1) {
 				if (error == ERR_DEVICEOPENERROR) 	continue;
 				if (error == ERR_DEVICELOCKED) 	 	continue;
@@ -375,88 +522,25 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 				if (error == ERR_DEVICENOPERMISSION) 	continue;
 				if (error == ERR_DEVICENODRIVER) 	continue;
 				if (error == ERR_DEVICENOTWORK) 	continue;
+				if (error == ERR_TIMEOUT)	 	continue;
 			}
- 			if (error!=ERR_NONE) {
- 				if (s->LockFile!=NULL) unlock_device(&(s->LockFile));
- 				return error;
- 			}
-
-			s->opened = true;
-
-			error=s->Protocol.Functions->Initialise(s);
-			if (error!=ERR_NONE) return error;
-
-			/* If still auto model, try to get model by asking phone for it */
-			if (s->Phone.Data.Model[0]==0) {
-				smprintf(s,"[Module           - \"auto\"]\n");
-				switch (s->ConnectionType) {
-#ifdef GSM_ENABLE_ATGEN
-					case GCT_AT:
-					case GCT_BLUEAT:
-					case GCT_IRDAAT:
-					case GCT_DKU2AT:
-						s->Phone.Functions = &ATGENPhone;
-						break;
-#endif
-#ifdef GSM_ENABLE_OBEXGEN
-					case GCT_IRDAOBEX:
-					case GCT_BLUEOBEX:
-						s->Phone.Functions = &OBEXGENPhone;
-						break;
-#endif
-#ifdef GSM_ENABLE_GNAPGEN
-					case GCT_BLUEGNAPBUS:
-					case GCT_IRDAGNAPBUS:
-						s->Phone.Functions = &GNAPGENPhone;
-						break;
-#endif
-#if defined(GSM_ENABLE_NOKIA_DCT3) || defined(GSM_ENABLE_NOKIA_DCT4)
-					case GCT_MBUS2:
-					case GCT_FBUS2:
-					case GCT_FBUS2DLR3:
-					case GCT_FBUS2PL2303:
-					case GCT_FBUS2BLUE:
-					case GCT_FBUS2IRDA:
-					case GCT_DKU5FBUS2:
-					case GCT_ARK3116FBUS2:
-					case GCT_DKU2PHONET:
-					case GCT_PHONETBLUE:
-					case GCT_IRDAPHONET:
-					case GCT_BLUEFBUS2:
-					case GCT_BLUEPHONET:
-						s->Phone.Functions = &NAUTOPhone;
-						break;
-#endif
-					default:
-						s->Phone.Functions = NULL;
-				}
-				if (s->Phone.Functions == NULL) return ERR_UNKNOWN;
-
-				/* Please note, that AT module need to send first
-				 * command for enabling echo
-				 */
-				error=s->Phone.Functions->Initialise(s);
-				if (error == ERR_TIMEOUT && i != s->ConfigNum - 1) continue;
-				if (error != ERR_NONE) return error;
-
-				error=s->Phone.Functions->GetModel(s);
-				if (error == ERR_TIMEOUT && i != s->ConfigNum - 1) continue;
-				if (error != ERR_NONE) return error;
-			}
+			if (error != ERR_NONE) return error;
 		}
 
 		/* Switching to "correct" module */
-		error=GSM_RegisterAllPhoneModules(s);
-		if (error!=ERR_NONE) return error;
+		error = GSM_RegisterAllPhoneModules(s);
+		/* If user selected soemthing which is not supported, try autodetection */
+		if (s->CurrentConfig->Model[0] != 0 && error == ERR_UNKNOWNMODELSTRING) {
+			smprintf(s, "Configured model %s is not known, retrying with autodetection!\n",
+					s->CurrentConfig->Model);
+			s->CurrentConfig->Model[0] = 0;
+			goto autodetect;
+		}
+		if (error != ERR_NONE) return error;
 
 		/* We didn't open device earlier ? Make it now */
 		if (!s->opened) {
-			if (strcasecmp(s->CurrentConfig->LockDevice,"yes") == 0) {
-				error = lock_device(s->CurrentConfig->Device, &(s->LockFile));
-				if (error != ERR_NONE) return error;
-			}
-
-			error=s->Device.Functions->OpenDevice(s);
+			error = GSM_OpenConnection(s);
 			if (i != s->ConfigNum - 1) {
 				if (error == ERR_DEVICEOPENERROR) 	continue;
 				if (error == ERR_DEVICELOCKED) 	 	continue;
@@ -466,17 +550,10 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 				if (error == ERR_DEVICENODRIVER) 	continue;
 				if (error == ERR_DEVICENOTWORK) 	continue;
 			}
-			if (error!=ERR_NONE) {
-				if (s->LockFile!=NULL) unlock_device(&(s->LockFile));
-				return error;
-			}
-
-			s->opened = true;
-
-			error=s->Protocol.Functions->Initialise(s);
-			if (error!=ERR_NONE) return error;
+			if (error != ERR_NONE) return error;
 		}
 
+		/* Initialize phone layer */
 		error=s->Phone.Functions->Initialise(s);
 		if (error == ERR_TIMEOUT && i != s->ConfigNum - 1) continue;
 		if (error != ERR_NONE) return error;
@@ -487,24 +564,27 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 		}
 
 		if (strcasecmp(s->CurrentConfig->SyncTime,"yes") == 0) {
-			GSM_GetCurrentDateTime (&time);
-			s->Phone.Functions->SetDateTime(s,&time);
+			GSM_GetCurrentDateTime (&current_time);
+			s->Phone.Functions->SetDateTime(s,&current_time);
 		}
 
 		/* For debug it's good to have firmware and real model version and manufacturer */
 		error=s->Phone.Functions->GetManufacturer(s);
 		if (error == ERR_TIMEOUT && i != s->ConfigNum - 1) continue;
 		if (error != ERR_NONE) return error;
+
 		error=s->Phone.Functions->GetModel(s);
 		if (error != ERR_NONE) return error;
+
 		error=s->Phone.Functions->GetFirmware(s);
 		if (error != ERR_NONE) return error;
+
 		return ERR_NONE;
 	}
 	return ERR_UNCONFIGURED;
 }
 
-int GSM_ReadDevice (GSM_StateMachine *s, bool wait)
+int GSM_ReadDevice (GSM_StateMachine *s, bool waitforreply)
 {
 	unsigned char	buff[255];
 	int		res = 0, count;
@@ -516,7 +596,7 @@ int GSM_ReadDevice (GSM_StateMachine *s, bool wait)
 	i=Date.Second;
 	while (i==Date.Second) {
 		res = s->Device.Functions->ReadDevice(s, buff, 255);
-		if (!wait) break;
+		if (!waitforreply) break;
 		if (res > 0) break;
 		my_sleep(5);
 		GSM_GetCurrentDateTime(&Date);
@@ -572,7 +652,7 @@ bool GSM_IsConnected(GSM_StateMachine *s) {
 }
 
 GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
-			  int length, unsigned char type, int time)
+			  int length, unsigned char type, int timeout)
 {
 	GSM_Phone_Data			*Phone = &s->Phone.Data;
 	GSM_Protocol_Message 		sentmsg;
@@ -600,13 +680,13 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 		if (Phone->RequestID==ID_None) return Phone->DispatchError;
 
 		i++;
-	} while (i<time);
+	} while (i<timeout);
 
 	return ERR_TIMEOUT;
 }
 
 GSM_Error GSM_WaitFor (GSM_StateMachine *s, unsigned const char *buffer,
-		       int length, unsigned char type, int time,
+		       int length, unsigned char type, int timeout,
 		       GSM_Phone_RequestID request)
 {
 	GSM_Phone_Data		*Phone = &s->Phone.Data;
@@ -625,20 +705,15 @@ GSM_Error GSM_WaitFor (GSM_StateMachine *s, unsigned const char *buffer,
 
 	for (reply=0;reply<s->ReplyNum;reply++) {
 		if (reply!=0) {
-			if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL || s->di.dl == DL_TEXTERROR ||
-			    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE || s->di.dl == DL_TEXTERRORDATE)
-			{
-				smprintf(s, "[Retrying %i type 0x%02X]\n", reply, type);
-			}
+			smprintf_level(s, D_ERROR, "[Retrying %i type 0x%02X]\n", reply, type);
 		}
 		error = s->Protocol.Functions->WriteMessage(s, buffer, length, type);
 		if (error!=ERR_NONE) return error;
 
-		error = GSM_WaitForOnce(s, buffer, length, type, time);
+		error = GSM_WaitForOnce(s, buffer, length, type, timeout);
 		if (error != ERR_TIMEOUT) return error;
         }
 
-//	return Phone->DispatchError;
 	return ERR_TIMEOUT;
 }
 
@@ -663,7 +738,7 @@ static GSM_Error CheckReplyFunctions(GSM_StateMachine *s, GSM_Reply_Function *Re
 				} else execute=true;
 			}
 		} else {
-			if ((int)strlen(Reply[i].msgtype)<msg->Length) {
+			if (strlen(Reply[i].msgtype) < msg->Length) {
 				if (strncmp(Reply[i].msgtype,msg->Buffer,strlen(Reply[i].msgtype))==0) {
 					execute=true;
 				}
@@ -696,25 +771,10 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 	GSM_Phone_Data 		*Phone	= &s->Phone.Data;
 	bool			disp    = false;
 	GSM_Reply_Function	*Reply;
-	int			reply, i;
+	int			reply;
 
-	if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-	    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
-		smprintf(s, "RECEIVED frame ");
-		smprintf(s, "type 0x%02X/length 0x%02X/%i", msg->Type, msg->Length, msg->Length);
-		DumpMessage(&s->di, msg->Buffer, msg->Length);
-		if (msg->Length == 0) smprintf(s, "\n");
-		fflush(s->di.df);
-	}
-	if (s->di.dl==DL_BINARY) {
-		smprintf(s,"%c",0x02);	/* Receiving */
-		smprintf(s,"%c",msg->Type);
-		smprintf(s,"%c",msg->Length/256);
-		smprintf(s,"%c",msg->Length%256);
-		for (i=0;i<msg->Length;i++) {
-			smprintf(s,"%c",msg->Buffer[i]);
-		}
-	}
+	GSM_DumpMessageLevel2Recv(s, msg->Buffer, msg->Length, msg->Type);
+	GSM_DumpMessageLevel3Recv(s, msg->Buffer, msg->Length, msg->Type);
 
 	Reply=s->User.UserReplyFunctions;
 	if (Reply!=NULL) error=CheckReplyFunctions(s,Reply,&reply);
@@ -736,22 +796,19 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 	}
 
 	if (strcmp(s->Phone.Functions->models,"NAUTO")) {
-		if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL || s->di.dl==DL_TEXTERROR ||
-		    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE || s->di.dl==DL_TEXTERRORDATE) {
-			disp = true;
-			switch (error) {
-			case ERR_UNKNOWNRESPONSE:
-				smprintf(s, "\nUNKNOWN response");
-				break;
-			case ERR_UNKNOWNFRAME:
-				smprintf(s, "\nUNKNOWN frame");
-				break;
-			case ERR_FRAMENOTREQUESTED:
-				smprintf(s, "\nFrame not request now");
-				break;
-			default:
-				disp = false;
-			}
+		disp = true;
+		switch (error) {
+		case ERR_UNKNOWNRESPONSE:
+			smprintf_level(s, D_ERROR, "\nUNKNOWN response");
+			break;
+		case ERR_UNKNOWNFRAME:
+			smprintf_level(s, D_ERROR, "\nUNKNOWN frame");
+			break;
+		case ERR_FRAMENOTREQUESTED:
+			smprintf_level(s, D_ERROR, "\nFrame not request now");
+			break;
+		default:
+			disp = false;
 		}
 
 		if (error == ERR_UNKNOWNFRAME || error == ERR_FRAMENOTREQUESTED) {
@@ -763,11 +820,11 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 		smprintf(s,". If you can, please report it (see <http://cihar.com/gammu/report>). Thank you\n");
 		if (Phone->SentMsg != NULL) {
 			smprintf(s,"LAST SENT frame ");
-			smprintf(s, "type 0x%02X/length %i", Phone->SentMsg->Type, Phone->SentMsg->Length);
+			smprintf(s, "type 0x%02X/length " SIZE_T_FORMAT, Phone->SentMsg->Type, Phone->SentMsg->Length);
 			DumpMessage(&s->di, Phone->SentMsg->Buffer, Phone->SentMsg->Length);
 		}
 		smprintf(s, "RECEIVED frame ");
-		smprintf(s, "type 0x%02X/length 0x%02X/%i", msg->Type, msg->Length, msg->Length);
+		smprintf(s, "type 0x%02X/length 0x" SIZE_T_HEX_FORMAT "/" SIZE_T_FORMAT, msg->Type, msg->Length, msg->Length);
 		DumpMessage(&s->di, msg->Buffer, msg->Length);
 		smprintf(s, "\n");
 	}
@@ -775,57 +832,62 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 	return error;
 }
 
+GSM_Error GSM_TryReadGammuRC (const char *path, INI_Section **result)
+{
+	dbgprintf("Open config: \"%s\"\n", path);
+	return  INI_ReadFile(path, false, result);
+}
+
 GSM_Error GSM_FindGammuRC (INI_Section **result)
 {
-        char		*HomeDrive,*HomePath,*FileName=malloc(1);
-	int		FileNameUsed=1;
-	GSM_Error	error;
-	GSM_Error	error2;
+	char configfile[PATH_MAX + 1];
+	char *envpath;
+	GSM_Error error;
 
 	*result = NULL;
 
-	FileName[0] = 0;
-#if defined(WIN32) || defined(DJGPP)
-	FileName = (char *)realloc(FileName, MAX_PATH);
+#ifdef WIN32
+	/* Get Windows application data path */
+	if (SUCCEEDED(SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, SHGFP_TYPE_CURRENT, configfile))) {
+		strcat(configfile, GAMMURC_NAME);
 
-	if (!SUCCEEDED(SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, 0, FileName))) {
-		HomeDrive = getenv("HOMEDRIVE");
-		if (HomeDrive) {
-			FileName 	=  realloc(FileName,FileNameUsed+strlen(HomeDrive)+1);
-			FileName 	=  strcat(FileName, HomeDrive);
-			FileNameUsed	+= strlen(HomeDrive)+1;
-		}
-		HomePath = getenv("HOMEPATH");
-		if (HomePath) {
-			FileName 	=  realloc(FileName,FileNameUsed+strlen(HomePath)+1);
-			FileName 	=  strcat(FileName, HomePath);
-			FileNameUsed	+= strlen(HomePath)+1;
-		}
-		FileName = (char *)realloc(FileName,FileNameUsed+8+1);
+		error = GSM_TryReadGammuRC(configfile, result);
+		if (error == ERR_NONE) return ERR_NONE;
 	}
-        strcat(FileName, "\\gammurc");
-#else
-	HomeDrive = NULL;
-        HomePath  = getenv("HOME");
-        if (HomePath) {
-		FileName 	=  realloc(FileName,FileNameUsed+strlen(HomePath)+1);
-		FileName 	=  strcat(FileName, HomePath);
-		FileNameUsed	+= strlen(HomePath)+1;
-	}
-	FileName = realloc(FileName,FileNameUsed+9+1);
-        strcat(FileName, "/.gammurc");
 #endif
-	dbgprintf("Open config: \"%s\"\n", FileName);
 
-	error = INI_ReadFile(FileName, false, result);
-	free(FileName);
-        if (error != ERR_NONE) {
-		dbgprintf("Open fallback config: \"%s\"\n", FALLBACK_GAMMURC);
-		error2 = INI_ReadFile(FALLBACK_GAMMURC, false, result);
-		if (error2 == ERR_NONE) error = ERR_NONE;
-        }
+	/* Reset as we're using strcat */
+	configfile[0] = 0;
 
-	return error;
+	/* Try user home */
+	envpath  = getenv("HOME");
+	if (envpath) {
+		strcat(configfile, envpath);
+		strcat(configfile, GAMMURC_NAME);
+
+		error = GSM_TryReadGammuRC(configfile, result);
+		if (error == ERR_NONE) return ERR_NONE;
+	}
+
+#if defined(WIN32)
+	/* This makes sense only on Windows */
+	envpath = getenv("HOMEDRIVE");
+	if (envpath) {
+		strcat(configfile, envpath);
+	}
+
+	envpath = getenv("HOMEPATH");
+	if (envpath) {
+		strcat(configfile, envpath);
+		strcat(configfile, GAMMURC_NAME);
+
+		error = GSM_TryReadGammuRC(configfile, result);
+		if (error == ERR_NONE) return ERR_NONE;
+	}
+#endif
+
+	/* Try fallback config as last */
+	return GSM_TryReadGammuRC(FALLBACK_GAMMURC, result);
 }
 
 GSM_Config *GSM_GetConfig(GSM_StateMachine *s, int num)
@@ -877,8 +939,7 @@ void GSM_ExpandUserPath(char **string)
 	*string = tmp;
 }
 
-
-bool GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
+GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 {
 	INI_Section 	*h;
 	unsigned char 	section[50];
@@ -900,36 +961,21 @@ bool GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 
 	/* By default all debug output will go to one filedescriptor */
 	static const bool DefaultUseGlobalDebugFile 	= true;
+	GSM_Error error = ERR_UNKNOWN;
 
 	cfg->UseGlobalDebugFile	 = DefaultUseGlobalDebugFile;
 
 	/* If we don't have valid config, bail out */
 	if (cfg_info==NULL) {
-
-		/* Special case, this config needs to be somehow valid */
-		if (num == 0) {
-			cfg->Device		 	 = strdup(DefaultPort);
-			cfg->Connection	 		 = strdup(DefaultConnection);
-			cfg->SyncTime		 	 = strdup(DefaultSynchronizeTime);
-			cfg->DebugFile		 	 = strdup(DefaultDebugFile);
-			cfg->LockDevice	 		 = strdup(DefaultLockDevice);
-			strcpy(cfg->Model,DefaultModel);
-			strcpy(cfg->DebugLevel,DefaultDebugLevel);
-			cfg->StartInfo	 		 = strdup(DefaultStartInfo);
-			strcpy(cfg->TextReminder,"Reminder");
-			strcpy(cfg->TextMeeting,"Meeting");
-			strcpy(cfg->TextCall,"Call");
-			strcpy(cfg->TextBirthday,"Birthday");
-			strcpy(cfg->TextMemo,"Memo");
-		}
-		return false;
+		error = ERR_UNCONFIGURED;
+		goto fail;
 	}
 
 	/* Which section should we read? */
 	if (num == 0) {
-		sprintf(section,"gammu");
+		snprintf(section, sizeof(section) - 1, "gammu");
 	} else {
-		sprintf(section,"gammu%i",num);
+		snprintf(section, sizeof(section) - 1, "gammu%i", num);
 	}
 
 	/* Scan for section */
@@ -939,7 +985,10 @@ bool GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 			break;
 		}
         }
-	if (!found) return false;
+	if (!found) {
+		error = ERR_NONE_SECTION;
+		goto fail;
+	}
 
 	/* Set device name */
 	free(cfg->Device);
@@ -989,7 +1038,7 @@ bool GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 
 	/* Set model */
 	Temp		 = INI_GetValue(cfg_info, section, "model", 		false);
-	if (!Temp) {
+	if (!Temp || strcmp(Temp, "auto") == 0) {
 		strcpy(cfg->Model,DefaultModel);
 	} else {
 		strcpy(cfg->Model,Temp);
@@ -1049,37 +1098,85 @@ bool GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 		strcpy(cfg->TextMemo,Temp);
 	}
 
-	return true;
+	/* Phone features */
+	Temp		 = INI_GetValue(cfg_info, section, "features", 		false);
+	if (!Temp) {
+		cfg->PhoneFeatures[0] = 0;
+	} else {
+		error = GSM_SetFeatureString(cfg->PhoneFeatures, Temp);
+		if (error != ERR_NONE) goto fail;
+	}
+	return ERR_NONE;
+
+fail:
+	/* Special case, this config needs to be somehow valid */
+	if (num == 0) {
+		cfg->Device		 	 = strdup(DefaultPort);
+		cfg->Connection	 		 = strdup(DefaultConnection);
+		cfg->SyncTime		 	 = strdup(DefaultSynchronizeTime);
+		cfg->DebugFile		 	 = strdup(DefaultDebugFile);
+		cfg->LockDevice	 		 = strdup(DefaultLockDevice);
+		strcpy(cfg->Model,DefaultModel);
+		strcpy(cfg->DebugLevel,DefaultDebugLevel);
+		cfg->StartInfo	 		 = strdup(DefaultStartInfo);
+		strcpy(cfg->TextReminder,"Reminder");
+		strcpy(cfg->TextMeeting,"Meeting");
+		strcpy(cfg->TextCall,"Call");
+		strcpy(cfg->TextBirthday,"Birthday");
+		strcpy(cfg->TextMemo,"Memo");
+		cfg->PhoneFeatures[0] = 0;
+		/* Indicate that we used defaults */
+		return ERR_USING_DEFAULTS;
+	}
+	return error;
+}
+
+void GSM_DumpMessageLevel2_Text(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, const char *text)
+{
+	if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
+	    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
+		smprintf(s, "%s", text);
+		smprintf(s, "type 0x%02X/length 0x%02X/%i",
+				type, messagesize, messagesize);
+		DumpMessage(&s->di, message, messagesize);
+		if (messagesize == 0)
+			smprintf(s,"\n");
+	}
 }
 
 void GSM_DumpMessageLevel2(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
 {
-	if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-	    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
-		smprintf(s,"SENDING frame ");
-		smprintf(s,"type 0x%02X/length 0x%02X/%i", type, messagesize, messagesize);
-		DumpMessage(&s->di, message, messagesize);
-		if (messagesize == 0) smprintf(s,"\n");
-		if (s->di.df) fflush(s->di.df);
-	}
+	GSM_DumpMessageLevel2_Text(s, message, messagesize, type, "SENDING frame");
 }
 
-void GSM_DumpMessageLevel3(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+void GSM_DumpMessageLevel2Recv(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+{
+	GSM_DumpMessageLevel2_Text(s, message, messagesize, type, "RECEIVED frame");
+}
+
+void GSM_DumpMessageLevel3_Custom(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, int direction)
 {
 	int i;
 
 	if (s->di.dl==DL_BINARY) {
-		smprintf(s,"%c",0x01);	/* Sending */
+		smprintf(s,"%c", direction);
 		smprintf(s,"%c",type);
 		smprintf(s,"%c",messagesize/256);
 		smprintf(s,"%c",messagesize%256);
 		for (i=0;i<messagesize;i++) smprintf(s,"%c",message[i]);
 	}
 }
+void GSM_DumpMessageLevel3(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+{
+	GSM_DumpMessageLevel3_Custom(s, message, messagesize, type, 0x01);
+}
 
-#if defined(__GNUC__) && !defined(printf)
-__attribute__((format(printf, 2, 3)))
-#endif
+void GSM_DumpMessageLevel3Recv(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type)
+{
+	GSM_DumpMessageLevel3_Custom(s, message, messagesize, type, 0x02);
+}
+
+PRINTF_STYLE(2, 3)
 int smprintf(GSM_StateMachine *s, const char *format, ...)
 {
 	va_list		argp;
@@ -1089,7 +1186,42 @@ int smprintf(GSM_StateMachine *s, const char *format, ...)
 	va_start(argp, format);
 
 	if ((s != NULL && s->di.df != 0) || (s == NULL && di.df != 0)) {
-		result = vsprintf(buffer, format, argp);
+		result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
+		result = smfprintf((s == NULL) ? &di : &(s->di), "%s", buffer);
+	}
+
+	va_end(argp);
+	return result;
+}
+
+PRINTF_STYLE(3, 4)
+int smprintf_level(GSM_StateMachine * s, GSM_DebugSeverity severity, const char *format, ...)
+{
+	va_list		argp;
+	int 		result=0;
+	char		buffer[2000];
+
+	if (severity == D_TEXT) {
+		if (s->di.dl != DL_TEXT &&
+				s->di.dl != DL_TEXTALL &&
+				s->di.dl != DL_TEXTDATE &&
+				s->di.dl != DL_TEXTALLDATE) {
+			return 0;
+		}
+	} else if (severity == D_ERROR) {
+		if (s->di.dl != DL_TEXT &&
+				s->di.dl != DL_TEXTALL &&
+				s->di.dl != DL_TEXTDATE &&
+				s->di.dl != DL_TEXTALLDATE &&
+				s->di.dl != DL_TEXTERROR &&
+				s->di.dl != DL_TEXTERRORDATE) {
+			return 0;
+		}
+	}
+	va_start(argp, format);
+
+	if ((s != NULL && s->di.df != 0) || (s == NULL && di.df != 0)) {
+		result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
 		result = smfprintf((s == NULL) ? &di : &(s->di), "%s", buffer);
 	}
 
@@ -1113,7 +1245,7 @@ void GSM_OSErrorInfo(GSM_StateMachine *s, char *description)
 				FORMAT_MESSAGE_IGNORE_INSERTS,
 				NULL,
 				GetLastError(),
-				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
+				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language */
 				(LPTSTR) &lpMsgBuf,
 				0,
 				NULL
@@ -1150,7 +1282,6 @@ void GSM_GetPhoneFeaturesForBackup(GSM_StateMachine *s, GSM_Backup_Info *info)
 	GSM_MultiWAPSettings	WAPSettings;
  	GSM_FMStation		FMStation;
  	GSM_GPRSAccessPoint	GPRSPoint;
-//	GSM_Profile		Profile;
 
 	if (info->PhonePhonebook) {
 		MemStatus.MemoryType = MEM_ME;
