@@ -17,7 +17,7 @@
 /**
  * Read information from file in Windows INI format style
  */
-GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
+GSM_Error INI_ReadFile(const char *FileName, bool Unicode, INI_Section **result)
 {
 	FILE		*f;
 	bool		FFEEUnicode=false;
@@ -27,6 +27,7 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 	unsigned char	*buffer2 = NULL, *buffer1 = NULL, buff[1000];
         INI_Section 	*INI_info = NULL, *INI_head = NULL, *heading;
         INI_Entry 	*entry;
+	GSM_Error	error = ERR_NONE;
 
 	*result = NULL;
 
@@ -42,11 +43,8 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 				buffused = fread(buff,1,1000,f);
 				buffread = 0;
 				if (buffused == 0) {
-					free(buffer); free(buffer1); free(buffer2);
-					fclose(f);
-					*result = INI_head;
-					if (INI_head == NULL) return ERR_FILENOTSUPPORTED;
-					return ERR_NONE;
+					error = ERR_NONE;
+					goto done;
 				}
 			}
 			if (Unicode) {
@@ -83,7 +81,6 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 			buffer[bufferused+1] 	= ch[1];
 			bufferused		= bufferused + 2;
 		}
-//		printf("line \"%s\"\n",DecodeUnicodeConsole(buffer));
 
 		buffer1used = 0;
 		buffer2used = 0;
@@ -94,13 +91,13 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 		for (i=0;i<bufferused/2;i++) {
 			ch[0] = buffer[i*2];
 			ch[1] = buffer[i*2+1];
-			if (level == 0) { //search for name of section
+			if (level == 0) { /* search for name of section */
 				if (ch[0] == 0 && ch[1] == '[') level = 1;
 				if (ch[0] == 0 && ch[1] == ';') break;
 				if (ch[0] == 0 && ch[1] == '#') break;
 				continue;
 			}
-			if (level == 1) { //section name
+			if (level == 1) { /* section name */
 				if (ch[0] == 0 && ch[1] == ']') {
 					if (buffer1used == 0) break;
 					if (Unicode) {
@@ -115,9 +112,8 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 					}
 					heading = (INI_Section *)malloc(sizeof(*heading));
 		                        if (heading == NULL) {
-						free(buffer); free(buffer1); free(buffer2);
-						fclose(f);
-						return ERR_MOREMEMORY;
+						error = ERR_MOREMEMORY;
+						goto done;
 		                        }
 					heading->SectionName = (char *)malloc(buffer1used);
 					memcpy(heading->SectionName,buffer1,buffer1used);
@@ -131,7 +127,6 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 		                        INI_info 		= heading;
 					INI_info->SubEntries 	= NULL;
 					level 	 		= 2;
-//					printf("[%s]\n",DecodeUnicodeConsole(buffer1));
 					break;
 				}
 				if (Unicode) {
@@ -146,7 +141,7 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 				}
 				continue;
 			}
-			if (level == 2) { //search for key name
+			if (level == 2) { /* search for key name */
 				if (ch[0] == 0 && ch[1] == ';') break;
 				if (ch[0] == 0 && ch[1] == '#') break;
 				if (ch[0] == 0 && ch[1] == '[') {
@@ -160,7 +155,7 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 				}
 				level = 3;
 			}
-			if (level == 3) { //key name
+			if (level == 3) { /* key name */
 				if (ch[0] == 0 && ch[1] == '=') {
 					if (buffer1used == 0) break;
 					while(1) {
@@ -186,7 +181,7 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 					buffer1used		= buffer1used + 1;
 				}
 			}
-			if (level == 4) { //search for key value
+			if (level == 4) { /* search for key value */
 				if (Unicode) {
 			                if (myiswspace(ch)) continue;
 				} else {
@@ -194,7 +189,7 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 				}
 				level = 5;
 			}
-			if (level == 5) { //key value
+			if (level == 5) { /* key value */
 				if (Unicode) {
 					buffer2 		= realloc(buffer2,buffer2used+2);
 					buffer2[buffer2used] 	= ch[0];
@@ -212,9 +207,8 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 
 			entry = (INI_Entry *)malloc(sizeof(*entry));
                         if (entry == NULL) {
-				free(buffer); free(buffer1); free(buffer2);
-				fclose(f);
-                                return ERR_MOREMEMORY;
+                                error = ERR_MOREMEMORY;
+				goto done;
                         }
 			if (Unicode) {
 				buffer1 		= realloc(buffer1,buffer1used+2);
@@ -233,9 +227,6 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
 				buffer2[buffer2used] 	= 0x00;
 				buffer2used		= buffer2used + 1;
 			}
-//			printf("\"%s\"=\"%s\"\n",buffer1,buffer2);
-//			printf("\"%s\"=",DecodeUnicodeConsole(buffer1));
-//			printf("\"%s\"\n",DecodeUnicodeConsole(buffer2));
 
 			entry->EntryName = (char *)malloc(buffer1used);
 			memcpy(entry->EntryName,buffer1,buffer1used);
@@ -249,11 +240,16 @@ GSM_Error INI_ReadFile(char *FileName, bool Unicode, INI_Section **result)
                         INI_info->SubEntries = entry;
 		}
 	}
+done:
 	free(buffer); free(buffer1); free(buffer2);
 	fclose(f);
-	*result = INI_head;
-	if (INI_head == NULL) return ERR_FILENOTSUPPORTED;
-	return ERR_NONE;
+	if (error == ERR_NONE) {
+		*result = INI_head;
+		if (INI_head == NULL) {
+			error = ERR_FILENOTSUPPORTED;
+		}
+	}
+	return error;
 }
 
 /*
