@@ -400,57 +400,47 @@ unsigned char EncodeWithHexBinAlphabet (int digit)
 	return 0;
 }
 
-void DecodeHexUnicode (unsigned char *dest, const unsigned char *src, int len)
+void DecodeHexUnicode (unsigned char *dest, const char *src, size_t len)
 {
-	int 	i,current=0;
-	bool 	first = false;
+	size_t i, current = 0;
 
-	if (len != 0 && src[0] == '0' && src[1] == '0') {
-		first = true;
-	} else if (len != 0 && src[2] == '0' && src[3] == '0') {
-		first = false;
-	} else {
-		first = (10 * (src[0] - '0') + (src[1] - '0')) < (10 * (src[2] - '0')+ (src[3] - '0'));
-	}
-	for (i = 0; i < len/4 ; i++) {
-		if (first) {
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+0])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+1]);
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+2])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+3]);
-		} else {
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+2])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+3]);
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+0])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+1]);
-		}
+	for (i = 0; i < len ; i += 4) {
+		dest[current++] = 
+			(DecodeWithHexBinAlphabet(src[i + 0]) << 4) +
+			DecodeWithHexBinAlphabet(src[i + 1]);
+		dest[current++] = 
+			(DecodeWithHexBinAlphabet(src[i + 2]) << 4) +
+			DecodeWithHexBinAlphabet(src[i + 3]);
 	}
 	dest[current++] = 0;
 	dest[current++] = 0;
 }
 
-void EncodeHexUnicode (unsigned char *dest, const unsigned char *src, int len)
+void EncodeHexUnicode (char *dest, const unsigned char *src, size_t len)
 {
-	int i,current=0;
+	size_t i, current = 0;
 
 	for (i = 0; i < len; i++) {
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] >> 0x04);
+		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] >> 4);
 		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] & 0x0f);
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] >> 0x04);
+		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] >> 4);
 		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] & 0x0f);
 	}
 	dest[current++] = 0;
 }
 
-void DecodeHexBin (unsigned char *dest, const unsigned char *src, int len)
+bool DecodeHexBin (unsigned char *dest, const unsigned char *src, int len)
 {
-	int i,current=0;
+	int i,current=0, low, high;
 
 	for (i = 0; i < len/2 ; i++) {
-		dest[current++] = DecodeWithHexBinAlphabet(src[i*2])*16+
-				  DecodeWithHexBinAlphabet(src[i*2+1]);
+		low = DecodeWithHexBinAlphabet(src[i*2+1]);
+		high = DecodeWithHexBinAlphabet(src[i*2]);
+		if (low < 0 || high < 0) return false;
+		dest[current++] = (high << 4) | low;
 	}
 	dest[current++] = 0;
+	return true;
 }
 
 void EncodeHexBin (unsigned char *dest, const unsigned char *src, int len)
@@ -524,9 +514,9 @@ static unsigned char GSM_DefaultAlphabetCharsExtension[][4] =
 	{0x00,0x00,0x00,0x00}
 };
 
-void DecodeDefault (unsigned char *dest, const unsigned char *src, int len, bool UseExtensions, unsigned char *ExtraAlphabet)
+void DecodeDefault (unsigned char *dest, const unsigned char *src, size_t len, bool UseExtensions, unsigned char *ExtraAlphabet)
 {
-	int 	i,current=0,j;
+	size_t 	i,current=0,j;
 	bool	FoundSpecial = false;
 
 #ifdef DEBUG
@@ -588,9 +578,9 @@ static unsigned char ConvertTable[] =
 "\x00\x74\x01\x66\x00\x54\x01\x67\x00\x74\x00\xd9\x00\x55\x00\xda\x00\x55\x00\xfa\x00\x75\x00\xdb\x00\x55\x00\xfb\x00\x75\x01\x68\x00\x55\x01\x69\x00\x75\x01\x6a\x00\x55\x01\x6b\x00\x75\x01\x6c\x00\x55\x01\x6d\x00\x75\x01\x6e\x00\x55\x01\x6f\x00\x75\x01\x70\x00\x55\x01\x71\x00\x75\x01\x72\x00\x55\x01\x73\x00\x75\x01\xaf\x00\x55\x01\xb0\x00\x75\x01\xd3\x00\x55\x01\xd4\x00\x75\x01\xd5\x00\x55\x01\xd6\x00\x75\x01\xd7\x00\x55\x01\xd8\x00\x75\x01\xd9\x00\x55\x01\xda\x00\x75\x01\xdb\x00\x55\x01\xdc\x00\x75\x1e\xe4\x00\x55\x1e\xe5\x00\x75\x1e\xe6\x00\x55\x1e\xe7\x00\x75\x1e\xe8\x00\x55\x1e\xe9\x00\x75\x1e\xea\x00\x55\x1e\xeb\x00\x75\x1e\xec\x00\x55\x1e\xed\x00\x75\x1e\xee\x00\x55\x1e\xef\x00\x75\x1e\xf0\x00\x55\x1e\xf1\x00\x75\x01\x74\x00\x57\x01\x75\x00\x77\x1e\x80\x00\x57\x1e\x81\x00\x77\x1e\x82"\
 "\x00\x57\x1e\x83\x00\x77\x1e\x84\x00\x57\x1e\x85\x00\x77\x00\xdd\x00\x59\x00\xfd\x00\x79\x00\xff\x00\x79\x01\x76\x00\x59\x01\x77\x00\x79\x01\x78\x00\x59\x1e\xf2\x00\x59\x1e\xf3\x00\x75\x1e\xf4\x00\x59\x1e\xf5\x00\x79\x1e\xf6\x00\x59\x1e\xf7\x00\x79\x1e\xf8\x00\x59\x1e\xf9\x00\x79\x01\x79\x00\x5a\x01\x7a\x00\x7a\x01\x7b\x00\x5a\x01\x7c\x00\x7a\x01\x7d\x00\x5a\x01\x7e\x00\x7a\x01\xfc\x00\xc6\x01\xfd\x00\xe6\x01\xfe\x00\xd8\x01\xff\x00\xf8\x00\x00";
 
-void EncodeDefault(unsigned char *dest, const unsigned char *src, int *len, bool UseExtensions, unsigned char *ExtraAlphabet)
+void EncodeDefault(unsigned char *dest, const unsigned char *src, size_t *len, bool UseExtensions, unsigned char *ExtraAlphabet)
 {
-	int 	i,current=0,j,z;
+	size_t 	i,current=0,j,z;
 	char 	ret;
 	bool	FoundSpecial,FoundNormal;
 
@@ -670,9 +660,9 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, int *len, bool
 }
 
 /* You don't have to use ConvertTable here - 1 char is replaced there by 1 char */
-void FindDefaultAlphabetLen(const unsigned char *src, int *srclen, int *smslen, int maxlen)
+void FindDefaultAlphabetLen(const unsigned char *src, size_t *srclen, size_t *smslen, size_t maxlen)
 {
-	int 	current=0,j,i;
+	size_t 	current=0,j,i;
 	bool	FoundSpecial;
 
 	i = 0;
@@ -786,37 +776,56 @@ int GSM_PackSevenBitsToEight(int offset, unsigned char *input, unsigned char *ou
 
 void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, bool semioctet)
 {
-	unsigned char	Buffer[50]	= "";
+	unsigned char	Buffer[GSM_MAX_NUMBER_LENGTH + 1];
 	int		length		= Number[0];
 
+	/* Default ouput on error */
+	strcpy(Buffer, "<NOT DECODED>");
+
 	if (semioctet) {
 		/* Convert number of semioctets to number of chars */
 		if (length % 2) length++;
 		length=length / 2 + 1;
 	}
 
+	/* Check length */
+	if (length > GSM_MAX_NUMBER_LENGTH) {
+		dbgprintf("Number too big, not decoding! (Length=%d, MAX=%d)\n", length, GSM_MAX_NUMBER_LENGTH);
+		goto out;
+	}
+
 	/*without leading byte with format of number*/
 	length--;
 
-	switch ((Number[1] & 112)) {
-	case (NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN & 112):
+	switch ((Number[1] & 0x70)) {
+	case (NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN & 0x70):
 		if (length > 6) length++;
 		dbgprintf("Alphanumeric number, length %i\n",length);
 		GSM_UnpackEightBitsToSeven(0, length, length, Number+2, Buffer);
 		Buffer[length]=0;
 		break;
-	case (NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN & 112):
+	case (NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN & 0x70):
 		dbgprintf("International number\n");
 		Buffer[0]='+';
 		DecodeBCD(Buffer+1,Number+2, length);
 		break;
 	default:
-		dbgprintf("Default number %02x\n",Number[1]);
+		dbgprintf("Default number %02x (%d %d %d %d|%d %d %d %d)\n",Number[1],
+				Number[1] & 0x80 ? 1 : 0,
+				Number[1] & 0x40 ? 1 : 0,
+				Number[1] & 0x20 ? 1 : 0,
+				Number[1] & 0x10 ? 1 : 0,
+				Number[1] & 0x08 ? 1 : 0,
+				Number[1] & 0x04 ? 1 : 0,
+				Number[1] & 0x02 ? 1 : 0,
+				Number[1] & 0x01 ? 1 : 0
+				);
 		DecodeBCD (Buffer, Number+2, length);
 		break;
 	}
 
 	dbgprintf("Len %i\n",length);
+out:
 	EncodeUnicode(retval,Buffer,strlen(Buffer));
 }
 
@@ -837,21 +846,23 @@ void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, boo
  */
 int GSM_PackSemiOctetNumber(unsigned char *Number, unsigned char *Output, bool semioctet)
 {
-	unsigned char	format, buffer[50];
+	unsigned char	format, buffer[GSM_MAX_NUMBER_LENGTH + 1];
 	int		length, i;
 
-	length=UnicodeLength(Number);
-	memcpy(buffer,DecodeUnicodeString(Number),length+1);
+	length = UnicodeLength(Number);
+	memcpy(buffer, DecodeUnicodeString(Number), length + 1);
 
 	/* Checking for format number */
-	format = NUMBER_UNKNOWN_NUMBERING_PLAN_ISDN;
-	for (i=0;i<length;i++) {
-		/* first byte is '+'. Number can be international */
-		if (i==0 && buffer[i]=='+') {
-			format=NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
-		} else {
-			/*char is not number. It must be alphanumeric*/
-			if (!isdigit(buffer[i])) format=NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN;
+	if (buffer[0] == '+' || (buffer[0] == '0' && buffer[1] == '0')) {
+		format = NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
+	} else {
+		format = NUMBER_UNKNOWN_NUMBERING_PLAN_ISDN;
+	}
+	for (i = 0; i < length; i++) {
+		/* If there is something which can not be in normal
+		 * number, mark it as alphanumberic */
+		if (strchr("+0123456789*#pP", buffer[i]) == NULL) {
+			format = NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN;
 		}
 	}
 
@@ -939,22 +950,22 @@ void ReadUnicodeFile(unsigned char *Dest, const unsigned char *Source)
 	Dest[current++]	= 0;
 }
 
-INLINE int GetBit(unsigned char *Buffer, int BitNum)
+INLINE int GetBit(unsigned char *Buffer, size_t BitNum)
 {
 	return Buffer[BitNum/8] & 1<<(7-(BitNum%8));
 }
 
-INLINE int SetBit(unsigned char *Buffer, int BitNum)
+INLINE int SetBit(unsigned char *Buffer, size_t BitNum)
 {
 	return Buffer[BitNum/8] |= 1<<(7-(BitNum%8));
 }
 
-int ClearBit(unsigned char *Buffer, int BitNum)
+int ClearBit(unsigned char *Buffer, size_t BitNum)
 {
 	return Buffer[BitNum/8] &= 255 - (1 << (7-(BitNum%8)));
 }
 
-void BufferAlign(unsigned char *Destination, int *CurrentBit)
+void BufferAlign(unsigned char *Destination, size_t *CurrentBit)
 {
 	int i=0;
 
@@ -966,7 +977,7 @@ void BufferAlign(unsigned char *Destination, int *CurrentBit)
 	(*CurrentBit) = (*CurrentBit) + i;
 }
 
-void BufferAlignNumber(int *CurrentBit)
+void BufferAlignNumber(size_t *CurrentBit)
 {
 	int i=0;
 
@@ -978,11 +989,11 @@ void BufferAlignNumber(int *CurrentBit)
 }
 
 void AddBuffer(unsigned char 	*Destination,
-	       int 		*CurrentBit,
+	       size_t		*CurrentBit,
 	       unsigned char 	*Source,
-	       int 		BitsToProcess)
+	       size_t 		BitsToProcess)
 {
-	int i=0;
+	size_t i=0;
 
 	while (i!=BitsToProcess) {
 		if (GetBit(Source, i)) {
@@ -996,9 +1007,9 @@ void AddBuffer(unsigned char 	*Destination,
 }
 
 void AddBufferByte(unsigned char *Destination,
-		   int 		 *CurrentBit,
+		   size_t		 *CurrentBit,
 		   unsigned char Source,
-		   int 		 BitsToProcess)
+		   size_t		 BitsToProcess)
 {
 	unsigned char Byte;
 
@@ -1008,11 +1019,11 @@ void AddBufferByte(unsigned char *Destination,
 }
 
 void GetBuffer(unsigned char *Source,
-	       int 	     *CurrentBit,
+	       size_t	     *CurrentBit,
 	       unsigned char *Destination,
-	       int 	     BitsToProcess)
+	       size_t	     BitsToProcess)
 {
-	int i=0;
+	size_t i=0;
 
 	while (i!=BitsToProcess) {
 		if (GetBit(Source, (*CurrentBit)+i)) {
@@ -1026,11 +1037,11 @@ void GetBuffer(unsigned char *Source,
 }
 
 void GetBufferInt(unsigned char *Source,
-		  int 		*CurrentBit,
+		  size_t	*CurrentBit,
 		  int 		*integer,
-		  int 		BitsToProcess)
+		  size_t	BitsToProcess)
 {
-	int l=0,z=128,i=0;
+	size_t l=0,z=128,i=0;
 
 	while (i!=BitsToProcess) {
 		if (GetBit(Source, (*CurrentBit)+i)) l=l+z;
@@ -1042,11 +1053,11 @@ void GetBufferInt(unsigned char *Source,
 }
 
 void GetBufferI(unsigned char 	*Source,
-		int 		*CurrentBit,
+		size_t 		*CurrentBit,
 		int 		*result,
-		int 		BitsToProcess)
+		size_t 		BitsToProcess)
 {
-	int l=0,z,i=0;
+	size_t l=0,z,i=0;
 
 	z = 1<<(BitsToProcess-1);
 
@@ -1364,6 +1375,7 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 {
 	bool skip = false;
 	bool quoted_printable = false;
+	bool was_cr = false, was_lf = false;
 	size_t pos;
 	int tmp;
 
@@ -1376,6 +1388,15 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 			return;
 		case 0x0A:
 		case 0x0D:
+			if (skip) {
+				if (Buffer[*Pos] == 0x0d) {
+					if (was_cr && skip) return;
+					was_cr = true;
+				} else {
+					if (was_lf && skip) return;
+					was_lf = true;
+				}
+			}
 			if (pos != 0 && !skip) {
 				if (MergeLines) {
 					/* (Quote printable new line) Does string end with = ? */
@@ -1383,6 +1404,8 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 						pos--;
 						OutBuffer[pos] = 0;
 						skip = true;
+						was_cr = (Buffer[*Pos] == 0x0d);
+						was_lf = (Buffer[*Pos] == 0x0a);
 						break;
 					}
 					/* (vCard continuation) Next line start with space? */
@@ -1438,13 +1461,13 @@ int EncodeWithUTF8Alphabet(unsigned char mychar1, unsigned char mychar2, unsigne
 {
 	int src = mychar1*256+mychar2;
 
-	//Unicode 0080-07FF -> UTF8 110xxxxx 10xxxxxx
+	/* Unicode 0080-07FF -> UTF8 110xxxxx 10xxxxxx */
 	if (src >=128 && src <=2047) {
 		ret[0] = 192 + (src / 64);
 		ret[1] = 128 + (src % 64);
 		return 2;
 	}
-	//Unicode 0800-FFFF -> UTF8 1110xxxx 10xxxxxx 10xxxxxx
+	/* Unicode 0800-FFFF -> UTF8 1110xxxx 10xxxxxx 10xxxxxx */
 	if (src >2047) {
 		ret[0] = 224 + (src / 4096);
 		ret[1] = 128 + ((src / 64) % 64);
@@ -1452,7 +1475,7 @@ int EncodeWithUTF8Alphabet(unsigned char mychar1, unsigned char mychar2, unsigne
 		return 3;
 	}
 
-	//Unicode 0000-007F -> UTF8 0xxxxxxx
+	/* Unicode 0000-007F -> UTF8 0xxxxxxx */
 	ret[0] = mychar2;
 	return 1;
 }
@@ -1509,7 +1532,7 @@ bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 }
 
 /* Decode UTF8 char to Unicode char */
-int DecodeWithUTF8Alphabet(unsigned char *src, wchar_t *dest, int len)
+int DecodeWithUTF8Alphabet(const unsigned char *src, wchar_t *dest, int len)
 {
 	if (len < 1) return 0;
 	if (src[0] < 128) {
@@ -1577,7 +1600,7 @@ void DecodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src, in
 		}
 		if (z>0) {
 			i += z * 3;
-			// we ignore wrong sequence
+			/*  we ignore wrong sequence */
 			if (DecodeWithUTF8Alphabet(mychar,&ret,z)==0) continue;
 		} else {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
@@ -1595,7 +1618,7 @@ void DecodeUTF8(unsigned char *dest, const unsigned char *src, int len)
 	wchar_t		ret;
 
 	while (i < len) {
-		z = DecodeWithUTF8Alphabet((unsigned char *)src+i,&ret,len-i);
+		z = DecodeWithUTF8Alphabet(src+i,&ret,len-i);
 		if (z<2) {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
 		} else {
@@ -1741,7 +1764,7 @@ OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
-static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], int len)
+static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], size_t len)
 {
 	/* BASE64 translation Table as described in RFC1113 */
 	unsigned char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -1752,10 +1775,10 @@ static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], int len
 	out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
 }
 
-void EncodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
+void EncodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Length)
 {
 	unsigned char 	in[3], out[4];
-	int 		i, pos = 0, len, outpos = 0;
+	size_t 		i, pos = 0, len, outpos = 0;
 
 	while (pos < Length) {
 		len = 0;
@@ -1783,11 +1806,11 @@ static void DecodeBASE64Block(unsigned char in[4], unsigned char out[3])
 	out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
 }
 
-int DecodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
+int DecodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Length)
 {
 	unsigned char 	cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";
 	unsigned char 	in[4], out[3], v;
-	int 		i, len, pos = 0, outpos = 0;
+	size_t 		i, len, pos = 0, outpos = 0;
 
 	while (pos < Length) {
 		len = 0;
