@@ -4,6 +4,7 @@
 #include <gammu-config.h>
 #include <gammu-misc.h>
 #include "misc.h"
+#include "coding/coding.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -11,6 +12,7 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <locale.h>
+#include <stdlib.h>
 #include <sys/timeb.h>
 #ifdef WIN32
 #  define WIN32_LEAN_AND_MEAN
@@ -31,6 +33,17 @@ int RecalcDateTime(struct tm *st, const int year, const int month, const int day
 {
 	const int days[] = {31,28,31,30,31,30,31,31,30,31,30,31};
 	int i, p, q, r;
+	GSM_DateTime Date;
+
+	Date.Year	= year;
+	Date.Month	= month;
+	Date.Day	= day;
+	Date.Hour	= hour;
+	Date.Minute	= minute;
+	Date.Second	= second;
+	Date.Timezone	= 0;
+
+	if (!CheckDate(&Date) || !CheckTime(&Date)) return 0;
 
 	memset(st, 0, sizeof(*st));
 
@@ -212,18 +225,23 @@ char *OSDateTime (GSM_DateTime dt, bool TimeZone)
 	struct tm 	timeptr;
 	static char 	retval[200],retval2[200];
 
+	if (!RecalcDateTime(&timeptr, dt.Year, dt.Month, dt.Day,
+				dt.Hour, dt.Minute, dt.Second)) {
+		retval2[0] = 0;
+		return retval2;
+	}
+
 #ifdef WIN32
 	setlocale(LC_ALL, ".OCP");
 #endif
 
-	RecalcDateTime(&timeptr, dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
-
+	/* This is not Y2K safe */
 	strftime(retval2, 200, "%c", &timeptr);
 	if (TimeZone) {
 		if (dt.Timezone >= 0) {
-			sprintf(retval," +%02i00",dt.Timezone);
+			snprintf(retval, sizeof(retval) - 1, " +%02i00",dt.Timezone);
 		} else {
-			sprintf(retval," -%02i00",dt.Timezone);
+			snprintf(retval, sizeof(retval) - 1, " -%02i00",dt.Timezone);
 		}
 		strcat(retval2,retval);
 	}
@@ -268,6 +286,7 @@ char *OSDate (GSM_DateTime dt)
 	timeptr.tm_zone		= NULL;
 #endif
 
+	/* This is not Y2K safe */
 	strftime(retval2, 200, "%x", &timeptr);
 
 	/* If don't have weekday name, include it */
@@ -325,16 +344,39 @@ int GetLine(FILE *File, char *Line, int count)
 	return -1;
 }
 
-void SplitLines(unsigned char *message, int messagesize, GSM_Lines *lines, unsigned char *whitespaces, int spaceslen, bool eot)
+void InitLines(GSM_CutLines *lines)
+{
+	lines->numbers = NULL;
+	lines->allocated = 0;
+}
+
+void FreeLines(GSM_CutLines *lines)
+{
+	free(lines->numbers);
+	lines->numbers = NULL;
+	lines->allocated = 0;
+}
+
+void SplitLines(const char *message, int messagesize, GSM_CutLines *lines, unsigned char *whitespaces, int spaceslen, bool eot)
 {
 	int 	i,number=0,j;
 	bool 	whitespace=true, nowwhite;
 
-	for (i=0;i<MAX_LINES*2;i++) lines->numbers[i]=0;
+	for (i = 0; i < lines->allocated; i++)
+		lines->numbers[i]=0;
+
+	for (i = 0; i < messagesize; i++) {
+		/* Reallocate buffer if needed */
+		if (number + 1 >= lines->allocated - 1) {
+			lines->allocated += 20;
+			lines->numbers = realloc(lines->numbers, lines->allocated * sizeof(int));
+			if (lines->numbers == NULL) return;
+			for (j = lines->allocated - 20; j < lines->allocated; j++)
+				lines->numbers[j] = 0;
+		}
 
-	for (i=0;i<messagesize;i++) {
 		nowwhite = false;
-		for (j=0;j<spaceslen;j++) {
+		for (j = 0; j < spaceslen; j++) {
 			if (whitespaces[j] == message[i]) {
 				nowwhite = true;
 				break;
@@ -355,28 +397,39 @@ void SplitLines(unsigned char *message, int messagesize, GSM_Lines *lines, unsig
 
 		}
 	}
-    	if (eot && !whitespace) lines->numbers[number]=messagesize;
+    	if (eot && !whitespace)
+		lines->numbers[number] = messagesize;
 }
 
-char *GetLineString(unsigned char *message, GSM_Lines lines, int start)
+char *GetLineString(const char *message, const GSM_CutLines *lines, int start)
 {
-	static char retval[800];
+	static char *retval = NULL;
+	int len;
 
-	memcpy(retval,message + lines.numbers[start*2-2],lines.numbers[start*2-2+1]-lines.numbers[start*2-2]);
-	retval[lines.numbers[start*2-2+1]-lines.numbers[start*2-2]]=0;
+	len = GetLineLength(message, lines, start);
+	retval = realloc(retval, len + 1);
+	if (retval == NULL) {
+		dbgprintf("Allocation failed!\n");
+		return NULL;
+	}
+
+	memcpy(retval, message + lines->numbers[start * 2 - 2], len);
+
+	retval[len] = 0;
 
 	return retval;
 }
 
-int GetLineLength(unsigned char *message UNUSED, GSM_Lines lines, int start)
+int GetLineLength(const char *message UNUSED, const GSM_CutLines *lines, int start)
 {
-	return lines.numbers[start*2-2+1]-lines.numbers[start*2-2];
+	return lines->numbers[start*2-2+1] - lines->numbers[start*2-2];
 }
 
-void CopyLineString(unsigned char *dest, unsigned char *src, GSM_Lines lines, int start)
+void CopyLineString(char *dest, const char *src, const GSM_CutLines *lines, int start)
 {
-	memcpy(dest,GetLineString(src, lines, start),strlen(GetLineString(src, lines, start)));
-	dest[strlen(GetLineString(src, lines, start))] = 0;
+	int len = GetLineLength(src, lines, start);
+	memcpy(dest, GetLineString(src, lines, start), len);
+	dest[len] = 0;
 }
 
 GSM_Debug_Info di = {0,NULL,false,"",false};
@@ -392,7 +445,7 @@ int dbgprintf(const char *format, ...)
 	GSM_DateTime 		date_time;
 
 	va_start(argp, format);
-	result = vsprintf(buffer, format, argp);
+	result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
 	va_end(argp);
 
 	if (di.df != NULL && (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE)) {
@@ -440,7 +493,7 @@ int smfprintf(GSM_Debug_Info *d, const char *format, ...)
 	if (l == DL_NONE || f == NULL) return 0;
 
 	va_start(argp, format);
-	result = vsprintf(buffer, format, argp);
+	result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
 	va_end(argp);
 	pos = buffer;
 
@@ -536,13 +589,13 @@ void DumpMessage(GSM_Debug_Info *d, const unsigned char *message, int messagesiz
 	buffer[len*5-1]=0;
 
 	for (i = 0; i < messagesize; i++) {
-		sprintf(buffer+j*4,"%02X",message[i]);
+		snprintf(buffer+j*4, sizeof(buffer) - j*4 - 1, "%02X",message[i]);
 		buffer[j*4+2] = 0x20;
-		// 9 = tab
+		/* 9 = tab */
 		if (isprint(message[i]) && message[i]!=0x09
-			// 0x01 = beep in windows xp
+			/* 0x01 = beep in windows xp */
 			&& message[i]!=0x01
-			// these are empty in windows xp
+			/* these are empty in windows xp */
 			&& message[i]!=0x85 && message[i]!=0x95
   			&& message[i]!=0xA6 && message[i]!=0xB7) {
 			if (j != len-1) buffer[j*4+2] 	= message[i];
@@ -568,9 +621,10 @@ const char *GetOS(void)
 #ifdef WIN32
 	OSVERSIONINFOEX Ver;
 	bool		Extended = true;
-#endif
-#ifdef HAVE_SYS_UTSNAME_H
+#else
+#  ifdef HAVE_SYS_UTSNAME_H
 	struct utsname	Ver;
+#  endif
 #endif
 	static char 	Buffer[100] = {0x00};
 
@@ -582,14 +636,8 @@ const char *GetOS(void)
 		Extended 		= false;
 	      	Ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
 	        if (!GetVersionEx((OSVERSIONINFO *)&Ver)) {
-//#ifdef _MSC_VER
-//			Ver.dwMajorVersion = _winmajor;
-//			Ver.dwMinorVersion = _winminor;
-//			Ver.dwBuildNumber  = _osver;
-//#else
-			sprintf(Buffer, "Windows");
+			snprintf(Buffer, sizeof(Buffer) - 1, "Windows");
 			return Buffer;
-//#endif
 		}
 	}
 
@@ -597,90 +645,90 @@ const char *GetOS(void)
 
 	/* no info about Win95 SP1, Win95 OSR2.1, Win95 OSR2.5.... */
 	if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 950) {
-		sprintf(Buffer,"Windows 95");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 95");
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 1111) {
-		sprintf(Buffer,"Windows 95 OSR2.x");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 95 OSR2.x");
 
 	/* no info about Win98 SP1.... */
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 10 && Ver.dwBuildNumber == 1998) {
-		sprintf(Buffer,"Windows 98");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 98");
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 10 && Ver.dwBuildNumber == 2222) {
-		sprintf(Buffer,"Windows 98 SE");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 98 SE");
 
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 90 && Ver.dwBuildNumber == 3000) {
-		sprintf(Buffer,"Windows ME");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows ME");
 
 	/* ---------------- NT family ------------------- */
 
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 1381) {
-		sprintf(Buffer,"Windows NT 4.0");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows NT 4.0");
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 2195) {
-		sprintf(Buffer,"Windows 2000");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 2000");
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 1 && Ver.dwBuildNumber == 2600) {
-		sprintf(Buffer,"Windows XP");
-#if _MSC_VER > 1200 //6.0 has it undeclared
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows XP");
+#if _MSC_VER > 1200 /* 6.0 has it undeclared */
 		if (Extended) {
 			if (Ver.wSuiteMask & VER_SUITE_PERSONAL) {
-				sprintf(Buffer+strlen(Buffer)," Home");
+				snprintf(Buffer+strlen(Buffer), sizeof(Buffer) - 1 - strlen(Buffer)," Home");
 			} else {
-				sprintf(Buffer+strlen(Buffer)," Pro");
+				snprintf(Buffer+strlen(Buffer), sizeof(Buffer) - 1 - strlen(Buffer)," Pro");
 			}
 		}
 #endif
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 2) {
-		sprintf(Buffer,"Windows 2003");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 2003");
 
 	} else if (Ver.dwMajorVersion == 6 && Ver.dwMinorVersion == 0) {
-		sprintf(Buffer,"Windows Vista");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows Vista");
 
 	} else if (Ver.dwMajorVersion == 6 && Ver.dwMinorVersion > 0) {
-		sprintf(Buffer,"Windows Server 2007");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows Server 2007");
 
 	} else {
-		sprintf(Buffer, "Windows %i.%i.%i",(int)Ver.dwMajorVersion,(int)Ver.dwMinorVersion,(int)Ver.dwBuildNumber);
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows %i.%i.%i",(int)Ver.dwMajorVersion,(int)Ver.dwMinorVersion,(int)Ver.dwBuildNumber);
 	}
 
 	if (Extended && Ver.wServicePackMajor != 0) {
-		sprintf(Buffer+strlen(Buffer)," SP%i",Ver.wServicePackMajor);
+		snprintf(Buffer+strlen(Buffer), sizeof(Buffer) - 1 - strlen(Buffer)," SP%i",Ver.wServicePackMajor);
 	}
 #elif defined(HAVE_SYS_UTSNAME_H)
 	uname(&Ver);
-	sprintf(Buffer, "%s, kernel %s (%s)", Ver.sysname, Ver.release, Ver.version);
+	snprintf(Buffer, sizeof(Buffer) - 1, "%s, kernel %s (%s)", Ver.sysname, Ver.release, Ver.version);
 #elif defined(__FreeBSD__)
-	sprintf(Buffer, "FreeBSD");
+	snprintf(Buffer, sizeof(Buffer) - 1, "FreeBSD");
 #elif defined(__NetBSD__)
-	sprintf(Buffer, "NetBSD");
+	snprintf(Buffer, sizeof(Buffer) - 1, "NetBSD");
 #elif defined(__OpenBSD__)
-	sprintf(Buffer, "OpenBSD");
+	snprintf(Buffer, sizeof(Buffer) - 1, "OpenBSD");
 #elif defined(__GNU__)
-	sprintf(Buffer, "GNU/Hurd");
+	snprintf(Buffer, sizeof(Buffer) - 1, "GNU/Hurd");
 #elif defined(sun) || defined(__sun) || defined(__sun__)
 #  ifdef __SVR4
-	sprintf(Buffer, "Sun Solaris");
+	snprintf(Buffer, sizeof(Buffer) - 1, "Sun Solaris");
 #  else
-	sprintf(Buffer, "SunOS");
+	snprintf(Buffer, sizeof(Buffer) - 1, "SunOS");
 #  endif
 #elif defined(hpux) || defined(__hpux) || defined(__hpux__)
-	sprintf(Buffer, "HP-UX");
+	snprintf(Buffer, sizeof(Buffer) - 1, "HP-UX");
 #elif defined(ultrix) || defined(__ultrix) || defined(__ultrix__)
-	sprintf(Buffer, "DEC Ultrix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "DEC Ultrix");
 #elif defined(sgi) || defined(__sgi)
-	sprintf(Buffer, "SGI Irix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "SGI Irix");
 #elif defined(__osf__)
-	sprintf(Buffer, "OSF Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "OSF Unix");
 #elif defined(bsdi) || defined(__bsdi__)
-	sprintf(Buffer, "BSDI Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "BSDI Unix");
 #elif defined(_AIX)
-	sprintf(Buffer, "AIX Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "AIX Unix");
 #elif defined(_UNIXWARE)
-	sprintf(Buffer, "SCO Unixware");
+	snprintf(Buffer, sizeof(Buffer) - 1, "SCO Unixware");
 #elif defined(DGUX)
-	sprintf(Buffer, "DG Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "DG Unix");
 #elif defined(__QNX__)
-	sprintf(Buffer, "QNX");
+	snprintf(Buffer, sizeof(Buffer) - 1, "QNX");
 #endif
 	return Buffer;
 }
@@ -691,35 +739,55 @@ const char *GetCompiler(void)
 
 #ifdef WIN32
 #  ifdef _MSC_VER
-	if (_MSC_VER == 1200) { //?
-		sprintf(Buffer, "MS VC 6.0");
+	if (_MSC_VER == 1200) { /* ? */
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC 6.0");
 	} else if (_MSC_VER == 1300) {
-		sprintf(Buffer, "MS VC .NET 2002");
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC .NET 2002");
 	} else if (_MSC_VER == 1310) {
-		sprintf(Buffer, "MS VC .NET 2003");
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC .NET 2003");
 	} else if (_MSC_VER == 1400) {
-		sprintf(Buffer, "MS VC .NET 2005");
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC .NET 2005");
 	} else {
-		sprintf(Buffer, "MS VC %i",_MSC_VER);
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC %i",_MSC_VER);
 	}
 #  elif defined(__BORLANDC__)
-	sprintf(Buffer, "Borland C++ %i",__BORLANDC__);
+	snprintf(Buffer, sizeof(Buffer) - 1, "Borland C++ %i",__BORLANDC__);
+#  elif defined(__MINGW32__)
+	snprintf(Buffer, sizeof(Buffer) - 1, "GCC %i.%i, MinGW %i.%i", __GNUC__, __GNUC_MINOR__, __MINGW32_MAJOR_VERSION, __MINGW32_MINOR_VERSION);
+#  elif defined(__GNUC__)
+	snprintf(Buffer, sizeof(Buffer) - 1, "GCC %i.%i", __GNUC__, __GNUC_MINOR__);
+#  else
+	snprintf(Buffer, sizeof(Buffer) - 1, "unknown compiler");
 #  endif
 #elif defined(DJGPP)
-	sprintf(Buffer, "djgpp");
+	snprintf(Buffer, sizeof(Buffer) - 1, "djgpp %d.%d", __DJGPP, __DJGPP_MINOR);
 #elif defined(__GNUC__)
-	sprintf(Buffer, "gcc %i.%i", __GNUC__, __GNUC_MINOR__);
+	snprintf(Buffer, sizeof(Buffer) - 1, "GCC %i.%i", __GNUC__, __GNUC_MINOR__);
 #elif defined(__SUNPRO_CC)
-	sprintf(Buffer, "Sun C++ %x", __SUNPRO_CC);
+	snprintf(Buffer, sizeof(Buffer) - 1, "Sun C++ %x", __SUNPRO_CC);
 #elif defined(__INTEL_COMPILER)
-	sprintf(Buffer, "Intel Compiler %ld", __INTEL_COMPILER);
+	snprintf(Buffer, sizeof(Buffer) - 1, "Intel Compiler %ld", __INTEL_COMPILER);
 #else
-	sprintf(Buffer, "unknown compiler");
+	snprintf(Buffer, sizeof(Buffer) - 1, "unknown compiler");
 #endif
 
 	return Buffer;
 }
 
+bool GSM_IsNewerVersion(const char *latest_version, const char *current_version)
+{
+	size_t i;
+	size_t len = strlen(latest_version);
+
+	for (i = 0; i < len ; i++) {
+		if (latest_version[i] > current_version[i]) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
