@@ -27,7 +27,7 @@
 #include <gammu-error.h>
 #include <gammu-statemachine.h>
 
-#include "../../misc/misc.h" /* For GSM_Lines */
+#include "../../misc/misc.h" /* For GSM_CutLines */
 
 #include "motorola.h"
 
@@ -73,6 +73,9 @@ typedef enum {
 	AT_Samsung,
 	AT_Mitsubishi,
 	AT_Sharp,
+	AT_Option,
+	AT_Motorola,
+	AT_Wavecom,
 	AT_Unknown
 } GSM_AT_Manufacturer;
 
@@ -92,10 +95,18 @@ typedef enum {
 	 * Hex encoded UCS2.
 	 */
 	AT_CHARSET_UCS2,
+	/**
+	 * Hex encoded UCS2.
+	 */
+	AT_CHARSET_UCS_2,
 	/**
 	 * UTF-8.
 	 */
 	AT_CHARSET_UTF8,
+	/**
+	 * UTF-8.
+	 */
+	AT_CHARSET_UTF_8,
 	/**
 	 * CP-437, not fully supported.
 	 */
@@ -104,6 +115,10 @@ typedef enum {
 	 * IRA, what is same as 7-bit ASCII.
 	 */
 	AT_CHARSET_IRA,
+	/**
+	 * 7-bit ASCII.
+	 */
+	AT_CHARSET_ASCII,
 	/**
 	 * ISO-8859-1, supported only with iconv.
 	 */
@@ -183,7 +198,7 @@ typedef struct {
 	/**
 	 * Allow to simply get each line in response
 	 */
-	GSM_Lines		Lines;
+	GSM_CutLines		Lines;
 	/**
 	 * What response type - error, OK, etc.
 	 */
@@ -234,8 +249,8 @@ typedef struct {
 	 */
 	GSM_AT_Charset		IRACharset;
 	GSM_AT_SBNR		PBKSBNR;
-	int			NumberLength;
-	int			TextLength;
+	size_t			NumberLength;
+	size_t			TextLength;
 	int			MemorySize;
 
 	GSM_SMSMemoryStatus	LastSMSStatus;
@@ -254,6 +269,10 @@ typedef struct {
 	 * Is phone SMS memory available ?
 	 */
 	GSM_AT_SMSMemory	PhoneSMSMemory;
+	/**
+	 * Whether to use Motorola like SMS folders.
+	 */
+	bool			MotorolaSMS;
 	/**
 	 * Is SIM SMS memory available ?
 	 */
@@ -266,6 +285,11 @@ typedef struct {
 	 * PDU or TEXT mode for SMS ?
 	 */
 	GSM_AT_SMS_Modes	SMSMode;
+	/**
+	 * Does phone provide enhanced text mode readout of SMS flags
+	 * in text mode?
+	 */
+	bool			SMSTextDetails;
 
 	/**
 	 * Mode used for incoming message notification.
@@ -295,6 +319,18 @@ typedef struct {
 	 */
 	int			CurrentMode;
 	GSM_File		file;
+	/**
+	 * Number of entries in SMSLocations.
+	 */
+	int			SMSCount;
+	/**
+	 * Locations of non empty SMSes.
+	 */
+	int			*SMSLocations;
+	/**
+	 * Which folder do we read SMS from.
+	 */
+	int			SMSReadFolder;
 } GSM_Phone_ATGENData;
 
 /**
@@ -332,21 +368,26 @@ GSM_Error ATGEN_DispatchMessage	(GSM_StateMachine *s);
  *
  * \par Parser tokens:
  * - \b \@i - Number, expects pointer to int.
+ * - \b \@I - Optional number (string can be empty), expects pointer to int.
  * - \b \@l - Number, expects pointer to long int.
  * - \b \@s - String, will be converted from phone encoding, stripping
  *   quotes, expects pointer to unsigned char and size of storage.
- * - \b \@S - String with Samsung specials (0x02 at beginning and 0x03 
+ *   Encoding is somehow guessed.
+ * - \b \@e - String, will be converted from phone encoding, stripping
+ *   quotes, expects pointer to unsigned char and size of storage.
+ *   No encoding guessing.
+ * - \b \@S - String with Samsung specials (0x02 at beginning and 0x03
  *   at the end, encoded in utf-8), otherwise same as \@s.
- * - \b \@p - String with phone number hint for heuristics, otherwise 
+ * - \b \@p - String with phone number hint for heuristics, otherwise
  *   same as \@s.
- * - \b \@r - Raw string, no conversion will be done, only stripped 
+ * - \b \@r - Raw string, no conversion will be done, only stripped
  *   quotes, expects pointer to char and size of storage.
  * - \b \@d - Date, expects pointer to GSM_DateTime.
  * - \b \@\@ - \@ literal.
  * - \b \@0 - Ignore rest of input, same as .* regular expression.
  *
  * \par Special behaviour:
- * Any space is automatically treated as [[:space:]]* regular 
+ * Any space is automatically treated as [[:space:]]* regular
  * expression. So use space whenever some weird implementation in phone
  * can insert it space.
  *
