@@ -251,7 +251,7 @@ GSM_Error SAMSUNG_ReplyGetBitmap(GSM_Protocol_Message msg, GSM_StateMachine *s)
 
 		return ERR_NONE;
 	case AT_Reply_Error:
-		return ERR_UNKNOWN;
+		return ERR_NOTSUPPORTED;
 	case AT_Reply_CMSError:
 	        return ATGEN_HandleCMSError(s);
 	case AT_Reply_CMEError:
@@ -433,7 +433,7 @@ GSM_Error SAMSUNG_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, int *
 	if ((dot = strrchr(name, '.')) != NULL) *dot = 0;
 
 	crc = GetCRC(Ringtone->BinaryTone.Buffer, Ringtone->BinaryTone.Length);
-	sprintf(req, "AT+MELW=0,\"%s\",4,%d,%u\r", name,
+	sprintf(req, "AT+MELW=0,\"%s\",4," SIZE_T_FORMAT ",%u\r", name,
 		Ringtone->BinaryTone.Length, (unsigned int)crc);
 
 	error = s->Protocol.Functions->WriteMessage(s, req, strlen(req), 0x00);
