@@ -19,7 +19,7 @@
 /* method 3 */
 GSM_Error N6510_ReplyGetCalendarInfo3(GSM_Protocol_Message msg, GSM_StateMachine *s, GSM_NOKIACalToDoLocations *Last)
 {
-	int i=0,j=0;
+	size_t i=0,j=0;
 
 	while (Last->Location[j] != 0x00) j++;
 	if (j >= GSM_MAXCALENDARTODONOTES) {
@@ -36,7 +36,7 @@ GSM_Error N6510_ReplyGetCalendarInfo3(GSM_Protocol_Message msg, GSM_StateMachine
 		smprintf(s, "%i ",Last->Location[j-1]);
 		i++;
 	}
-	smprintf(s, "\nNumber of Entries in frame: %i\n",i);
+	smprintf(s, "\nNumber of Entries in frame: " SIZE_T_FORMAT "\n",i);
 	Last->Location[j] = 0;
 	smprintf(s, "\n");
 	if (i == 1 && msg.Buffer[12+0*4]*256+msg.Buffer[13+0*4] == 0) return ERR_EMPTY;
@@ -374,7 +374,7 @@ static GSM_Error N6510_FindCalendarIconID3(GSM_StateMachine *s, GSM_CalendarEntr
                  * Please note, that method 1 is the oldest one and in some
                  * moment Nokia can remove it from firmware
                  */
-//		if (Entry->Type == GSM_CAL_MEETING) {
+/* 		if (Entry->Type == GSM_CAL_MEETING) { */
 		if (Entry->Type == GSM_CAL_MEETING || Entry->Type == GSM_CAL_CALL || Entry->Type == GSM_CAL_MEMO) {
 			error=N71_65_AddCalendar1(s, Entry, &s->Phone.Data.Priv.N6510.FirstCalendarPos);
 		} else {
@@ -421,7 +421,7 @@ static GSM_Error N6510_FindCalendarIconID3(GSM_StateMachine *s, GSM_CalendarEntr
 }
 
 /* method 3 */
-GSM_Error N6510_ReplyAddCalendar3(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyAddCalendar3(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Calendar note added\n");
 	return ERR_NONE;
@@ -593,7 +593,7 @@ GSM_Error N6510_GetNextCalendar(GSM_StateMachine *s,  GSM_CalendarEntry *Note, b
 		return N71_65_GetNextCalendar1(s,Note,start,&s->Phone.Data.Priv.N6510.LastCalendar,&s->Phone.Data.Priv.N6510.LastCalendarYear,&s->Phone.Data.Priv.N6510.LastCalendarPos);
 
 	    	/* Method 2. In known phones texts of notes cut to 50 chars. Some features missed */
-//		return N71_65_GetNextCalendar2(s,Note,start,&s->Phone.Data.Priv.N6510.LastCalendarYear,&s->Phone.Data.Priv.N6510.LastCalendarPos);
+/* 		return N71_65_GetNextCalendar2(s,Note,start,&s->Phone.Data.Priv.N6510.LastCalendarYear,&s->Phone.Data.Priv.N6510.LastCalendarPos); */
 	} else {
 	    	/* Method 3. All DCT4 features supported. Not supported by 8910 */
 	    	return N6510_GetNextCalendar3(s,Note,start,&s->Phone.Data.Priv.N6510.LastCalendar,&s->Phone.Data.Priv.N6510.LastCalendarYear,&s->Phone.Data.Priv.N6510.LastCalendarPos);
@@ -625,7 +625,7 @@ GSM_Error N6510_GetCalendarStatus(GSM_StateMachine *s, GSM_CalendarStatus *Statu
 		return ERR_NONE;
 
 	    	/* Method 2 */
-//		return ERR_NOTSUPPORTED;
+/* 		return ERR_NOTSUPPORTED; */
 	} else {
 		/* Method 3 */
 		error=N6510_GetCalendarInfo3(s,&s->Phone.Data.Priv.N6510.LastCalendar,0);
@@ -643,7 +643,7 @@ GSM_Error N6510_AddCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
 	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62)) {
 		return N71_65_AddCalendar2(s,Note);
-//		return N71_65_AddCalendar1(s, Note, &s->Phone.Data.Priv.N6510.FirstCalendarPos);
+/* 		return N71_65_AddCalendar1(s, Note, &s->Phone.Data.Priv.N6510.FirstCalendarPos); */
 	} else {
 		/* Method 3. All DCT4 features supported. Not supported by 8910 */
 		return N6510_AddCalendar3(s, Note, &s->Phone.Data.Priv.N6510.FirstCalendarPos);
@@ -712,7 +712,7 @@ GSM_Error N6510_ReplyGetNoteFirstLoc(GSM_Protocol_Message msg, GSM_StateMachine
 	return ERR_NONE;
 }
 
-GSM_Error N6510_ReplyAddNote(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyAddNote(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "ToDo added\n");
 	return ERR_NONE;
@@ -1027,7 +1027,7 @@ GSM_Error N6510_GetNextToDo(GSM_StateMachine *s, GSM_ToDoEntry *ToDo, bool refre
 }
 
 /* ToDo support - 6310 style */
-GSM_Error N6510_ReplyDeleteAllToDo1(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyDeleteAllToDo1(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "All TODO deleted\n");
 	return ERR_NONE;
@@ -1076,7 +1076,7 @@ GSM_Error N6510_ReplyGetToDoFirstLoc1(GSM_Protocol_Message msg, GSM_StateMachine
 }
 
 /* ToDo support - 6310 style */
-GSM_Error N6510_ReplyAddToDo1(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyAddToDo1(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "TODO set OK\n");
 	return ERR_NONE;
@@ -1121,7 +1121,7 @@ static GSM_Error N6510_AddToDo1(GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
 	return GSM_WaitFor (s, reqSet, 12+ulen*2, 0x55, 4, ID_SetToDo);
 }
 
-GSM_Error N6510_ReplyAddToDo2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyAddToDo2(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "ToDo added method 2\n");
 	return ERR_NONE;
@@ -1271,6 +1271,9 @@ GSM_Error N6510_ReplyGetCalendarSettings(GSM_Protocol_Message msg, GSM_StateMach
 	case 0x8E:
 		smprintf(s, "Start day for calendar received\n");
 		switch(msg.Buffer[4]) {
+		case 0x04:
+			sett->StartDay = 1;
+			return ERR_NONE;
 		case 0x03:
 			sett->StartDay = 6;
 			return ERR_NONE;
