@@ -17,7 +17,7 @@
 #include "../dct4func.h"
 #include "n6510.h"
 
-//shared
+/* shared */
 
 static int N6510_FindFileCheckSum12(unsigned char *ptr, int len)
 {
@@ -68,7 +68,7 @@ GSM_Error N6510_ReplyGetFileCRC12(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	return ERR_NONE;
 }
 
-//filesystem 1
+/* filesystem 1 */
 
 static GSM_Error N6510_GetFileCRC1(GSM_StateMachine *s, unsigned char *id)
 {
@@ -100,8 +100,8 @@ GSM_Error N6510_ReplyGetFileFolderInfo1(GSM_Protocol_Message msg, GSM_StateMachi
 		if (!strncmp(DecodeUnicodeString(File->Name),"GMSTemp",7)) return ERR_EMPTY;
 		if (File->Name[0] == 0x00 && File->Name[1] == 0x00) return ERR_UNKNOWN;
 
-//		EncodeHexUnicode (buffer, File->Name, UnicodeLength(File->Name));
-//		smprintf(s,"Name encoded: %s\n",buffer);
+/* 		EncodeHexUnicode (buffer, File->Name, UnicodeLength(File->Name)); */
+/* 		smprintf(s,"Name encoded: %s\n",buffer); */
 
 		i = msg.Buffer[8]*256+msg.Buffer[9];
 		dbgprintf("%02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
@@ -119,7 +119,7 @@ GSM_Error N6510_ReplyGetFileFolderInfo1(GSM_Protocol_Message msg, GSM_StateMachi
 		if (msg.Buffer[i+2] == 0x01) File->Protected    = true;
 		if (msg.Buffer[i+4] == 0x01) File->ReadOnly     = true;
 		if (msg.Buffer[i+5] == 0x01) File->Hidden       = true;
-		if (msg.Buffer[i+6] == 0x01) File->System       = true;//fixme
+		if (msg.Buffer[i+6] == 0x01) File->System       = true;/* fixme */
 
 		File->ModifiedEmpty = false;
 		NOKIA_DecodeDateTime(s, msg.Buffer+i-22, &File->Modified);
@@ -323,7 +323,7 @@ static GSM_Error N6510_GetFileSystemStatus1(GSM_StateMachine *s, GSM_FileSystemS
 	return GSM_WaitFor (s, req, 10, 0x6D, 4, ID_FileSystemStatus);
 }
 
-static GSM_Error N6510_GetFilePart1(GSM_StateMachine *s, GSM_File *File, int *Handle, int *Size)
+static GSM_Error N6510_GetFilePart1(GSM_StateMachine *s, GSM_File *File, int *Handle UNUSED, int *Size)
 {
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
 	int		     	old;
@@ -397,10 +397,10 @@ static GSM_Error N6510_SetFileAttributes1(GSM_StateMachine *s, GSM_File *File)
 	error = N6510_GetFileFolderInfo1(s, &file2, false);
 	if (error != ERR_NONE) return error;
 
-	// setting folder attrib works, but we block it
+	/*  setting folder attrib works, but we block it */
 	if (file2.Folder) return ERR_SHOULDBEFILE;
 
-	// todo
+	/*  todo */
 	if (file2.System != File->System ||
 	    file2.Hidden != File->Hidden ||
 	    file2.Protected != File->Protected) {
@@ -410,7 +410,7 @@ static GSM_Error N6510_SetFileAttributes1(GSM_StateMachine *s, GSM_File *File)
 	return N6510_SetReadOnly1(s, File->ID_FullName, File->ReadOnly);
 }
 
-//function checks if there is file/folder with searched name in folder with specified ID
+/* function checks if there is file/folder with searched name in folder with specified ID */
 static GSM_Error N6510_SearchForFileName1(GSM_StateMachine *s, GSM_File *File)
 {
 	GSM_Error	       	error;
@@ -420,25 +420,25 @@ static GSM_Error N6510_SearchForFileName1(GSM_StateMachine *s, GSM_File *File)
 
 	File->Folder = false;
 
-	//making backup
+	/* making backup */
 	if (Priv->FilesLocationsUsed > 400) return ERR_MOREMEMORY;
 	memcpy(Files, Priv->Files, sizeof(GSM_File)*400);
 	FilesLocationsUsed = Priv->FilesLocationsUsed;
 
-	//putting own data
+	/* putting own data */
 	Priv->Files[0].Level    	= 1;
 	Priv->FilesLocationsUsed 	= 1;
 	CopyUnicodeString(Priv->Files[0].ID_FullName,File->ID_FullName);
 
-	//checking
+	/* checking */
 	error = N6510_GetFileFolderInfo1(s, &Priv->Files[0], true);
 
-	//backuping new data
+	/* backuping new data */
 	if (Priv->FilesLocationsUsed > 400) return ERR_MOREMEMORY;
 	memcpy(Files2, Priv->Files, sizeof(GSM_File)*400);
 	FilesLocationsUsed2 = Priv->FilesLocationsUsed;
 
-	//restoring
+	/* restoring */
 	memcpy(Priv->Files, Files, sizeof(GSM_File)*400);
 	Priv->FilesLocationsUsed = FilesLocationsUsed;
 
@@ -476,18 +476,18 @@ GSM_Error N6510_ReplyAddFileHeader1(GSM_Protocol_Message msg, GSM_StateMachine *
 	return ERR_UNKNOWNRESPONSE;
 }
 
-GSM_Error N6510_ReplyAddFilePart1(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyAddFilePart1(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s UNUSED)
 {
 	return ERR_NONE;
 }
 
-static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Pos, int *Handle)
+static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Pos, int *Handle UNUSED)
 {
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
 	GSM_File		File2;
 	GSM_Error	       	error;
 	int		     	j;
-	unsigned char	   	Header[400] = {
+	unsigned char	   	Header[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
 		N7110_FRAME_HEADER, 0x02, 0x00, 0x00, 0x00, 0x01,
 		0x00, 0x0C,	     /* parent folder ID */
 		0x00, 0x00, 0x00, 0xE8};
@@ -525,7 +525,7 @@ static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Po
 			case GSM_File_Image_GIF    : Header[231]=0x02; Header[233]=0x05; break;
 			case GSM_File_Image_WBMP   : Header[231]=0x02; Header[233]=0x09; break;
 			case GSM_File_Sound_AMR    : Header[231]=0x04; Header[233]=0x01; break;
-			case GSM_File_Sound_MIDI   : Header[231]=0x04; Header[233]=0x05; break; //Header[238]=0x01;
+			case GSM_File_Sound_MIDI   : Header[231]=0x04; Header[233]=0x05; break; /* Header[238]=0x01; */
 			case GSM_File_Sound_NRT    : Header[231]=0x04; Header[233]=0x06; break;
 			case GSM_File_Video_3GP    : Header[231]=0x08; Header[233]=0x05; break;
 			case GSM_File_Java_JAR     : Header[231]=0x10; Header[233]=0x01; break;
@@ -546,9 +546,9 @@ static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Po
 		Header[235] = 0x01;
 		Header[236] = atoi(DecodeUnicodeString(File->ID_FullName)) / 256;
 		Header[237] = atoi(DecodeUnicodeString(File->ID_FullName)) % 256;
-		if (File->Protected) Header[238] = 0x01; //Nokia forward lock
+		if (File->Protected) Header[238] = 0x01; /* Nokia forward lock */
 		if (File->Hidden)    Header[241] = 0x01;
-		if (File->System)    Header[242] = 0x01; //fixme
+		if (File->System)    Header[242] = 0x01; /* fixme */
 		smprintf(s, "Adding file header\n");
 		error=GSM_WaitFor (s, Header, 246, 0x6D, 4, ID_AddFile);
 		if (error != ERR_NONE) return error;
@@ -567,6 +567,7 @@ static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Po
 	*Pos = *Pos + j;
 
 	if (j < 1000) {
+		/* FIXME: This looks strange */
 		end[8] = atoi(DecodeUnicodeString(File->ID_FullName)) / 256;
 		end[9] = atoi(DecodeUnicodeString(File->ID_FullName)) % 256;
 		smprintf(s, "Frame for ending adding file\n");
@@ -603,7 +604,7 @@ static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Po
 				case GSM_File_Image_GIF    : Header[231]=0x02; Header[233]=0x05; break;
 				case GSM_File_Image_WBMP   : Header[231]=0x02; Header[233]=0x09; break;
 				case GSM_File_Sound_AMR    : Header[231]=0x04; Header[233]=0x01; break;
-				case GSM_File_Sound_MIDI   : Header[231]=0x04; Header[233]=0x05; break; //Header[238]=0x01;
+				case GSM_File_Sound_MIDI   : Header[231]=0x04; Header[233]=0x05; break; /* Header[238]=0x01; */
 				case GSM_File_Sound_NRT    : Header[231]=0x04; Header[233]=0x06; break;
 				case GSM_File_Video_3GP    : Header[231]=0x08; Header[233]=0x05; break;
 				case GSM_File_Java_JAR     : Header[231]=0x10; Header[233]=0x01; break;
@@ -649,7 +650,7 @@ static GSM_Error N6510_AddFilePart1(GSM_StateMachine *s, GSM_File *File, int *Po
 	return ERR_NONE;
 }
 
-GSM_Error N6510_ReplyDeleteFileFolder1(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyDeleteFileFolder1(GSM_Protocol_Message msg, GSM_StateMachine *s UNUSED)
 {
 	if (msg.Buffer[4] == 0x01) {
 		return ERR_NONE;
@@ -678,13 +679,14 @@ static GSM_Error N6510_PrivDeleteFileFolder1(GSM_StateMachine *s, unsigned char
 		if (File.Folder) return ERR_SHOULDBEFILE;
 	} else {
 		if (!File.Folder) return ERR_SHOULDBEFOLDER;
-		//dont allow to delete non empty folder
+		/* dont allow to delete non empty folder */
 		if (Priv->FilesLocationsUsed != 0) return ERR_FOLDERNOTEMPTY;
 	}
 
 	error = N6510_SetReadOnly1(s, ID, false);
 	if (error != ERR_NONE) return error;
 
+	/* FIXME: This looks wrong */
 	Delete[8] = atoi(DecodeUnicodeString(ID)) / 256;
 	Delete[9] = atoi(DecodeUnicodeString(ID)) % 256;
 
@@ -714,7 +716,7 @@ static GSM_Error N6510_AddFolder1(GSM_StateMachine *s, GSM_File *File)
 {
 	GSM_File	File2;
 	GSM_Error       error;
-	unsigned char   Header[400] = {
+	unsigned char   Header[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
 		N7110_FRAME_HEADER, 0x04, 0x00, 0x00, 0x00, 0x01,
 		0x00, 0x0C,	     /* parent folder ID */
 		0x00, 0x00, 0x00, 0xE8};
@@ -777,7 +779,7 @@ static GSM_Error N6510_GetFolderListing1(GSM_StateMachine *s, GSM_File *File, bo
 		}
 		Priv->FilesLocationsUsed--;
 
-		//3510 for example
+		/* 3510 for example */
 		if (error == ERR_EMPTY) continue;
 
 		break;
@@ -785,7 +787,7 @@ static GSM_Error N6510_GetFolderListing1(GSM_StateMachine *s, GSM_File *File, bo
 	return error;
 }
 
-//filesystem 2
+/* filesystem 2 */
 
 GSM_Error N6510_ReplyOpenFile2(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
@@ -813,10 +815,11 @@ GSM_Error N6510_ReplyOpenFile2(GSM_Protocol_Message msg, GSM_StateMachine *s)
 
 static GSM_Error N6510_OpenFile2(GSM_StateMachine *s, char *Name, int *Handle, bool Create)
 {
-	unsigned char	 req[200] = {N6110_FRAME_HEADER, 0x72,
-				     0x00,	  // mode 0 - open read only, 0x11 - read write create
+	unsigned char	 req[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] =
+		{N6110_FRAME_HEADER, 0x72,
+				     0x00,	  /*  mode 0 - open read only, 0x11 - read write create */
 				     0x02,
-				     0xFF, 0xFF}; // name length
+				     0xFF, 0xFF}; /*  name length */
 	int		 Pos = 8;
 	GSM_Error	 error;
 
@@ -839,7 +842,7 @@ static GSM_Error N6510_OpenFile2(GSM_StateMachine *s, char *Name, int *Handle, b
 static GSM_Error N6510_CloseFile2(GSM_StateMachine *s, int *Handle)
 {
 	unsigned char	 req[200] = {N6110_FRAME_HEADER, 0x74, 0x00, 0x00,
-				     0x00, 0x00, 0x00, 0x00}; //file handle
+				     0x00, 0x00, 0x00, 0x00}; /* file handle */
 
 	req[6]		 = (*Handle) / (256*256*256);
 	req[7]		 = (*Handle) / (256*256);
@@ -854,7 +857,7 @@ static GSM_Error N6510_GetFileCRC2(GSM_StateMachine *s, int *Handle)
 {
 	unsigned char req2[15000] = {
 		N7110_FRAME_HEADER, 0x66, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00}; 	// handle
+		0x00, 0x00, 0x00, 0x00}; 	/*  handle */
 
 	req2[6]	= (*Handle) / (256*256*256);
 	req2[7]	= (*Handle) / (256*256);
@@ -878,7 +881,7 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 			smprintf(s,"Probably no MMC card\n");
 			Priv->filesystem2error  = ERR_MEMORY;
 			Priv->FilesEnd 		= true;
-			return ERR_NONE;
+			return ERR_MEMORY;
 		case 0x00:
 		case 0x0D:
 			switch (msg.Buffer[5]) {
@@ -899,9 +902,9 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 			if (msg.Buffer[3] == 0x69) {
 				if (Priv->FilesLocationsUsed==1000) {
 					smprintf(s,"Too small buffer for folder data\n");
-					Priv->filesystem2error  = ERR_UNKNOWN;
+					Priv->filesystem2error  = ERR_MOREMEMORY;
 					Priv->FilesEnd 		= true;
-					return ERR_NONE;
+					return ERR_MOREMEMORY;
 				}
 
 				for (i=Priv->FilesLocationsUsed+1;i>0;i--) {
@@ -969,7 +972,7 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 			smprintf(s,"File or folder details received - not available ?\n");
 			Priv->filesystem2error  = ERR_FILENOTEXIST;
 			Priv->FilesEnd 		= true;
-			return ERR_NONE;
+			return ERR_FILENOTEXIST;
 		case 0x0E:
 			smprintf(s,"File or folder details received - empty\n");
 			Priv->FilesEnd = true;
@@ -982,9 +985,9 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 static GSM_Error N6510_GetFileFolderInfo2(GSM_StateMachine *s, GSM_File *File)
 {
 	int		     	Pos=6;
-	unsigned char	   	req[900] = {
+	unsigned char	   	req[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
 		N7110_FRAME_HEADER,0x6C,
-		0xFF, 0xFF}; 			// name length
+		0xFF, 0xFF}; 			/*  name length */
 
 	s->Phone.Data.FileInfo  = File;
 
@@ -1005,8 +1008,9 @@ static GSM_Error N6510_PrivGetFolderListing2(GSM_StateMachine *s, GSM_File *File
 {
 	GSM_Error		error;
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
-	unsigned char	   	req[200] = {N6110_FRAME_HEADER, 0x68,
-					    0xFF, 0xFF}; // name length
+	unsigned char	   	req[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
+		N6110_FRAME_HEADER, 0x68,
+					    0xFF, 0xFF}; /*  name length */
 	int			Pos = 6, i = 0;
 
 	req[4] = (UnicodeLength(File->ID_FullName)*2 + 6)/ 256 ;
@@ -1080,12 +1084,6 @@ static GSM_Error N6510_GetNextFileFolder2(GSM_StateMachine *s, GSM_File *File, b
 	error = N6510_PrivGetFolderListing2(s, &Priv->Files[0]);
 	if (error != ERR_NONE) return error;
 
-	if (Priv->filesystem2error == ERR_UNKNOWN) return ERR_UNKNOWN;
-	//no mmc in phone in this moment
-	if (Priv->filesystem2error == ERR_MEMORY) return ERR_EMPTY;
-	//mmc not supported at all
-	if (Priv->filesystem2error == ERR_FILENOTEXIST) return ERR_EMPTY;
-
 	memcpy(File,&Priv->Files[0],sizeof(GSM_File));
 	for (i=0;i<Priv->FilesLocationsUsed-1;i++) {
 		memcpy(&Priv->Files[i],&Priv->Files[i+1],sizeof(GSM_File));
@@ -1104,10 +1102,10 @@ static GSM_Error N6510_GetFilePart2(GSM_StateMachine *s, GSM_File *File, int *Ha
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
 	unsigned char	   	req[] = {
 		N7110_FRAME_HEADER, 0x5E, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x01,		//file handle
-		0x00, 0x00, 0x00, 0x00, 	//position
-		0x00, 0x00, 0x03, 0xE8, 	//length
-		0x00, 0x00, 0x03, 0xE8};	//buffer length
+		0x00, 0x00, 0x00, 0x01,		/* file handle */
+		0x00, 0x00, 0x00, 0x00, 	/* position */
+		0x00, 0x00, 0x03, 0xE8, 	/* length */
+		0x00, 0x00, 0x03, 0xE8};	/* buffer length */
 
 	if (File->Used == 0x00) {
 		error = N6510_GetFileFolderInfo2(s, File);
@@ -1164,12 +1162,12 @@ static GSM_Error N6510_GetFilePart2(GSM_StateMachine *s, GSM_File *File, int *Ha
 	return ERR_NONE;
 }
 
-GSM_Error N6510_ReplySetFileDate2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplySetFileDate2(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s UNUSED)
 {
 	return ERR_NONE;
 }
 
-GSM_Error N6510_ReplySetAttrib2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplySetAttrib2(GSM_Protocol_Message msg, GSM_StateMachine *s UNUSED)
 {
 	if (msg.Buffer[4] == 0x00) {
 		return ERR_NONE;
@@ -1184,16 +1182,16 @@ static GSM_Error N6510_SetFileAttributes2(GSM_StateMachine *s, GSM_File *File)
 	int		P = 10;
 	GSM_Error	error;
 	GSM_File	File2;
-	unsigned char	Header2[1500] = {
+	unsigned char	Header2[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
 		N7110_FRAME_HEADER, 0x6E,
-		0x00, 0x0c};			//name len
+		0x00, 0x0c};			/* name len */
 
 	memcpy(&File2,File,sizeof(GSM_File));
 
 	error = N6510_GetFileFolderInfo2(s, File);
 	if (error != ERR_NONE) return error;
 
-	//haven't checked.
+	/* haven't checked. */
 	if (File->Folder) return ERR_SHOULDBEFILE;
 
 	Header2[4] = (UnicodeLength(File2.ID_FullName) + 1)/ 256 ;
@@ -1218,11 +1216,11 @@ static GSM_Error N6510_SetFileAttributes2(GSM_StateMachine *s, GSM_File *File)
 	error = N6510_GetFileFolderInfo2(s, File);
 	if (error != ERR_NONE) return error;
 
-	//mmc doesn't support protected
+	/* mmc doesn't support protected */
 	if (File2.System    != File->System     ||
 	    File2.ReadOnly  != File->ReadOnly   ||
 	    File2.Hidden    != File->Hidden     ) {
-//	    File2.Protected != File->Protected) {
+/* 	    File2.Protected != File->Protected) { */
 		return ERR_NOTSUPPORTED;
 	}
 
@@ -1233,15 +1231,15 @@ static GSM_Error N6510_AddFilePart2(GSM_StateMachine *s, GSM_File *File, int *Po
 {
 	GSM_Error	       	error;
 	int		     	j,P;
-//	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
-//	unsigned char		buffer[500];
+/* 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510; */
+/* 	unsigned char		buffer[500]; */
 	unsigned char	   	req[15000] = {
 		N7110_FRAME_HEADER, 0x58, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 	// handle
-		0x00, 0x00, 0x04, 0x00};	// buffer len
-	unsigned char	   	Header[1500] = {
+		0x00, 0x00, 0x00, 0x00, 	/*  handle */
+		0x00, 0x00, 0x04, 0x00};	/*  buffer len */
+	unsigned char	   	Header[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
 		N7110_FRAME_HEADER, 0x86,
-		0x00, 0x0c};			//name len
+		0x00, 0x0c};			/* name len */
 
 	s->Phone.Data.File = File;
 
@@ -1309,20 +1307,20 @@ static GSM_Error N6510_AddFilePart2(GSM_StateMachine *s, GSM_File *File, int *Po
 		error = N6510_SetFileAttributes2(s,File);
 		if (error != ERR_NONE) return error;
 
-//		error = N6510_OpenFile2(s, File->ID_FullName, Handle, false);
-//		if (error != ERR_NONE) return error;
-//		if ((*Handle) == 0) {
-//			error = N6510_OpenFile2(s, File->ID_FullName, Handle, false);
-//			if (error != ERR_NONE) return error;
-//		}
-//		error = N6510_GetFileCRC2(s, Handle);
-//		if (error != ERR_NONE) return error;
-//		error = N6510_CloseFile2(s, Handle);
-//		if (error != ERR_NONE) return error;
-//		if (N6510_FindFileCheckSum12(File->Buffer, File->Used) != Priv->FileCheckSum) {
-//			smprintf(s,"File2 checksum is %i, File checksum is %i\n",N6510_FindFileCheckSum12(File->Buffer, File->Used),Priv->FileCheckSum);
-//			return ERR_WRONGCRC;
-//		}
+/* 		error = N6510_OpenFile2(s, File->ID_FullName, Handle, false); */
+/* 		if (error != ERR_NONE) return error; */
+/* 		if ((*Handle) == 0) { */
+/* 			error = N6510_OpenFile2(s, File->ID_FullName, Handle, false); */
+/* 			if (error != ERR_NONE) return error; */
+/* 		} */
+/* 		error = N6510_GetFileCRC2(s, Handle); */
+/* 		if (error != ERR_NONE) return error; */
+/* 		error = N6510_CloseFile2(s, Handle); */
+/* 		if (error != ERR_NONE) return error; */
+/* 		if (N6510_FindFileCheckSum12(File->Buffer, File->Used) != Priv->FileCheckSum) { */
+/* 			smprintf(s,"File2 checksum is %i, File checksum is %i\n",N6510_FindFileCheckSum12(File->Buffer, File->Used),Priv->FileCheckSum); */
+/* 			return ERR_WRONGCRC; */
+/* 		} */
 
 		return ERR_EMPTY;
 	}
@@ -1342,7 +1340,7 @@ static GSM_Error N6510_GetFolderListing2(GSM_StateMachine *s, GSM_File *File, bo
 		    strcasecmp(DecodeUnicodeString(File->ID_FullName),"d:") == 0   ||
 		    strcasecmp(DecodeUnicodeString(File->ID_FullName),"d:\\") == 0) {
 		} else {
-			//we must check, if user gave folder name or not
+			/* we must check, if user gave folder name or not */
 			error = N6510_GetFileFolderInfo2(s, File);
 			if (error != ERR_NONE) return error;
 			if (!File->Folder) return ERR_SHOULDBEFOLDER;
@@ -1353,13 +1351,6 @@ static GSM_Error N6510_GetFolderListing2(GSM_StateMachine *s, GSM_File *File, bo
 		error = N6510_PrivGetFolderListing2(s, File);
 		if (error != ERR_NONE) return error;
 
-		if (Priv->filesystem2error == ERR_UNKNOWN) return ERR_UNKNOWN;
-
-		//no mmc
-		if (Priv->filesystem2error == ERR_MEMORY) return ERR_MEMORY;
-
-		if (Priv->filesystem2error == ERR_FILENOTEXIST) return ERR_FILENOTEXIST;
-
 		memcpy(File,&Priv->Files[0],sizeof(GSM_File));
 		for (i=0;i<Priv->FilesLocationsUsed-1;i++) {
 			memcpy(&Priv->Files[i],&Priv->Files[i+1],sizeof(GSM_File));
@@ -1380,12 +1371,12 @@ static GSM_Error N6510_GetFolderListing2(GSM_StateMachine *s, GSM_File *File, bo
 	return ERR_NONE;
 }
 
-GSM_Error N6510_ReplyDeleteFile2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyDeleteFile2(GSM_Protocol_Message msg, GSM_StateMachine *s UNUSED)
 {
 	if (msg.Buffer[4] == 0x00) {
 		return ERR_NONE;
 	} else if (msg.Buffer[4] == 0x03) {
-		//trying to delete read only
+		/* trying to delete read only */
 		return ERR_UNKNOWN;
 	} else if (msg.Buffer[4] == 0x06) {
 		return ERR_FILENOTEXIST;
@@ -1396,12 +1387,13 @@ GSM_Error N6510_ReplyDeleteFile2(GSM_Protocol_Message msg, GSM_StateMachine *s)
 
 static GSM_Error N6510_DeleteFile2(GSM_StateMachine *s, unsigned char *ID)
 {
-	unsigned char   req[1000] = {N7110_FRAME_HEADER, 0x62};
+	unsigned char   req[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
+		N7110_FRAME_HEADER, 0x62};
 	int		Pos = 6;
 	GSM_File	file;
 	GSM_Error	error;
 
-	//first remove readonly
+	/* first remove readonly */
 	file.ReadOnly  = false;
 	file.Hidden    = false;
 	file.System    = false;
@@ -1424,7 +1416,7 @@ static GSM_Error N6510_DeleteFile2(GSM_StateMachine *s, unsigned char *ID)
 	return GSM_WaitFor (s, req, Pos, 0x6D, 4, ID_DeleteFile);
 }
 
-GSM_Error N6510_ReplyAddFolder2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyAddFolder2(GSM_Protocol_Message msg, GSM_StateMachine *s UNUSED)
 {
 	if (msg.Buffer[4] == 0x00) {
 		return ERR_NONE;
@@ -1441,7 +1433,8 @@ GSM_Error N6510_ReplyAddFolder2(GSM_Protocol_Message msg, GSM_StateMachine *s)
 static GSM_Error N6510_AddFolder2(GSM_StateMachine *s, GSM_File *File)
 {
 	GSM_Error	error;
-	unsigned char   req[1000] = {N7110_FRAME_HEADER, 0x64};
+	unsigned char   req[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
+		N7110_FRAME_HEADER, 0x64};
 	int		Pos = 6;
 	int		Len = 0;
 
@@ -1470,7 +1463,7 @@ static GSM_Error N6510_AddFolder2(GSM_StateMachine *s, GSM_File *File)
 	return error;
 }
 
-GSM_Error N6510_ReplyDeleteFolder2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N6510_ReplyDeleteFolder2(GSM_Protocol_Message msg, GSM_StateMachine *s UNUSED)
 {
 	if (msg.Buffer[4] == 0x00) {
 		return ERR_NONE;
@@ -1488,10 +1481,11 @@ static GSM_Error N6510_DeleteFolder2(GSM_StateMachine *s, unsigned char *ID)
 {
 	GSM_File	File2;
 	GSM_Error	error;
-	unsigned char   req[1000] = {N7110_FRAME_HEADER, 0x6A};
+	unsigned char   req[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))] = {
+		N7110_FRAME_HEADER, 0x6A};
 	int		Pos = 6;
 
-	//we don't want to allow deleting non empty folders
+	/* we don't want to allow deleting non empty folders */
 	CopyUnicodeString(File2.ID_FullName,ID);
 	error = N6510_GetFolderListing2(s, &File2, true);
 	switch (error) {
@@ -1516,7 +1510,7 @@ static GSM_Error N6510_DeleteFolder2(GSM_StateMachine *s, unsigned char *ID)
 	return GSM_WaitFor (s, req, Pos, 0x6D, 4, ID_DeleteFolder);
 }
 
-//shared
+/* shared */
 
 GSM_Error N6510_GetFolderListing(GSM_StateMachine *s, GSM_File *File, bool start)
 {
@@ -1536,7 +1530,7 @@ GSM_Error N6510_GetFolderListing(GSM_StateMachine *s, GSM_File *File, bool start
 		CopyUnicodeString(File2.ID_FullName,File->ID_FullName+3*2);
 		error = N6510_GetFolderListing1(s,&File2,start);
 		memcpy(File,&File2,sizeof(GSM_File));
-		//GetFolderListing changes ID
+		/* GetFolderListing changes ID */
 		EncodeUnicode(File->ID_FullName,"c:/",3);
 		CopyUnicodeString(File->ID_FullName+UnicodeLength(File->ID_FullName)*2,File2.ID_FullName);
 		return error;
@@ -1553,14 +1547,14 @@ GSM_Error N6510_GetNextFileFolder(GSM_StateMachine *s, GSM_File *File, bool star
 {
 	GSM_Error	       	error;
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
-	char		    	buf[200];
+	char		    	buf[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))];
 
 	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (start) {
 		Priv->Use2 = true;
 		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
-			//series 40 3.0 don't have filesystem 1
+			/* series 40 3.0 don't have filesystem 1 */
 			Priv->Use2 = false;
 		}
 		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
@@ -1598,7 +1592,7 @@ GSM_Error N6510_GetNextFileFolder(GSM_StateMachine *s, GSM_File *File, bool star
 GSM_Error N6510_GetFilePart(GSM_StateMachine *s, GSM_File *File, int *Handle, int *Size)
 {
 	GSM_File	File2;
-	char	    	buf[200];
+	char	    	buf[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))];
 	GSM_Error       error;
 
 	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
@@ -1643,7 +1637,7 @@ GSM_Error N6510_AddFilePart(GSM_StateMachine *s, GSM_File *File, int *Pos, int *
 		CopyUnicodeString(File2.ID_FullName,File->ID_FullName+3*2);
 		error = N6510_AddFilePart1(s,&File2,Pos,Handle);
 		memcpy(File,&File2,sizeof(GSM_File));
-		//addfilepart returns new ID
+		/* addfilepart returns new ID */
 		EncodeUnicode(File->ID_FullName,"c:/",3);
 		CopyUnicodeString(File->ID_FullName+UnicodeLength(File->ID_FullName)*2,File2.ID_FullName);
 		return error;
@@ -1694,7 +1688,7 @@ GSM_Error N6510_AddFolder(GSM_StateMachine *s, GSM_File *File)
 		CopyUnicodeString(File2.ID_FullName,File->ID_FullName+3*2);
 		error = N6510_AddFolder1(s,&File2);
 		memcpy(File,&File2,sizeof(GSM_File));
-		//addfolder returns new ID
+		/* addfolder returns new ID */
 		EncodeUnicode(File->ID_FullName,"c:/",3);
 		CopyUnicodeString(File->ID_FullName+UnicodeLength(File->ID_FullName)*2,File2.ID_FullName);
 		return error;
@@ -1745,7 +1739,7 @@ GSM_Error N6510_GetFileSystemStatus(GSM_StateMachine *s, GSM_FileSystemStatus *s
 GSM_Error N6510_SetFileAttributes(GSM_StateMachine *s, GSM_File *File)
 {
 	GSM_File	File2;
-	char	    	buf[200];
+	char	    	buf[20 + (2 * (GSM_MAX_FILENAME_ID_LENGTH + 1))];
 	GSM_Error       error;
 
 	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
@@ -1887,7 +1881,7 @@ GSM_Error N6510_PrivGet3220FilesystemMMSFolders(GSM_StateMachine *s, GSM_MMSFold
 	}
 }
 
-//Series 40 3.0
+/* Series 40 3.0 */
 GSM_Error N6510_PrivGetFilesystemMMSFolders(GSM_StateMachine *s, GSM_MMSFolders *folders)
 {
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
@@ -2000,7 +1994,7 @@ GSM_Error N6510_GetMMSFolders(GSM_StateMachine *s, GSM_MMSFolders *folders)
 		}
 	}
 
-	//6230i
+	/* 6230i */
 	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 		EncodeUnicode(Files.ID_FullName,"d:/predefmessages",17);
 		folders->Number = 0;
@@ -2044,7 +2038,8 @@ GSM_Error N6510_GetNextMMSFileInfo(GSM_StateMachine *s, unsigned char *FileID, i
 
 	if (start) {
 		error = N6510_GetMMSFolders(s, &folders);
-		if (error != ERR_NONE) return error;
+		if (error != ERR_NONE)
+			return error;
 
 		Priv->MMSFolderNum 	= 0;
 		Priv->MMSFolderError 	= ERR_EMPTY;
@@ -2053,19 +2048,23 @@ GSM_Error N6510_GetNextMMSFileInfo(GSM_StateMachine *s, unsigned char *FileID, i
 	while(true) {
 		if (Priv->MMSFolderError == ERR_NONE) {
 			Priv->MMSFolderError = N6510_GetFolderListing(s,&Priv->MMSFile,false);
-			if (Priv->MMSFolderError != ERR_EMPTY && Priv->MMSFolderError != ERR_NONE) return Priv->MMSFolderError;
+			if (Priv->MMSFolderError != ERR_EMPTY && Priv->MMSFolderError != ERR_NONE)
+				return Priv->MMSFolderError;
 		}
 
 		if (Priv->MMSFolderError == ERR_EMPTY) {
 			while (1) {
-				if (UnicodeLength(Priv->MMSFoldersID2[Priv->MMSFolderNum])==0) return ERR_EMPTY;
+				if (UnicodeLength(Priv->MMSFoldersID2[Priv->MMSFolderNum]) == 0)
+					return ERR_EMPTY;
 
 				CopyUnicodeString(Priv->MMSFile.ID_FullName,Priv->MMSFoldersID2[Priv->MMSFolderNum]);
 				Priv->MMSFolderNum++;
 
 				Priv->MMSFolderError = N6510_GetFolderListing(s,&Priv->MMSFile,true);
-				if (Priv->MMSFolderError == ERR_EMPTY) continue;
-				if (Priv->MMSFolderError != ERR_NONE) return Priv->MMSFolderError;
+				if (Priv->MMSFolderError == ERR_EMPTY)
+					continue;
+				if (Priv->MMSFolderError != ERR_NONE)
+					return Priv->MMSFolderError;
 				break;
 			}
 		}
@@ -2079,25 +2078,29 @@ GSM_Error N6510_GetNextMMSFileInfo(GSM_StateMachine *s, unsigned char *FileID, i
 			error = N6510_GetFilePart2(s, &file, &Handle, &Size);
 			if (error == ERR_NONE) {
 				error = N6510_CloseFile2(s, &Handle);
-				if (error != ERR_NONE) return error;
-			} else if (error == ERR_EMPTY) {
-			} else {
+				if (error != ERR_NONE)
+					return error;
+			} else if (error != ERR_EMPTY) {
 				return error;
 			}
 
-			//0x00 = SMS, 0x01,0x03 = MMS
+			/* 0x00 = SMS, 0x01,0x03 = MMS */
 			if (file.Buffer[6] != 0x00) {
 				free(file.Buffer);
+				file.Buffer = NULL;
 				break;
 			}
 			free(file.Buffer);
-		} else break;
+			file.Buffer = NULL;
+		} else {
+			break;
+		}
 	}
 
 	return ERR_NONE;
 }
 
-//Series 40 3.0
+/* Series 40 3.0 */
 GSM_Error N6510_PrivGetFilesystemSMSFolders(GSM_StateMachine *s, GSM_SMSFolders *folders, bool real)
 {
 	bool 			Start = true;
@@ -2158,13 +2161,13 @@ GSM_Error N6510_PrivGetFilesystemSMSFolders(GSM_StateMachine *s, GSM_SMSFolders
 	}
 }
 
-//Series 40 3.0
+/* Series 40 3.0 */
 GSM_Error N6510_GetFilesystemSMSFolders(GSM_StateMachine *s, GSM_SMSFolders *folders)
 {
 	return N6510_PrivGetFilesystemSMSFolders(s, folders, false);
 }
 
-//Series 40 3.0
+/* Series 40 3.0 */
 static void N26510_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char *folderid, int *location)
 {
 	int ifolderid;
@@ -2182,7 +2185,7 @@ static void N26510_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsi
 		sms->Folder,sms->Location,*folderid,*location);
 }
 
-//Series 40 3.0
+/* Series 40 3.0 */
 static void N26510_SetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char folderid, int location)
 {
 	sms->Folder	= 0;
@@ -2191,7 +2194,7 @@ static void N26510_SetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsi
 		folderid,location,sms->Folder,sms->Location);
 }
 
-//Series 40 3.0
+/* Series 40 3.0 */
 GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms, bool start)
 {
 	GSM_Phone_N6510Data	*Priv = &s->Phone.Data.Priv.N6510;
@@ -2201,7 +2204,8 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 	GSM_File		FFF;
 	bool			start2=start;
 	GSM_SMSMessageLayout 	Layout;
-	double			g;
+
+	GSM_SetDefaultReceivedSMSData(&sms->SMS[0]);
 
 	while (true) {
 		if (start2) {
@@ -2231,7 +2235,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 			Priv->SMSFileError = N6510_GetFolderListing(s,&Priv->SMSFile,true);
 		}
 
-		//readfile
+		/* readfile */
 		FFF.Buffer= NULL;
 		FFF.Used = 0;
 		FFF.ID_FullName[0] = 0;
@@ -2241,7 +2245,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		error = ERR_NONE;
 		while (error == ERR_NONE) {
 			error = N6510_GetFilePart(s,&FFF,&Handle,&Size);
-			//if mms, don't read all
+			/* if mms, don't read all */
 			if (error==ERR_NONE && FFF.Used>5 && FFF.Buffer[6] != 0x00) {
 				error = N6510_CloseFile2(s, &Handle);
 				if (error != ERR_NONE) return error;
@@ -2251,15 +2255,16 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		if (FFF.Buffer != NULL)  {
 			DumpMessage(&s->di, FFF.Buffer, FFF.Used);
 
-			//0x00 = SMS, 0x01,0x03 = MMS
+			/* 0x00 = SMS, 0x01,0x03 = MMS */
 			if (FFF.Buffer[6] == 0x00) break;
 
 			smprintf(s,"mms file");
 			free(FFF.Buffer);
+			FFF.Buffer = NULL;
 		}
 	}
 
-	//decode file
+	/* decode file */
 	sms->Number = 0;
 	i = 176;
 	GSM_SetDefaultSMSData(&sms->SMS[0]);
@@ -2274,7 +2279,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 
 		Layout.firstbyte = FFF.Buffer[i];
 
-		//semioctets to chars in phone number
+		/* semioctets to chars in phone number */
 		if (FFF.Buffer[i+1] % 2) FFF.Buffer[i+1]++;
 		FFF.Buffer[i+1]=FFF.Buffer[i+1] / 2 + 1;
 		i+=FFF.Buffer[i+1];
@@ -2285,10 +2290,8 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		Layout.Text = i+1;
 		GSM_DecodeSMSFrameText(&sms->SMS[0], FFF.Buffer, Layout);
 		if (sms->SMS[0].Coding == SMS_Coding_Default_No_Compression) {
-			//we need to find, if part of byte was used or not
-			g = ((double)FFF.Buffer[i]*7/8);
-			g -= (unsigned int)((FFF.Buffer[i]*7/8));
-			if (g != 0.0) {
+			/* we need to find, if part of byte was used or not */
+			if (FFF.Buffer[i] * 7 % 8 != 0) {
 				i+=(FFF.Buffer[i]*7/8)+1;
 			} else {
 				i+=(FFF.Buffer[i]*7/8);
@@ -2297,15 +2300,14 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 			i+=FFF.Buffer[i];
 		}
 		i+=11;
-		EncodeUnicode(sms->SMS[0].SMSC.Number,FFF.Buffer+i,strlen(FFF.Buffer+i));
 		sms->SMS[0].UDH.Type = UDH_NoUDH;
 		sms->SMS[0].PDU = SMS_Deliver;
 		sms->SMS[0].Class = -1;
-		sms->SMS[0].State = SMS_Read;//fixme
+		sms->SMS[0].State = SMS_Read;/* fixme */
 		sms->Number = 1;
 		CopyUnicodeString(sms->SMS[0].Number,FFF.Buffer+94);
 		i+=UnicodeLength(sms->SMS[0].SMSC.Number);
-		//if(FFF.Buffer[176]!=0x20)
+		/* if(FFF.Buffer[176]!=0x20) */
 		i+=4;
 		CopyUnicodeString(sms->SMS[0].Name,FFF.Buffer+i);
 		break;
@@ -2313,17 +2315,15 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		smprintf(s,"sms received and edited ?\n");
 
 		i+=15;
-		Layout.firstbyte = 1; //fixme
+		Layout.firstbyte = 1; /* fixme */
 		Layout.TPDCS=i;
 		i+=2;
 		Layout.TPUDL = i;
 		Layout.Text = i+1;
 		GSM_DecodeSMSFrameText(&sms->SMS[0], FFF.Buffer, Layout);
 		if (sms->SMS[0].Coding == SMS_Coding_Default_No_Compression) {
-			//we need to find, if part of byte was used or not
-			g = ((double)FFF.Buffer[i]*7/8);
-			g -= (unsigned int)((FFF.Buffer[i]*7/8));
-			if (g != 0.0) {
+			/* we need to find, if part of byte was used or not */
+			if (FFF.Buffer[i] * 7 % 8 != 0) {
 				i+=(FFF.Buffer[i]*7/8)+1;
 			} else {
 				i+=(FFF.Buffer[i]*7/8);
@@ -2332,11 +2332,15 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 			i+=FFF.Buffer[i];
 		}
 		i+=11;
-		EncodeUnicode(sms->SMS[0].SMSC.Number,FFF.Buffer+i,strlen(FFF.Buffer+i));
+		if (strlen(FFF.Buffer+i) > GSM_MAX_NUMBER_LENGTH) {
+			smprintf(s, "WARNING: Too long SMS number, ignoring!\n");
+		} else {
+			EncodeUnicode(sms->SMS[0].SMSC.Number,FFF.Buffer+i,strlen(FFF.Buffer+i));
+		}
 		sms->SMS[0].UDH.Type = UDH_NoUDH;
 		sms->SMS[0].PDU = SMS_Deliver;
 		sms->SMS[0].Class = -1;
-		sms->SMS[0].State = SMS_Read;//fixme
+		sms->SMS[0].State = SMS_Read;/* fixme */
 		sms->Number = 1;
 		CopyUnicodeString(sms->SMS[0].Number,FFF.Buffer+94);
 		if (UnicodeLength(sms->SMS[0].SMSC.Number)!=0) {
@@ -2349,7 +2353,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		CopyUnicodeString(sms->SMS[0].Number,FFF.Buffer+94);
 		if (sms->SMS[0].Number[0]==0x00 && sms->SMS[0].Number[1]==0x00) {
 			smprintf(s,"%02x\n",FFF.Buffer[i+2]);
-			//semioctets to chars in phone number
+			/* semioctets to chars in phone number */
 			if (FFF.Buffer[i+2] % 2) FFF.Buffer[i+2]++;
 			FFF.Buffer[i+2]=FFF.Buffer[i+2] / 2 + 1;
 			smprintf(s,"number avail. %i\n",FFF.Buffer[i+2]+2);
@@ -2357,7 +2361,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		} else {
 			i+=15;
 		}
-		Layout.firstbyte = 1; //fixme
+		Layout.firstbyte = 1; /* fixme */
 		smprintf(s,"TPDCS %02x\n",FFF.Buffer[i]);
 		Layout.TPDCS=i;
 		i+=2;
@@ -2366,10 +2370,8 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		Layout.Text = i+1;
 		GSM_DecodeSMSFrameText(&sms->SMS[0], FFF.Buffer, Layout);
 		if (sms->SMS[0].Coding == SMS_Coding_Default_No_Compression) {
-			//we need to find, if part of byte was used or not
-			g = ((double)FFF.Buffer[i]*7/8);
-			g -= (unsigned int)((FFF.Buffer[i]*7/8));
-			if (g != 0.0) {
+			/* we need to find, if part of byte was used or not */
+			if (FFF.Buffer[i] * 7 % 8 != 0) {
 				i+=(FFF.Buffer[i]*7/8)+1;
 			} else {
 				i+=(FFF.Buffer[i]*7/8);
@@ -2380,7 +2382,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		i+=11;
 		sms->SMS[0].UDH.Type = UDH_NoUDH;
 		sms->SMS[0].PDU = SMS_Deliver;
-		sms->SMS[0].Class = -1;//fixme
+		sms->SMS[0].Class = -1;/* fixme */
 		sms->SMS[0].State = SMS_Sent;
 		sms->Number = 1;
 		CopyUnicodeString(sms->SMS[0].Name,FFF.Buffer+i);
@@ -2415,7 +2417,7 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 			if (FFF.Buffer[176]==0x71) {
 				GSM_UnpackSemiOctetNumber(sms->SMS[0].Number,FFF.Buffer+i+2 , true);
 			}
-			//semioctets to chars in phone number
+			/* semioctets to chars in phone number */
 			if (FFF.Buffer[i+2] % 2) FFF.Buffer[i+2]++;
 			FFF.Buffer[i+2]=FFF.Buffer[i+2] / 2 + 1;
 			smprintf(s,"number avail. %i\n",FFF.Buffer[i+2]+2);
@@ -2432,10 +2434,8 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		Layout.Text = i+1;
 		GSM_DecodeSMSFrameText(&sms->SMS[0], FFF.Buffer, Layout);
 		if (sms->SMS[0].Coding == SMS_Coding_Default_No_Compression) {
-			//we need to find, if part of byte was used or not
-			g = ((double)FFF.Buffer[i]*7/8);
-			g -= (unsigned int)((FFF.Buffer[i]*7/8));
-			if (g != 0.0) {
+			/* we need to find, if part of byte was used or not */
+			if (FFF.Buffer[i] * 7 % 8 != 0) {
 				i+=(FFF.Buffer[i]*7/8)+1;
 			} else {
 				i+=(FFF.Buffer[i]*7/8);
@@ -2469,18 +2469,21 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		}
 
 		sms->SMS[0].PDU = SMS_Deliver;
-		sms->SMS[0].Class = -1;//fixme
+		sms->SMS[0].Class = -1;/* fixme */
 		sms->SMS[0].State = SMS_Sent;
 		sms->Number = 1;
 		break;
 	case 0x44:
-	case 0x40://40 without name
-	case 0x64://64 without name
+	case 0x40:/* 40 without name */
+	case 0x64:/* 64 without name */
+	case 0x60:/* don't know */
+	case 0xa0:/* don't know */
+	case 0xa4:/* don't know */
 		smprintf(s,"sms with udh and number\n");
 
 		Layout.firstbyte = i;
 
-		//semioctets to chars in phone number
+		/* semioctets to chars in phone number */
 		if (FFF.Buffer[i+1] % 2) FFF.Buffer[i+1]++;
 		FFF.Buffer[i+1]=FFF.Buffer[i+1] / 2 + 1;
 		i+=FFF.Buffer[i+1];
@@ -2494,10 +2497,12 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		GSM_DecodeSMSFrameText(&sms->SMS[0], FFF.Buffer, Layout);
 		if (FFF.Buffer[176]!=0x64) {
 			i+=FFF.Buffer[Layout.TPUDL];
-			EncodeUnicode(sms->SMS[0].SMSC.Number,FFF.Buffer+i+2,FFF.Buffer[i+1]);
+			if (FFF.Buffer[i+2] != 0xff && FFF.Buffer[i+1] != 0xff && FFF.Buffer[i+1] < GSM_MAX_NUMBER_LENGTH) {
+				EncodeUnicode(sms->SMS[0].SMSC.Number,FFF.Buffer+i+2,FFF.Buffer[i+1]);
+			}
 		}
 		sms->SMS[0].PDU = SMS_Deliver;
-		sms->SMS[0].Class = -1;//fixme
+		sms->SMS[0].Class = -1;/* fixme */
 		sms->SMS[0].State = SMS_Sent;
 		sms->Number = 1;
 		break;
@@ -2505,12 +2510,13 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		/* Fail if we don't know message type */
 		smprintf(s,"unknown sms file %02x\n",FFF.Buffer[i]);
 		free(FFF.Buffer);
+		FFF.Buffer = NULL;
 		return ERR_UNKNOWN;
 	}
 
 	sms->SMS[0].Memory = MEM_ME;
 
-	//from gsmsms.c - it should be shared
+	/* from gsmsms.c - it should be shared */
 	sms->SMS[0].Class = -1;
 	if (Layout.TPDCS != 255) {
 		/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
@@ -2536,11 +2542,12 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 	memcpy(&sms->SMS[0].DateTime,&FFF.Modified,sizeof(GSM_DateTime));
 	sms->SMS[0].DateTime.Timezone = 0;
 	free(FFF.Buffer);
+	FFF.Buffer = NULL;
 
 	folderid = 0;
 	N26510_SetSMSLocation(s, &sms->SMS[0], folderid, location);
 	sms->SMS[0].Folder = Priv->SMSFileFolder;
-	sms->SMS[0].Location = 0; //fixme
+	sms->SMS[0].Location = 0; /* fixme */
 
 	return ERR_NONE;
 }
