@@ -30,9 +30,9 @@ unsigned char N71_65_MEMORY_TYPES[] = {
 	  0x00,		 0x00
 };
 
-int N71_65_PackPBKBlock(GSM_StateMachine *s, int id, int size, int no, unsigned char *buf, unsigned char *block)
+size_t N71_65_PackPBKBlock(GSM_StateMachine *s, int id, size_t size, int no, unsigned char *buf, unsigned char *block)
 {
-	smprintf(s, "Packing phonebook block with ID = %i, block number = %i, block length = %i\n",id,no+1,size+6);
+	smprintf(s, "Packing phonebook block with ID = %i, block number = %i, block length = " SIZE_T_FORMAT "\n",id,no+1,size+6);
 
 	block[0] 	= id;
 	block[1] 	= 0;
@@ -45,7 +45,7 @@ int N71_65_PackPBKBlock(GSM_StateMachine *s, int id, int size, int no, unsigned
 	return (size + 6);
 }
 
-int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_MemoryEntry *entry, int *block2, bool DCT4, bool VoiceTag)
+size_t N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_MemoryEntry *entry, size_t *block2, bool DCT4, bool VoiceTag)
 {
 	int		count=0, len, i, block=0, j;
 	unsigned char	string[500];
@@ -328,8 +328,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	while (true) {
 		if (bs != 0) {
 			length = length + bs;
-//			if (length == MessageLength) break;
-			//bb5
+			/* bb5 */
 			if (length >= MessageLength-1) break;
 			Block = &Block[bs];
 		}
@@ -356,7 +355,13 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 				smprintf(s, "Too long text\n");
 				return ERR_UNKNOWNRESPONSE;
 			}
-			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			/* No text? */
+			if (Block[5] < 2) {
+				entry->Entries[entry->EntriesNum].Text[0] = 0;
+				entry->Entries[entry->EntriesNum].Text[1] = 0;
+			} else {
+				memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			}
 			entry->Entries[entry->EntriesNum].EntryType=Type;
 			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
 			entry->EntriesNum ++;
@@ -369,8 +374,6 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	while (true) {
 		if (bs != 0) {
 			length = length + bs;
-//			if (length == MessageLength) break;
-			//bb5
 			if (length >= MessageLength-1) break;
 			Block = &Block[bs];
 		}
@@ -411,15 +414,24 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 				smprintf(s, "Too long text\n");
 				return ERR_UNKNOWNRESPONSE;
 			}
-			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			/* No text? */
+			if (Block[5] < 2) {
+				entry->Entries[entry->EntriesNum].Text[0] = 0;
+				entry->Entries[entry->EntriesNum].Text[1] = 0;
+			} else {
+				memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			}
 			entry->Entries[entry->EntriesNum].EntryType=Type;
 			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
 			if (Block[0] == N7110_PBK_NAME) {
-				if (entry->MemoryType==MEM7110_CG) {
-					memcpy(bitmap->Text,Block+6,Block[5]);
-				}
-				if (entry->MemoryType==MEM6510_CG2) {
-					memcpy(bitmap->Text,Block+6,Block[5]);
+				if (entry->MemoryType == MEM7110_CG || entry->MemoryType == MEM6510_CG2) {
+					/* No text? */
+					if (Block[5] < 2) {
+						bitmap->Text[0] = 0;
+						bitmap->Text[1] = 0;
+					} else {
+						memcpy(bitmap->Text,Block+6,Block[5]);
+					}
 				}
 			}
 			entry->EntriesNum ++;
@@ -428,7 +440,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		if (Block[0] == N7110_PBK_DATETIME) {
 			entry->Entries[entry->EntriesNum].EntryType=PBK_Date;
 			NOKIA_DecodeDateTime(s, Block+6, &entry->Entries[entry->EntriesNum].Date);
-			//some phones reverse it
+			/* some phones reverse it */
 			if (entry->Entries[entry->EntriesNum].Date.Year > 3000) {
 				entry->Entries[entry->EntriesNum].Date.Year = Block[7]*256+Block[6];
 			}
@@ -505,7 +517,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
-		//to checking
+		/* to checking */
 		if (Block[0] == S4030_PBK_CALLLENGTH) {
 			entry->Entries[entry->EntriesNum].CallLength = Block[9]*256*256+Block[10]*256+Block[11];
 			entry->Entries[entry->EntriesNum].EntryType=PBK_CallLength;
@@ -675,9 +687,9 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		if (Block[0] == S4030_PBK_BIRTHDAY) {
 			entry->Entries[entry->EntriesNum].EntryType=PBK_Date;
 			NOKIA_DecodeDateTime(s, Block+6, &entry->Entries[entry->EntriesNum].Date);
-			// We don't have seconds available here
+			/* We don't have seconds available here */
 			entry->Entries[entry->EntriesNum].Date.Second = 0;
-			//some phones reverse it
+			/* some phones reverse it */
 			if (entry->Entries[entry->EntriesNum].Date.Year > 3000) {
 				entry->Entries[entry->EntriesNum].Date.Year = Block[7]*256+Block[6];
 			}
@@ -795,12 +807,11 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 				bitmap->RingtoneID		= Block[10]*256+Block[11];
 				bitmap->DefaultRingtone 	= false;
 			} else {
-				//series 40 3.0
+				/* series 40 3.0 */
 				smprintf(s, "Filesystem ringtone ID: %02x\n",Block[10]*256+Block[11]);
 				entry->Entries[entry->EntriesNum].EntryType=PBK_RingtoneID;
 				entry->Entries[entry->EntriesNum].Number=Block[10]*256+Block[11];
 				entry->EntriesNum ++;
-//				return ERR_UNKNOWNRESPONSE;
 			}
 			continue;
 		}
@@ -840,6 +851,10 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
+		if (Block[0] == N6510_PBK_UNKNOWN5) {
+			smprintf(s,"Unknown entry\n");
+			continue;
+		}
 		if (Block[0] == N6510_PBK_GROUP2_ID) {
 			smprintf(s,"Group ID (6230i or later)\n");
 
@@ -1418,22 +1433,26 @@ GSM_Error DCT3DCT4_AnswerCall(GSM_StateMachine *s, int ID)
 
 GSM_Error DCT3DCT4_ReplyGetModelFirmware(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	GSM_Lines	lines;
+	GSM_CutLines	lines;
 	GSM_Phone_Data	*Data = &s->Phone.Data;
 
+	InitLines(&lines);
+
 	SplitLines(msg.Buffer, msg.Length, &lines, "\x20\x0A", 2, false);
 
-	strcpy(Data->Model,GetLineString(msg.Buffer, lines, 4));
+	strcpy(Data->Model,GetLineString(msg.Buffer, &lines, 4));
 	smprintf(s, "Received model %s\n",Data->Model);
-	Data->ModelInfo = GetModelData(NULL,Data->Model,NULL);
+	Data->ModelInfo = GetModelData(s, NULL, Data->Model, NULL);
 
-	strcpy(Data->VerDate,GetLineString(msg.Buffer, lines, 3));
+	strcpy(Data->VerDate,GetLineString(msg.Buffer, &lines, 3));
 	smprintf(s, "Received firmware date %s\n",Data->VerDate);
 
-	strcpy(Data->Version,GetLineString(msg.Buffer, lines, 2));
+	strcpy(Data->Version,GetLineString(msg.Buffer, &lines, 2));
 	smprintf(s, "Received firmware version %s\n",Data->Version);
 	GSM_CreateFirmwareNumber(s);
 
+	FreeLines(&lines);
+
 	return ERR_NONE;
 }
 
@@ -1446,13 +1465,13 @@ GSM_Error DCT3DCT4_GetModel (GSM_StateMachine *s)
 
 	smprintf(s, "Getting model\n");
 	error=GSM_WaitFor (s, req, 5, 0xd1, 3, ID_GetModel);
-	if (error==ERR_NONE) {
-		if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-		    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
-			smprintf(s, "[Connected model  - \"%s\"]\n",s->Phone.Data.Model);
-			smprintf(s, "[Firmware version - \"%s\"]\n",s->Phone.Data.Version);
-			smprintf(s, "[Firmware date    - \"%s\"]\n",s->Phone.Data.VerDate);
-		}
+	if (error == ERR_NONE) {
+		smprintf_level(s, D_TEXT, "[Connected model  - \"%s\"]\n",
+				s->Phone.Data.Model);
+		smprintf_level(s, D_TEXT, "[Firmware version - \"%s\"]\n",
+				s->Phone.Data.Version);
+		smprintf_level(s, D_TEXT, "[Firmware date    - \"%s\"]\n",
+				s->Phone.Data.VerDate);
 	}
 	return error;
 }
@@ -1466,13 +1485,13 @@ GSM_Error DCT3DCT4_GetFirmware (GSM_StateMachine *s)
 
 	smprintf(s, "Getting firmware version\n");
 	error=GSM_WaitFor (s, req, 5, 0xd1, 3, ID_GetFirmware);
-	if (error==ERR_NONE) {
-		if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-		    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
-			smprintf(s, "[Connected model  - \"%s\"]\n",s->Phone.Data.Model);
-			smprintf(s, "[Firmware version - \"%s\"]\n",s->Phone.Data.Version);
-			smprintf(s, "[Firmware date    - \"%s\"]\n",s->Phone.Data.VerDate);
-		}
+	if (error == ERR_NONE) {
+		smprintf_level(s, D_TEXT, "[Connected model  - \"%s\"]\n",
+				s->Phone.Data.Model);
+		smprintf_level(s, D_TEXT, "[Firmware version - \"%s\"]\n",
+				s->Phone.Data.Version);
+		smprintf_level(s, D_TEXT, "[Firmware date    - \"%s\"]\n",
+				s->Phone.Data.VerDate);
 	}
 	return error;
 }
@@ -1486,9 +1505,12 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 		smprintf(s, "No PIN\n");
 		return ERR_SECURITYERROR;
 	case 0x30:
+		if (s->Phone.Data.Memory->MemoryType == MEM_ME ||
+				s->Phone.Data.Memory->MemoryType == MEM_SM) {
+			smprintf(s, "Empty entry\n");
+			return ERR_EMPTY;
+		}
 		smprintf(s, "Invalid memory type\n");
-		if (s->Phone.Data.Memory->MemoryType == MEM_ME) return ERR_EMPTY;
-		if (s->Phone.Data.Memory->MemoryType == MEM_SM) return ERR_EMPTY;
 		return ERR_NOTSUPPORTED;
 	case 0x31:
 		smprintf(s, "Invalid memory type?\n");
@@ -1501,7 +1523,7 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 	case 0x34:
 		smprintf(s, "Too high location ?\n");
 		return ERR_INVALIDLOCATION;
-	case 0x3B: //Tim Dreessen, 6230
+	case 0x3B: /* Tim Dreessen, 6230 */
 		smprintf(s, "Empty location\n");
 		s->Phone.Data.Memory->EntriesNum = 0;
 		return ERR_EMPTY;
@@ -1513,10 +1535,9 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 
 GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	smprintf(s, "Phonebook entry written ");
 	switch (msg.Buffer[6]) {
 	case 0x0f:
-		smprintf(s, " - error\n");
+		smprintf(s, "Phonebook entry writing failed\n");
 		switch (msg.Buffer[10]) {
 		case 0x25:
 			smprintf(s, "when you try to save into entry with caller group assigment in phone with caller groups standard 2 (like in 6230i)\n");
@@ -1544,7 +1565,7 @@ GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine
 			return ERR_UNKNOWNRESPONSE;
 		}
 	default:
-		smprintf(s, " - OK\n");
+		smprintf(s, "Phonebook entry written\n");
 		return ERR_NONE;
 	}
 }
@@ -1741,7 +1762,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x03:
 		smprintf(s, "Call started\n");
-		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		tmp = 6;
 		NOKIA_GetUnicodeString(s, &tmp, msg.Buffer,buffer,false);
 		smprintf(s, "Number     : \"%s\"\n",DecodeUnicodeString(buffer));
@@ -1751,7 +1772,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x04:
 		smprintf(s, "Remote end hang up\n");
-		smprintf(s, "Cause Type : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Cause Type : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		smprintf(s, "CC         : %i\n",msg.Buffer[6]);
 		smprintf(s, "MM(?)      : %i\n",msg.Buffer[7]);
 		smprintf(s, "RR(?)      : %i\n",msg.Buffer[8]);
@@ -1760,7 +1781,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x05:
 		smprintf(s, "Incoming call\n");
-		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		tmp = 6;
 		NOKIA_GetUnicodeString(s, &tmp, msg.Buffer,buffer,false);
 		smprintf(s, "Number     : \"%s\"\n",DecodeUnicodeString(buffer));
@@ -1789,7 +1810,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 				      else smprintf(s, "Audio disabled\n");
 		call.CallIDAvailable = false;
 		break;
-	case 0x0f: //6111
+	case 0x0f: /* 6111 */
 		if (msg.Buffer[8]==0x01) {
 			smprintf(s, "Calling from phone keypad ?\n");
 			if (msg.Buffer[14]==0x03) {
@@ -1832,7 +1853,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x53:
 		smprintf(s, "Outgoing call\n");
-		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		tmp = 6;
 		NOKIA_GetUnicodeString(s, &tmp, msg.Buffer,buffer,false);
 		smprintf(s, "Number     : \"%s\"\n",DecodeUnicodeString(buffer));
@@ -2240,7 +2261,7 @@ GSM_Error N71_65_DelCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 /* method 1 */
 GSM_Error N71_65_ReplyGetCalendarInfo1(GSM_Protocol_Message msg, GSM_StateMachine *s, GSM_NOKIACalToDoLocations *LastCalendar)
 {
-	int i,j=0;
+	size_t i,j=0;
 
 	smprintf(s, "Info with calendar notes locations received method 1\n");
 	while (LastCalendar->Location[j] != 0x00) j++;
@@ -2259,7 +2280,7 @@ GSM_Error N71_65_ReplyGetCalendarInfo1(GSM_Protocol_Message msg, GSM_StateMachin
 		smprintf(s, "%i ",LastCalendar->Location[j-1]);
 		i++;
 	}
-	smprintf(s, "\nNumber of Entries in frame: %i\n",i);
+	smprintf(s, "\nNumber of Entries in frame: " SIZE_T_FORMAT "\n",i);
 	smprintf(s, "\n");
 	LastCalendar->Location[j] = 0;
 	if (i == 1 && msg.Buffer[8+(0*2)]*256+msg.Buffer[9+(0*2)] == 0) return ERR_EMPTY;
@@ -2304,7 +2325,7 @@ GSM_Error N71_65_GetCalendarInfo1(GSM_StateMachine *s, GSM_NOKIACalToDoLocations
 /* method 1 */
 GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	int 			alarm,i;
+	int 			timedelta,i;
 	GSM_CalendarEntry	*entry = s->Phone.Data.Cal;
 
 	smprintf(s, "Calendar note received method 1\n");
@@ -2326,11 +2347,11 @@ GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachin
 		smprintf(s, "Meeting\n");
 		entry->Type = GSM_CAL_MEETING;
 
-		alarm=msg.Buffer[14]*256+msg.Buffer[15];
-		if (alarm != 0xffff) {
-			smprintf(s, "  Difference : %i seconds\n", alarm);
+		timedelta=msg.Buffer[14]*256+msg.Buffer[15];
+		if (timedelta != 0xffff) {
+			smprintf(s, "  Difference : %i seconds\n", timedelta);
 			memcpy(&entry->Entries[1].Date,&entry->Entries[0].Date,sizeof(GSM_DateTime));
-			GetTimeDifference(alarm, &entry->Entries[1].Date, false, 60);
+			GetTimeDifference(timedelta, &entry->Entries[1].Date, false, 60);
 			entry->Entries[1].EntryType = CAL_TONE_ALARM_DATETIME;
 			entry->EntriesNum++;
 		}
@@ -2347,11 +2368,11 @@ GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachin
 		smprintf(s, "Call\n");
 		entry->Type = GSM_CAL_CALL;
 
-		alarm=msg.Buffer[14]*256+msg.Buffer[15];
-		if (alarm != 0xffff) {
-			smprintf(s, "  Difference : %i seconds\n", alarm);
+		timedelta=msg.Buffer[14]*256+msg.Buffer[15];
+		if (timedelta != 0xffff) {
+			smprintf(s, "  Difference : %i seconds\n", timedelta);
 			memcpy(&entry->Entries[1].Date,&entry->Entries[0].Date,sizeof(GSM_DateTime));
-			GetTimeDifference(alarm, &entry->Entries[1].Date, false, 60);
+			GetTimeDifference(timedelta, &entry->Entries[1].Date, false, 60);
 			entry->Entries[1].EntryType = CAL_TONE_ALARM_DATETIME;
 			entry->EntriesNum++;
 		}
@@ -2382,14 +2403,14 @@ GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachin
 		entry->Entries[0].Date.Minute	= 59;
 		entry->Entries[0].Date.Second	= 58;
 
-		alarm  = ((unsigned int)msg.Buffer[14]) << 24;
-		alarm += ((unsigned int)msg.Buffer[15]) << 16;
-		alarm += ((unsigned int)msg.Buffer[16]) << 8;
-		alarm += msg.Buffer[17];
-		if (alarm != 0xffff) {
-			smprintf(s, "  Difference : %i seconds\n", alarm);
+		timedelta  = ((unsigned int)msg.Buffer[14]) << 24;
+		timedelta += ((unsigned int)msg.Buffer[15]) << 16;
+		timedelta += ((unsigned int)msg.Buffer[16]) << 8;
+		timedelta += msg.Buffer[17];
+		if (timedelta != 0xffff) {
+			smprintf(s, "  Difference : %i seconds\n", timedelta);
 			memcpy(&entry->Entries[1].Date,&entry->Entries[0].Date,sizeof(GSM_DateTime));
-			GetTimeDifference(alarm, &entry->Entries[1].Date, false, 1);
+			GetTimeDifference(timedelta, &entry->Entries[1].Date, false, 1);
 			entry->Entries[1].EntryType = CAL_TONE_ALARM_DATETIME;
 			if (msg.Buffer[20]!=0x00) {
 				entry->Entries[1].EntryType = CAL_SILENT_ALARM_DATETIME;
