@@ -36,13 +36,13 @@ static GSM_Error PHONET_WriteMessage (GSM_StateMachine 	*s,
 	buffer2 = (unsigned char *)malloc(MsgLength + 6);
 
 	buffer2[0] = PHONET_FRAME_ID,
-	buffer2[1] = PHONET_DEVICE_PHONE; 		//destination
-	buffer2[2] = PHONET_DEVICE_PC;    		//source
+	buffer2[1] = PHONET_DEVICE_PHONE; 		/* destination */
+	buffer2[2] = PHONET_DEVICE_PC;    		/* source */
 
 	if (s->ConnectionType==GCT_PHONETBLUE || s->ConnectionType==GCT_BLUEPHONET) {
 		buffer2[0] = PHONET_BLUE_FRAME_ID;
-		buffer2[1] = PHONET_DEVICE_PHONE;	//destination
-		buffer2[2] = PHONET_BLUE_DEVICE_PC;	//source
+		buffer2[1] = PHONET_DEVICE_PHONE;	/* destination */
+		buffer2[2] = PHONET_BLUE_DEVICE_PC;	/* source */
 	}
 	if (s->ConnectionType==GCT_DKU2PHONET) {
 		buffer2[0] = PHONET_DKU2_FRAME_ID;
@@ -108,10 +108,8 @@ static GSM_Error PHONET_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 	}
 	if (d->MsgRXState==RX_GetSource) {
 		if (rx_char != PHONET_DEVICE_PHONE) {
-			if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL || s->di.dl==DL_TEXTERROR ||
-			    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE || s->di.dl==DL_TEXTERRORDATE) {
-				smprintf(s,"[ERROR: incorrect char - %02x, not %02x]\n", rx_char, PHONET_DEVICE_PHONE);
-			}
+			smprintf_level(s, D_ERROR, "[ERROR: incorrect char - %02x, not %02x]\n", 
+					rx_char, PHONET_DEVICE_PHONE);
 			d->MsgRXState = RX_Sync;
 			return ERR_NONE;
 		}
@@ -134,10 +132,7 @@ static GSM_Error PHONET_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 			break;
 		}
 		if (!correct) {
-			if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL || s->di.dl==DL_TEXTERROR ||
-			    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE || s->di.dl==DL_TEXTERRORDATE) {
-				smprintf(s,"[ERROR: incorrect char - %02x, not %02x]\n", rx_char, PHONET_DEVICE_PC);
-			}
+			smprintf_level(s, D_ERROR, "[ERROR: incorrect char - %02x, not %02x]\n", rx_char, PHONET_DEVICE_PC);
 			d->MsgRXState = RX_Sync;
 			return ERR_NONE;
 		}
@@ -162,10 +157,7 @@ static GSM_Error PHONET_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 			break;
 		}
 		if (!correct) {
-			if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL || s->di.dl==DL_TEXTERROR ||
-			    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE || s->di.dl==DL_TEXTERRORDATE) {
-				smprintf(s,"[ERROR: incorrect char - %02x, not %02x]\n", rx_char, PHONET_FRAME_ID);
-			}
+			smprintf_level(s, D_ERROR, "[ERROR: incorrect char - %02x, not %02x]\n", rx_char, PHONET_FRAME_ID);
 			return ERR_NONE;
 		}
 		d->Msg.Count  = 0;
