@@ -48,7 +48,7 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 
 	if (frequency != -1 && dow == -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1) {
-			//each day
+			/* each day */
 			Recurrance = 24;
 		}
 	}
@@ -67,21 +67,21 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 	if (frequency != -1 && dow != -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1 &&
 		    entry->Entries[dow].Number == i) {
-			//one week
+			/* one week */
 			Recurrance = 24*7;
 		}
 	}
 	if (frequency != -1 && dow != -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 2 &&
 		    entry->Entries[dow].Number == i) {
-			//two weeks
+			/* two weeks */
 			Recurrance = 24*14;
 		}
 	}
 	if (frequency != -1 && dow == -1 && day != -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1 &&
 		    entry->Entries[day].Number == entry->Entries[start].Date.Day) {
-			//month
+			/* month */
 			Recurrance = 0xffff-1;
 		}
 	}
@@ -89,7 +89,7 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 		if (entry->Entries[frequency].Number == 1 &&
 		    entry->Entries[day].Number == entry->Entries[start].Date.Day &&
 		    entry->Entries[month].Number == entry->Entries[start].Date.Month) {
-			//year
+			/* year */
 			Recurrance = 0xffff;
 		}
 	}
@@ -384,9 +384,9 @@ GSM_Error GSM_Translate_Category (GSM_CatTranslation direction, char *string, GS
 	case TRANSL_TO_GSM:
 		if (strstr(string,"MEETING")) 			*Type = GSM_CAL_MEETING;
 		else if (strstr(string,"REMINDER")) 		*Type = GSM_CAL_REMINDER;
-		else if (strstr(string,"DATE"))	 		*Type = GSM_CAL_REMINDER; //SE
-		else if (strstr(string,"TRAVEL"))	 	*Type = GSM_CAL_TRAVEL;   //SE
-		else if (strstr(string,"VACATION"))	 	*Type = GSM_CAL_VACATION; //SE
+		else if (strstr(string,"DATE"))	 		*Type = GSM_CAL_REMINDER; /* SE */
+		else if (strstr(string,"TRAVEL"))	 	*Type = GSM_CAL_TRAVEL;   /* SE */
+		else if (strstr(string,"VACATION"))	 	*Type = GSM_CAL_VACATION; /* SE */
 		else if (strstr(string,"MISCELLANEOUS"))	*Type = GSM_CAL_MEMO;
 		else if (strstr(string,"PHONE CALL")) 		*Type = GSM_CAL_CALL;
 		else if (strstr(string,"SPECIAL OCCASION")) 	*Type = GSM_CAL_BIRTHDAY;
@@ -457,7 +457,7 @@ GSM_Error GSM_Calendar_GetValue(const GSM_CalendarEntry *note, int *start, const
  * Converts Gammu recurrence to vCal format. See GSM_DecodeVCAL_RRULE
  * for grammar description.
  */
-GSM_Error GSM_EncodeVCAL_RRULE(char *Buffer, int *Length, GSM_CalendarEntry *note, int TimePos UNUSED, GSM_VCalendarVersion Version)
+GSM_Error GSM_EncodeVCAL_RRULE(char *Buffer, size_t *Length, GSM_CalendarEntry *note, int TimePos UNUSED, GSM_VCalendarVersion Version)
 {
 	int i;
 	int j;
@@ -804,12 +804,12 @@ void GSM_ToDo_AdjustDate(GSM_ToDoEntry *note, GSM_DeltaTime *delta)
 	}
 }
 
-GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note, bool header, GSM_VCalendarVersion Version)
+GSM_Error GSM_EncodeVCALENDAR(char *Buffer, size_t *Length, GSM_CalendarEntry *note, bool header, GSM_VCalendarVersion Version)
 {
 	GSM_DateTime 	deltatime;
 	char 		dtstr[20];
 	char		category[100];
-	int		i, alarm = -1, date = -1;
+	int		i, alarm_pos = -1, date_pos = -1;
 
 	/* Write header */
 	if (header) {
@@ -832,7 +832,7 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note
 	for (i=0; i < note->EntriesNum; i++) {
 		switch (note->Entries[i].EntryType) {
 			case CAL_START_DATETIME :
-				date = i;
+				date_pos = i;
 				if (Version == Mozilla_iCalendar && (note->Type == GSM_CAL_MEMO || note->Type == GSM_CAL_BIRTHDAY)) {
 					SaveVCALDate(Buffer, Length, &note->Entries[i].Date, "DTSTART;VALUE=DATE");
 				} else {
@@ -847,14 +847,14 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note
 				}
 				break;
 			case CAL_TONE_ALARM_DATETIME :
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "AALARM");
 				}
 				break;
 			case CAL_SILENT_ALARM_DATETIME:
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "DALARM");
@@ -917,12 +917,12 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, int *Length, GSM_CalendarEntry *note
 			*Length+=sprintf(Buffer+(*Length), "RRULE:YM1%c%c",13,10);
 		}
 	} else {
-		GSM_EncodeVCAL_RRULE(Buffer, Length, note, date, Version);
+		GSM_EncodeVCAL_RRULE(Buffer, Length, note, date_pos, Version);
 	}
 
 	/* Include mozilla specific alarm encoding */
-	if (Version == Mozilla_iCalendar && alarm != -1 && date != -1) {
-		deltatime = VCALTimeDiff(&note->Entries[alarm].Date, &note->Entries[date].Date);
+	if (Version == Mozilla_iCalendar && alarm_pos != -1 && date_pos != -1) {
+		deltatime = VCALTimeDiff(&note->Entries[alarm_pos].Date, &note->Entries[date_pos].Date);
 
 		dtstr[0]='\0';
 		if (deltatime.Minute !=0) {
@@ -988,12 +988,10 @@ void GSM_ToDoFindDefaultTextTimeAlarmCompleted(GSM_ToDoEntry *entry, int *Text,
 	}
 }
 
-GSM_Error GSM_EncodeVTODO(char *Buffer, int *Length, GSM_ToDoEntry *note, bool header, GSM_VToDoVersion Version)
+GSM_Error GSM_EncodeVTODO(char *Buffer, size_t *Length, GSM_ToDoEntry *note, bool header, GSM_VToDoVersion Version)
 {
-	GSM_DateTime 	deltatime;
-	char 		dtstr[20];
 	char		category[100];
-	int		i, alarm = -1, date = -1;
+	int		i, alarm_pos = -1;
 
 	/* Write header */
 	if (header) {
@@ -1040,14 +1038,14 @@ GSM_Error GSM_EncodeVTODO(char *Buffer, int *Length, GSM_ToDoEntry *note, bool h
 				}
 				break;
 			case TODO_ALARM_DATETIME :
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "AALARM");
 				}
 				break;
 			case TODO_SILENT_ALARM_DATETIME:
-				alarm = i;
+				alarm_pos = i;
 				/* Disable alarm for birthday entries. Mozilla would generate an alarm before birth! */
 				if (Version != Mozilla_iCalendar || note->Type != GSM_CAL_BIRTHDAY) {
 					SaveVCALDateTime(Buffer, Length, &note->Entries[i].Date, "DALARM");
@@ -1092,35 +1090,6 @@ GSM_Error GSM_EncodeVTODO(char *Buffer, int *Length, GSM_ToDoEntry *note, bool h
 				/* Not supported */
 				break;
 		}
-
-		/* Include mozilla specific alarm encoding */
-		if (Version == Mozilla_iCalendar && alarm != -1 && date != -1) {
-			deltatime = VCALTimeDiff(&note->Entries[alarm].Date, &note->Entries[date].Date);
-
-			dtstr[0]='\0';
-			if (deltatime.Minute !=0) {
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-UNITS:minutes%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-LENGTH:%i%c%c",
-					deltatime.Minute,13,10);
-				sprintf(dtstr,"-PT%iM",deltatime.Minute);
-			} else if (deltatime.Hour !=0) {
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-UNITS:hours%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-LENGTH:%i%c%c",
-					deltatime.Hour,13,10);
-				sprintf(dtstr,"-PT%iH",deltatime.Hour);
-			} else if (deltatime.Day !=0) {
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-UNITS:days%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "X-MOZILLA-ALARM-DEFAULT-LENGTH:%i%c%c",
-					deltatime.Day,13,10);
-				sprintf(dtstr,"-P%iD",deltatime.Day);
-			}
-			if (dtstr[0] != '\0') {
-				*Length+=sprintf(Buffer+(*Length), "BEGIN:VALARM%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), "TRIGGER;VALUE=DURATION%c%c",13,10);
-				*Length+=sprintf(Buffer+(*Length), " :%s%c%c",dtstr,13,10);
-				*Length+=sprintf(Buffer+(*Length), "END:VALARM%c%c",13,10);
-			}
-		}
 	}
 
 	*Length+=sprintf(Buffer+(*Length), "END:VTODO%c%c",13,10);
@@ -1621,7 +1590,6 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 	trigger.Timezone = -999;
 
 	if (CalVer == Mozilla_iCalendar && *Pos ==0) {
-		int error;
 		error = GSM_Make_VCAL_Lines (Buffer, &lBuffer);
 		if (error != ERR_NONE) return error;
 	}
@@ -1648,9 +1616,17 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			break;
 		case 1: /* Calendar note */
 			if (strstr(Line,"END:VEVENT")) {
-				if (Time == -1) return ERR_UNKNOWN;
+				if (Time == -1) {
+					dbgprintf("vCalendar without date!\n");
+					return ERR_UNKNOWN;
+				}
 				if (rrule != NULL) {
-					error = GSM_DecodeVCAL_RRULE(rrule, Calendar, Time);
+					if (CalVer == Mozilla_iCalendar) {
+						/* @todo: We don't have parser for this right now */
+						error = ERR_NONE;
+					} else { 
+						error = GSM_DecodeVCAL_RRULE(rrule, Calendar, Time);
+					}
 					free(rrule);
 					if (error != ERR_NONE) {
 						return error;
@@ -1694,7 +1670,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			   can defined in several ways. We will use the trigger value only since this is the value
 			   Mozilla calendar uses when importing ics-files. */
 			if (strncmp(Line, "UID:", 4) == 0) {
-				ReadVCALText(Line, "UID", Buff, CalVer == Mozilla_iCalendar);  // Any use for UIDs?
+				ReadVCALText(Line, "UID", Buff, CalVer == Mozilla_iCalendar);  /*  Any use for UIDs? */
 				break;
 			}
 			if (strstr(Line,"X-MOZILLA-ALARM-DEFAULT-UNITS:")) {
@@ -1837,7 +1813,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			}
 
 			if (strncmp(Line, "UID:", 4) == 0) {
-				ReadVCALText(Line, "UID", Buff, ToDoVer == Mozilla_iCalendar);  // Any use for UIDs?
+				ReadVCALText(Line, "UID", Buff, ToDoVer == Mozilla_iCalendar);  /*  Any use for UIDs? */
 				break;
 			}
 			if (strstr(Line,"X-MOZILLA-ALARM-DEFAULT-UNITS:")) {
@@ -1972,7 +1948,7 @@ GSM_Error GSM_DecodeVNOTE(unsigned char *Buffer, int *Pos, GSM_NoteEntry *Note)
 	return ERR_BUG;
 }
 
-GSM_Error GSM_EncodeVNTFile(unsigned char *Buffer, int *Length, GSM_NoteEntry *Note)
+GSM_Error GSM_EncodeVNTFile(unsigned char *Buffer, size_t *Length, GSM_NoteEntry *Note)
 {
 	*Length+=sprintf(Buffer+(*Length), "BEGIN:VNOTE%c%c",13,10);
 	*Length+=sprintf(Buffer+(*Length), "VERSION:1.1%c%c",13,10);
