@@ -19,6 +19,7 @@
 #include <gammu-debug.h>
 
 #include "../misc/coding/coding.h"
+#include "../misc/misc.h"
 #include "../gsmcomon.h"
 #include "gsmmisc.h"
 
@@ -53,7 +54,7 @@ static struct keys_table_position Keys[] = {
 	{' ',0}
 };
 
-GSM_Error MakeKeySequence(char *text, GSM_KeyCode *KeyCode, int *Length)
+GSM_Error MakeKeySequence(char *text, GSM_KeyCode *KeyCode, size_t *Length)
 {
 	int 		i,j;
 	unsigned char 	key;
@@ -105,7 +106,7 @@ GSM_Error GSM_ReadFile(char *FileName, GSM_File *File)
 	if (stat(FileName,&fileinfo) == 0) {
 		File->ModifiedEmpty = false;
 		dbgprintf("File info read correctly\n");
-		//st_mtime is time of last modification of file
+		/* st_mtime is time of last modification of file */
 		Fill_GSM_DateTime(&File->Modified, fileinfo.st_mtime);
 		File->Modified.Year = File->Modified.Year + 1900;
 		dbgprintf("FileTime: %02i-%02i-%04i %02i:%02i:%02i\n",
@@ -185,7 +186,7 @@ void GSM_IdentifyFileFormat(GSM_File *File)
 	}
 }
 
-void SaveVCALDateTime(char *Buffer, int *Length, GSM_DateTime *Date, char *Start)
+void SaveVCALDateTime(char *Buffer, size_t *Length, GSM_DateTime *Date, char *Start)
 {
 	if (Start != NULL) {
 		*Length+=sprintf(Buffer+(*Length), "%s:",Start);
@@ -197,7 +198,7 @@ void SaveVCALDateTime(char *Buffer, int *Length, GSM_DateTime *Date, char *Start
 			13,10);
 }
 
-void SaveVCALDate(char *Buffer, int *Length, GSM_DateTime *Date, char *Start)
+void SaveVCALDate(char *Buffer, size_t *Length, GSM_DateTime *Date, char *Start)
 {
 	if (Start != NULL) {
 		*Length+=sprintf(Buffer+(*Length), "%s:",Start);
@@ -208,7 +209,7 @@ void SaveVCALDate(char *Buffer, int *Length, GSM_DateTime *Date, char *Start)
 
 bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
 {
-	time_t time_t;
+	time_t timestamp;
 	char year[5]="", month[3]="", day[3]="", hour[3]="", minute[3]="", second[3]="";
 
 	memset(dt,0,sizeof(GSM_DateTime));
@@ -246,8 +247,8 @@ bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
 	}
 
 	if (dt->Timezone != 0) {
-		time_t = Fill_Time_T(*dt) + dt->Timezone*3600;
-		Fill_GSM_DateTime(dt, time_t);
+		timestamp = Fill_Time_T(*dt) + dt->Timezone*3600;
+		Fill_GSM_DateTime(dt, timestamp);
 		dt->Year +=1900;
 	}
 
@@ -278,40 +279,36 @@ bool ReadVCALInt(char *Buffer, char *Start, int *Value)
 
 bool ReadVCALDate(char *Buffer, char *Start, GSM_DateTime *Date, bool *is_date_only)
 {
-	unsigned char buff[200];
-	int lstart, lvalue;
+	char fullstart[200];
+	unsigned char datestring[200];
 
-	strcpy(buff,Start);
-	strcat(buff,":");
-	*is_date_only = false;
-	if (strncmp(Buffer,buff,strlen(buff))) {
-		strcpy(buff,Start);
-		strcat(buff,";VALUE=DATE:");
-		if (strncmp(Buffer,buff,strlen(buff)))
+	if (!ReadVCALText(Buffer, Start, datestring, false)) {
+		fullstart[0] = 0;
+		strcat(fullstart, Start);
+		strcat(fullstart, ";VALUE=DATE:");
+		if (!ReadVCALText(Buffer, fullstart, datestring, false)) {
 			return false;
+		}
 		*is_date_only = true;
 	}
 
-	lstart = strlen(buff);
-	lvalue = strlen(Buffer)-lstart;
-	strncpy(buff,Buffer+lstart,lvalue);
-	strncpy(buff+lvalue,"\0",1);
-	if (ReadVCALDateTime(buff,Date)) {
+	if (ReadVCALDateTime(DecodeUnicodeString(datestring), Date)) {
 		dbgprintf("ReadVCALDateTime is \"%04d.%02d.%02d %02d:%02d:%02d\"\n",
 			Date->Year, Date->Month, Date->Day,
 			Date->Hour, Date->Minute, Date->Second);
 		return true;
 	}
+
 	return false;
 }
 
 
-void SaveVCALText(char *Buffer, int *Length, unsigned char *Text, char *Start, bool UTF8)
+void SaveVCALText(char *Buffer, size_t *Length, unsigned char *Text, char *Start, bool UTF8)
 {
 	char buffer[1000];
 
 	if (UnicodeLength(Text) != 0) {
-		if (UTF8) { 
+		if (UTF8) {
 			EncodeUTF8(buffer, Text);
 			*Length += sprintf(Buffer+(*Length), "%s:%s%c%c", Start, DecodeUnicodeString(Text), 13, 10);
 		} else {
@@ -346,14 +343,20 @@ unsigned char *VCALGetTextPart(unsigned char *Buff, int *pos)
 	return tmp;
 }
 
+/**
+ * We separate matching text (Start) to tokens and then try to find all
+ * tokens in Buffer. We also accept tokens like PREF, CHARSET or ENCODING.
+ *
+ * When all tokens are matched we found matching line.
+ */
 bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 {
 	char *line = NULL;
 	char **tokens = NULL;
 	char *charset = NULL;
-	char *begin, *pos, *end;
+	char *begin, *pos, *end, *end2;
 	bool quoted_printable = false;
-	size_t numtokens, token, remtokens;
+	size_t numtokens, token;
 	size_t i, j, len;
 	bool found;
 	bool ret = false;
@@ -403,11 +406,9 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 		goto fail;
 	}
 	/* Advance position */
-	pos += len; 
+	pos += len;
 	/* No need to check this token anymore */
 	tokens[0][0] = 0;
-	/* We've already checked one token */
-	remtokens = numtokens - 1;
 
 	/* Check remaining tokens */
 	while (*pos != ':') {
@@ -425,8 +426,9 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 				continue;
 			}
 			if (strncmp(pos, tokens[token], len) == 0) {
+				dbgprintf("Found %s\n", tokens[token]);
 				/* Advance position */
-				pos += len; 
+				pos += len;
 				/* We need to check one token less */
 				tokens[token][0] = 0;
 				found = true;
@@ -437,20 +439,21 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 			if (strncmp(pos, "ENCODING=QUOTED-PRINTABLE", 25) == 0) {
 				quoted_printable = true;
 				/* Advance position */
-				pos += 25; 
+				pos += 25;
 				found = true;
 			} else if (strncmp(pos, "CHARSET=", 8) == 0) {
-				quoted_printable = true;
 				/* Advance position */
 				pos += 8;
 				/* Grab charset */
-				end = strchr(pos, ';');
-				if (end == NULL) {
-					end = strchr(pos, ':');
-				}
-				if (end == NULL) {
+				end = strchr(pos, ':');
+				end2 = strchr(pos, ';');
+				if (end == NULL && end2 == NULL) {
 					dbgprintf("Could not read charset!\n");
 					goto fail;
+				} else if (end == NULL) {
+					end = end2;
+				} else if (end2 != NULL && end2 < end) {
+					end = end2;
 				}
 				/* We basically want strndup, but it is not portable */
 				charset = strdup(pos);
@@ -460,6 +463,23 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 				}
 				charset[end - pos] = 0;
 
+				pos = end;
+				found = true;
+			} else if (strncmp(pos, "TZID=", 5) == 0) {
+				/* @todo: We ignore time zone for now */
+				/* Advance position */
+				pos += 5;
+				/* Go behind value */
+				end = strchr(pos, ':');
+				end2 = strchr(pos, ';');
+				if (end == NULL && end2 == NULL) {
+					dbgprintf("Could not read timezone!\n");
+					goto fail;
+				} else if (end == NULL) {
+					end = end2;
+				} else if (end2 != NULL && end2 < end) {
+					end = end2;
+				}
 				pos = end;
 				found = true;
 			} else if (strncmp(pos, "TYPE=PREF", 9) == 0) {
@@ -472,7 +492,7 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 				found = true;
 			}
 			if (!found) {
-				dbgprintf("Not found!\n");
+				dbgprintf("%s not found!\n", Start);
 				goto fail;
 			}
 		}
@@ -482,6 +502,14 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 	/* Length of rest */
 	len = strlen(pos);
 
+	/* Did we match all our tokens? */
+	for (token = 0; token < numtokens; token++) {
+		if (strlen(tokens[token]) > 0) {
+			dbgprintf("All tokens did not match!\n");
+			goto fail;
+		}
+	}
+
 	/* Decode the text */
 	if (charset == NULL) {
 		if (quoted_printable) {
@@ -498,13 +526,17 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 			}
 		}
 	} else {
-		if (strcmp(charset, "UTF-8") == 0) {
+		if (strcasecmp(charset, "UTF-8") == 0||
+				strcasecmp(charset, "\"UTF-8\"") == 0
+				) {
 			if (quoted_printable) {
 				DecodeUTF8QuotedPrintable(Value, pos, len);
 			} else {
 				DecodeUTF8(Value, pos, len);
 			}
-		} else if (strcmp(charset, "UTF-7") == 0) {
+		} else if (strcasecmp(charset, "UTF-7") == 0||
+				strcasecmp(charset, "\"UTF-7\"") == 0
+				) {
 			if (quoted_printable) {
 				dbgprintf("Unsupported charset: %s\n", charset);
 				goto fail;
@@ -544,7 +576,7 @@ bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
 	}
 
 	ret = true;
-	dbgprintf("ReadVCalText is \"%s\"\n", DecodeUnicodeConsole(Value));
+	dbgprintf("ReadVCalText(%s) is \"%s\"\n", Start, DecodeUnicodeConsole(Value));
 fail:
 	free(line);
 	free(tokens);
@@ -554,7 +586,8 @@ fail:
 
 bool GSM_ReadHTTPFile(unsigned char *server, unsigned char *filename, GSM_File *file)
 {
-	int			s, len;
+	socket_type s;
+	int len;
 	struct sockaddr_in 	address;
 	struct hostent 		*address2;
 	unsigned char 		buff[200];
@@ -565,11 +598,7 @@ bool GSM_ReadHTTPFile(unsigned char *server, unsigned char *filename, GSM_File *
 #endif
 
 	s = socket(AF_INET,SOCK_STREAM,0);
-#ifdef WIN32
-	if (s==INVALID_SOCKET) return false;
-#else
-	if (s==-1) return false;
-#endif
+	if (s == socket_invalid) return false;
 
 	address2 = gethostbyname(server);
 	if (address2 == NULL) return false;
