@@ -260,6 +260,7 @@ unsigned char *GSM_Networks[] = {
 	"230 03", "Vodafone CZ",
 	"231 01", "Orange SK",
 	"231 02", "T-Mobile SK",
+	"231 06", "O2 SK",
 	"232 01", "A1",
 	"232 03", "T-Mobile AT",
 	"232 05", "ONE",
@@ -541,7 +542,7 @@ unsigned char *GSM_Networks[] = {
 	"437 01", "Bitel",
 	"438 01", "BCTI",
 	"440 10", "NTT DoCoMo, Inc",
-	"440 20", "J-Phone",
+	"440 20", "SoftBank",
 	"450 02", "KT ICOM",
 	"452 01", "MOBIFONE",
 	"452 02", "Vinaphone",
@@ -767,14 +768,22 @@ char *GSM_GetNetworkName(char *NetworkCode)
 {
 	int		i = 0;
 	static char	retval[200];
-
+	char		NetworkCodeFull[7]="      ";
+	
 	EncodeUnicode(retval,"unknown",7);
-	while (GSM_Networks[i*2] != NULL) {
-		if (!strncmp(GSM_Networks[i*2],NetworkCode,6)) {
-			EncodeUnicode(retval, GSM_Networks[i*2+1], strlen(GSM_Networks[i*2+1]));
-			break;
+	if ((strlen(NetworkCode) == 5) || 
+			(strlen(NetworkCode) == 6)) {
+		strncpy(NetworkCodeFull, NetworkCode, 3);
+		strncpy(NetworkCodeFull + 4, NetworkCode + strlen(NetworkCode) - 2, 2);
+		while (GSM_Networks[i * 2] != NULL) {
+			if (strncmp(GSM_Networks[i * 2], NetworkCodeFull, 6) == 0) {
+				EncodeUnicode(retval, 
+						GSM_Networks[i * 2 + 1], 
+						strlen(GSM_Networks[i * 2 + 1]));
+				break;
+			}
+			i++;
 		}
-		i++;
 	}
 	return retval;
 }
