@@ -186,9 +186,53 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 #endif
 }
 
+GSM_Coding_Type GSM_GetMessageCoding(const char TPDCS) {
+
+	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
+	if ((TPDCS & 0xC0) == 0) {
+		/* bits 7..4 set to 00xx */
+		if ((TPDCS & 0xC) == 0xC) {
+			dbgprintf("WARNING: reserved alphabet value in TPDCS\n");
+		} else {
+			if (TPDCS == 0) 			return SMS_Coding_Default_No_Compression;
+			if ((TPDCS & 0x2C) == 0x00) 	return SMS_Coding_Default_No_Compression;
+			if ((TPDCS & 0x2C) == 0x20) 	return SMS_Coding_Default_Compression;
+			if ((TPDCS & 0x2C) == 0x08) 	return SMS_Coding_Unicode_No_Compression;
+			if ((TPDCS & 0x2C) == 0x28) 	return SMS_Coding_Unicode_Compression;
+		}
+	} else if ((TPDCS & 0xF0) >= 0x40 &&
+		   (TPDCS & 0xF0) <= 0xB0) {
+		/* bits 7..4 set to 0100 ... 1011 */
+		dbgprintf("WARNING: reserved coding group in TPDCS\n");
+	} else if (((TPDCS & 0xF0) == 0xC0) ||
+	      	   ((TPDCS & 0xF0) == 0xD0)) {
+		/* bits 7..4 set to 1100 or 1101 */
+		if ((TPDCS & 4) == 4) {
+			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
+		} else {
+			return SMS_Coding_Default_No_Compression;
+		}
+	} else if ((TPDCS & 0xF0) == 0xE0) {
+		/* bits 7..4 set to 1110 */
+		if ((TPDCS & 4) == 4) {
+			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
+		} else {
+			return SMS_Coding_Unicode_No_Compression;
+		}
+	} else if ((TPDCS & 0xF0) == 0xF0) {
+		/* bits 7..4 set to 1111 */
+		if ((TPDCS & 8) == 8) {
+			dbgprintf("WARNING: set reserved bit 3 in TPDCS\n");
+		} else {
+			if ((TPDCS & 4) == 0) return SMS_Coding_Default_No_Compression;
+		}
+	}
+	return 0;
+}
+
 GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
-	int		off=0;	 	// length of the User Data Header
+	int		off=0;	 	/* length of the User Data Header */
 	int 		w,i,tmp=0;
 	unsigned char	output[161];
 
@@ -206,46 +250,6 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 		GSM_DecodeUDHHeader(&SMS->UDH);
 	}
 
-	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
-	if ((buffer[Layout.TPDCS] & 0xC0) == 0) {
-		/* bits 7..4 set to 00xx */
-		if ((buffer[Layout.TPDCS] & 0xC) == 0xC) {
-			dbgprintf("WARNING: reserved alphabet value in TPDCS\n");
-		} else {
-			if (buffer[Layout.TPDCS] == 0) 			SMS->Coding=SMS_Coding_Default_No_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x00) 	SMS->Coding=SMS_Coding_Default_No_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x20) 	SMS->Coding=SMS_Coding_Default_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x08) 	SMS->Coding=SMS_Coding_Unicode_No_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x28) 	SMS->Coding=SMS_Coding_Unicode_Compression;
-		}
-	} else if ((buffer[Layout.TPDCS] & 0xF0) >= 0x40 &&
-		   (buffer[Layout.TPDCS] & 0xF0) <= 0xB0) {
-		/* bits 7..4 set to 0100 ... 1011 */
-		dbgprintf("WARNING: reserved coding group in TPDCS\n");
-	} else if (((buffer[Layout.TPDCS] & 0xF0) == 0xC0) ||
-	      	   ((buffer[Layout.TPDCS] & 0xF0) == 0xD0)) {
-		/* bits 7..4 set to 1100 or 1101 */
-		if ((buffer[Layout.TPDCS] & 4) == 4) {
-			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
-		} else {
-			SMS->Coding=SMS_Coding_Default_No_Compression;
-		}
-	} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xE0) {
-		/* bits 7..4 set to 1110 */
-		if ((buffer[Layout.TPDCS] & 4) == 4) {
-			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
-		} else {
-			SMS->Coding=SMS_Coding_Unicode_No_Compression;
-		}
-	} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xF0) {
-		/* bits 7..4 set to 1111 */
-		if ((buffer[Layout.TPDCS] & 8) == 8) {
-			dbgprintf("WARNING: set reserved bit 3 in TPDCS\n");
-		} else {
-			if ((buffer[Layout.TPDCS] & 4) == 0) SMS->Coding=SMS_Coding_Default_No_Compression;
-		}
-	}
-
 	switch (SMS->Coding) {
 		case SMS_Coding_Default_No_Compression:
 			i = 0;
@@ -364,27 +368,13 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	if (Layout.TPPID     != 255) dbgprintf("TPPID     : %02x %i\n",buffer[Layout.TPPID]    ,buffer[Layout.TPPID]);
 	if (Layout.TPUDL     != 255) dbgprintf("TPUDL     : %02x %i\n",buffer[Layout.TPUDL]    ,buffer[Layout.TPUDL]);
 	if (Layout.firstbyte != 255) dbgprintf("FirstByte : %02x %i\n",buffer[Layout.firstbyte],buffer[Layout.firstbyte]);
-	if (Layout.Text      != 255) dbgprintf("Text      : %02x %i\n",buffer[Layout.Text]     ,buffer[Layout.Text]);
+	if (Layout.Text      != 255 && Layout.TPUDL     != 255 && buffer[Layout.TPUDL] > 0) {
+		dbgprintf("Text      : %02x %i\n",buffer[Layout.Text]     ,buffer[Layout.Text]);
+	}
 #endif
 
-	SMS->UDH.Type 			= UDH_NoUDH;
-	SMS->Coding 			= SMS_Coding_8bit;
-	SMS->Length			= 0;
-	SMS->SMSC.Location		= 0;
-	SMS->SMSC.DefaultNumber[0]	= 0;
-	SMS->SMSC.DefaultNumber[1]	= 0;
-	SMS->SMSC.Number[0]		= 0;
-	SMS->SMSC.Number[1]		= 0;
-	SMS->SMSC.Name[0]		= 0;
-	SMS->SMSC.Name[1]		= 0;
-	SMS->SMSC.Validity.Format	= SMS_Validity_NotAvailable;
-	SMS->SMSC.Format		= SMS_FORMAT_Text;
-	SMS->Number[0]			= 0;
-	SMS->Number[1]			= 0;
-	SMS->OtherNumbersNum		= 0;
-	SMS->Name[0]			= 0;
-	SMS->Name[1]			= 0;
-	SMS->ReplyViaSameSMSC		= false;
+	GSM_SetDefaultReceivedSMSData(SMS);
+
 	if (Layout.SMSCNumber!=255) {
 		GSM_UnpackSemiOctetNumber(SMS->SMSC.Number,buffer+Layout.SMSCNumber,false);
 		dbgprintf("SMS center number : \"%s\"\n",DecodeUnicodeString(SMS->SMSC.Number));
@@ -397,7 +387,11 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 		GSM_UnpackSemiOctetNumber(SMS->Number,buffer+Layout.Number,true);
 		dbgprintf("Remote number : \"%s\"\n",DecodeUnicodeString(SMS->Number));
 	}
-	if (Layout.Text != 255 && Layout.TPDCS!=255 && Layout.TPUDL!=255) {
+	if (Layout.TPDCS != 255) {
+		/* Get message coding */
+		SMS->Coding = GSM_GetMessageCoding(buffer[Layout.TPDCS]);
+	}
+	if (Layout.Text != 255 && Layout.TPDCS!=255 && Layout.TPUDL!=255 && buffer[Layout.TPUDL] > 0) {
 		GSM_DecodeSMSFrameText(SMS, buffer, Layout);
 	}
 	if (Layout.DateTime != 255) {
@@ -480,8 +474,9 @@ static GSM_Error GSM_EncodeSMSDateTime(GSM_DateTime *DT, unsigned char *req)
 
 static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
-	int	off = 0;	// length of the User Data Header
-	int	size = 0, size2 = 0, w,p;
+	int	off = 0;	/*  length of the User Data Header */
+	int	size = 0, size2 = 0, w;
+	size_t p;
 	char	buff[200];
 
 	if (SMS->UDH.Type!=UDH_NoUDH) {
@@ -626,7 +621,7 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 
 	/* size is the length of the data in octets including UDH */
 	*length=GSM_EncodeSMSFrameText(SMS,buffer,Layout);
-//	if (*length == 0) return GE_UNKNOWN;
+/* 	if (*length == 0) return GE_UNKNOWN; */
 	*length += Layout.Text;
 
 	return ERR_NONE;
@@ -634,40 +629,49 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 
 /* ----------------- Some help functions ----------------------------------- */
 
-void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
+void GSM_SetDefaultReceivedSMSData(GSM_SMSMessage *SMS)
 {
-	SMS->Class			= -1;
-	SMS->SMSC.Location		= 1;
-	SMS->SMSC.Name[0]		= 0;
-	SMS->SMSC.Name[1]		= 0;
-	SMS->SMSC.Number[0]		= 0;
-	SMS->SMSC.Number[1]		= 0;
-	SMS->SMSC.DefaultNumber[0]	= 0;
-	SMS->SMSC.DefaultNumber[1]	= 0;
-	SMS->SMSC.Format		= SMS_FORMAT_Text;
-	SMS->SMSC.Validity.Format	= SMS_Validity_RelativeFormat;
-	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
-	SMS->ReplyViaSameSMSC		= false;
-	SMS->UDH.Type			= UDH_NoUDH;
+	SMS->UDH.Type 			= UDH_NoUDH;
 	SMS->UDH.Length			= 0;
 	SMS->UDH.Text[0] 		= 0;
 	SMS->UDH.ID8bit			= 0;
 	SMS->UDH.ID16bit		= 0;
 	SMS->UDH.PartNumber		= 0;
 	SMS->UDH.AllParts		= 0;
-	SMS->Coding			= SMS_Coding_8bit;
+	SMS->Coding 			= SMS_Coding_8bit;
+	SMS->Length			= 0;
+	SMS->SMSC.Location		= 0;
+	SMS->SMSC.DefaultNumber[0]	= 0;
+	SMS->SMSC.DefaultNumber[1]	= 0;
+	SMS->SMSC.Number[0]		= 0;
+	SMS->SMSC.Number[1]		= 0;
+	SMS->SMSC.Name[0]		= 0;
+	SMS->SMSC.Name[1]		= 0;
+	SMS->SMSC.Validity.Format	= SMS_Validity_NotAvailable;
+	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
+	SMS->SMSC.Format		= SMS_FORMAT_Text;
+	SMS->Number[0]			= 0;
+	SMS->Number[1]			= 0;
+	SMS->OtherNumbersNum		= 0;
+	SMS->Name[0]			= 0;
+	SMS->Name[1]			= 0;
+	SMS->ReplyViaSameSMSC		= false;
+	SMS->Class			= 0;
 	SMS->Text[0] 			= 0;
 	SMS->Text[1] 			= 0;
 	SMS->PDU			= SMS_Submit;
 	SMS->RejectDuplicates		= false;
 	SMS->MessageReference		= 0;
 	SMS->ReplaceMessage		= 0;
-	SMS->Length			= 0;
 	SMS->DeliveryStatus		= 0;
-	SMS->ReplyViaSameSMSC		= 0;
-	SMS->Class			= 0;
-	SMS->MessageReference		= 0;
-	SMS->OtherNumbersNum		= 0;
+}
+
+void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
+{
+	GSM_SetDefaultReceivedSMSData(SMS);
+
+	SMS->SMSC.Validity.Format	= SMS_Validity_RelativeFormat;
+	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
 
 	/* This part is required to save SMS */
 	SMS->State			= SMS_UnSent;
@@ -677,8 +681,6 @@ void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
 	SMS->InboxFolder		= false;
 	GSM_GetCurrentDateTime (&SMS->DateTime);
 	GSM_GetCurrentDateTime (&SMS->SMSCTime);
-	SMS->Name[0]			= 0;
-	SMS->Name[1]			= 0;
 }
 
 /**
@@ -746,8 +748,8 @@ bool GSM_DecodeSiemensOTASMS(GSM_SiemensOTASMSInfo	*Info,
 	if (SMS->UDH.Type != UDH_NoUDH) 	return false;
 	if (SMS->Length < 22) 			return false;
 
-	if (strncmp(SMS->Text,"//SEO",5)!=0) return false; //Siemens Exchange Object
-	if (SMS->Text[5]!=1) return false; //version 1
+	if (strncmp(SMS->Text,"//SEO",5)!=0) return false; /* Siemens Exchange Object */
+	if (SMS->Text[5]!=1) return false; /* version 1 */
 	Info->DataLen = SMS->Text[6] + SMS->Text[7]*256;
 	Info->SequenceID = SMS->Text[8] + SMS->Text[9]*256 +
 			 SMS->Text[10]*256*256 + SMS->Text[11]*256*256*256;
