@@ -6,7 +6,6 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include <unistd.h>
 #include <gammu.h>
 
 #include "common.h"
@@ -231,7 +230,7 @@ void PrintCalendar(GSM_CalendarEntry * Note)
 			case CAL_CONTACTID:
 				entry.Location = Note->Entries[i].Number;
 				entry.MemoryType = MEM_ME;
-				error = GSM_GetMemory(s, &entry);
+				error = GSM_GetMemory(gsm, &entry);
 				if (error == ERR_NONE) {
 					name =
 					    GSM_PhonebookGetEntryName(&entry);
@@ -371,7 +370,7 @@ void GetDateTime(int argc UNUSED, char *argv[]UNUSED)
 
 	GSM_Init(true);
 
-	error = GSM_GetDateTime(s, &date_time);
+	error = GSM_GetDateTime(gsm, &date_time);
 	switch (error) {
 		case ERR_EMPTY:
 			printf("%s\n", _("Date and time not set in phone"));
@@ -384,7 +383,7 @@ void GetDateTime(int argc UNUSED, char *argv[]UNUSED)
 			Print_Error(error);
 	}
 
-	error = GSM_GetLocale(s, &locale);
+	error = GSM_GetLocale(gsm, &locale);
 	switch (error) {
 		case ERR_NOTSUPPORTED:
 		case ERR_NOTIMPLEMENTED:
@@ -450,7 +449,7 @@ void SetDateTime(int argc, char *argv[])
 	} else {
 		/* update only parts the user specified,
 		   leave the rest in the phone as is */
-		error = GSM_GetDateTime(s, &date_time);
+		error = GSM_GetDateTime(gsm, &date_time);
 		Print_Error(error);
 
 		if (error == ERR_NONE) {
@@ -485,7 +484,7 @@ void SetDateTime(int argc, char *argv[])
 		}
 	}
 	if (error == ERR_NONE) {
-		error = GSM_SetDateTime(s, &date_time);
+		error = GSM_SetDateTime(gsm, &date_time);
 	}
 	Print_Error(error);
 
@@ -494,36 +493,36 @@ void SetDateTime(int argc, char *argv[])
 
 void GetAlarm(int argc, char *argv[])
 {
-	GSM_Alarm alarm;
+	GSM_Alarm Alarm;
 	GSM_Error error;
 
 	GSM_Init(true);
 
 	if (argc < 3) {
-		alarm.Location = 1;
+		Alarm.Location = 1;
 	} else {
-		alarm.Location = atoi(argv[2]);
+		Alarm.Location = atoi(argv[2]);
 	}
-	error = GSM_GetAlarm(s, &alarm);
+	error = GSM_GetAlarm(gsm, &Alarm);
 	switch (error) {
 		case ERR_EMPTY:
 			printf(_("Alarm (%i) not set in phone\n"),
-			       alarm.Location);
+			       Alarm.Location);
 			break;
 		case ERR_NONE:
-			printf(_("Alarm in location %i:\n"), alarm.Location);
-			if (alarm.Repeating) {
+			printf(_("Alarm in location %i:\n"), Alarm.Location);
+			if (Alarm.Repeating) {
 				printf(LISTFORMAT "%s\n", _("Date"),
 				       _("Every day"));
-			} else if (alarm.DateTime.Day != 0) {
+			} else if (Alarm.DateTime.Day != 0) {
 				printf(LISTFORMAT "%s\n", _("Date"),
-				       OSDate(alarm.DateTime));
+				       OSDate(Alarm.DateTime));
 			}
-			printf(_("Time: %02d:%02d\n"), alarm.DateTime.Hour,
-			       alarm.DateTime.Minute);
-			if (alarm.Text[0] != 0 || alarm.Text[1] != 0) {
+			printf(_("Time: %02d:%02d\n"), Alarm.DateTime.Hour,
+			       Alarm.DateTime.Minute);
+			if (Alarm.Text[0] != 0 || Alarm.Text[1] != 0) {
 				printf(LISTFORMAT "\"%s\"\n", _("Text"),
-				       DecodeUnicodeConsole(alarm.Text));
+				       DecodeUnicodeConsole(Alarm.Text));
 			}
 			break;
 		default:
@@ -535,20 +534,20 @@ void GetAlarm(int argc, char *argv[])
 
 void SetAlarm(int argc UNUSED, char *argv[])
 {
-	GSM_Alarm alarm;
+	GSM_Alarm Alarm;
 	GSM_Error error;
 
-	alarm.DateTime.Hour = atoi(argv[2]);
-	alarm.DateTime.Minute = atoi(argv[3]);
-	alarm.DateTime.Second = 0;
-	alarm.Location = 1;
-	alarm.Repeating = true;
-	alarm.Text[0] = 0;
-	alarm.Text[1] = 0;
+	Alarm.DateTime.Hour = atoi(argv[2]);
+	Alarm.DateTime.Minute = atoi(argv[3]);
+	Alarm.DateTime.Second = 0;
+	Alarm.Location = 1;
+	Alarm.Repeating = true;
+	Alarm.Text[0] = 0;
+	Alarm.Text[1] = 0;
 
 	GSM_Init(true);
 
-	error = GSM_SetAlarm(s, &alarm);
+	error = GSM_SetAlarm(gsm, &Alarm);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -566,7 +565,7 @@ void GetCalendar(int argc UNUSED, char *argv[])
 
 	for (i = start; i <= stop; i++) {
 		Note.Location = i;
-		error = GSM_GetCalendar(s, &Note);
+		error = GSM_GetCalendar(gsm, &Note);
 		if (error == ERR_EMPTY)
 			continue;
 		Print_Error(error);
@@ -589,7 +588,7 @@ void DeleteCalendar(int argc, char *argv[])
 
 	for (i = start; i <= stop; i++) {
 		Note.Location = i;
-		error = GSM_DeleteCalendar(s, &Note);
+		error = GSM_DeleteCalendar(gsm, &Note);
 		Print_Error(error);
 	}
 
@@ -608,7 +607,7 @@ void GetAllCalendar(int argc UNUSED, char *argv[]UNUSED)
 	GSM_Init(true);
 
 	while (!gshutdown) {
-		error = GSM_GetNextCalendar(s, &Note, refresh);
+		error = GSM_GetNextCalendar(gsm, &Note, refresh);
 		if (error == ERR_EMPTY)
 			break;
 		Print_Error(error);
@@ -627,7 +626,7 @@ void GetCalendarSettings(int argc UNUSED, char *argv[]UNUSED)
 
 	GSM_Init(true);
 
-	error = GSM_GetCalendarSettings(s, &settings);
+	error = GSM_GetCalendarSettings(gsm, &settings);
 	Print_Error(error);
 
 	if (settings.AutoDelete == 0) {
@@ -657,7 +656,7 @@ void DeleteToDo(int argc, char *argv[])
 	for (i = start; i <= stop; i++) {
 		ToDo.Location = i;
 		printf(LISTFORMAT "%i\n", _("Location"), i);
-		error = GSM_DeleteToDo(s, &ToDo);
+		error = GSM_DeleteToDo(gsm, &ToDo);
 		if (error != ERR_EMPTY)
 			Print_Error(error);
 
@@ -832,7 +831,7 @@ void PrintToDo(GSM_ToDoEntry * ToDo)
 			case TODO_CATEGORY:
 				Category.Location = ToDo->Entries[j].Number;
 				Category.Type = Category_ToDo;
-				error = GSM_GetCategory(s, &Category);
+				error = GSM_GetCategory(gsm, &Category);
 				if (error == ERR_NONE) {
 					printf(LISTFORMAT "\"%s\" (%i)\n",
 					       _("Category"),
@@ -847,7 +846,7 @@ void PrintToDo(GSM_ToDoEntry * ToDo)
 			case TODO_CONTACTID:
 				entry.Location = ToDo->Entries[j].Number;
 				entry.MemoryType = MEM_ME;
-				error = GSM_GetMemory(s, &entry);
+				error = GSM_GetMemory(gsm, &entry);
 				if (error == ERR_NONE) {
 					name =
 					    GSM_PhonebookGetEntryName(&entry);
@@ -901,7 +900,7 @@ void ListToDoCategoryEntries(int Category)
 	int j;
 
 	while (!gshutdown) {
-		error = GSM_GetNextToDo(s, &Entry, start);
+		error = GSM_GetNextToDo(gsm, &Entry, start);
 		if (error == ERR_EMPTY)
 			break;
 		Print_Error(error);
@@ -956,10 +955,10 @@ void ListToDoCategory(int argc UNUSED, char *argv[])
 		Category.Type = Category_ToDo;
 		Status.Type = Category_ToDo;
 
-		if (GSM_GetCategoryStatus(s, &Status) == ERR_NONE) {
+		if (GSM_GetCategoryStatus(gsm, &Status) == ERR_NONE) {
 			for (count = 0, j = 1; count < Status.Used; j++) {
 				Category.Location = j;
-				error = GSM_GetCategory(s, &Category);
+				error = GSM_GetCategory(gsm, &Category);
 
 				if (error != ERR_EMPTY) {
 					count++;
@@ -987,7 +986,7 @@ void GetToDo(int argc, char *argv[])
 
 	for (i = start; i <= stop; i++) {
 		ToDo.Location = i;
-		error = GSM_GetToDo(s, &ToDo);
+		error = GSM_GetToDo(gsm, &ToDo);
 		if (error == ERR_EMPTY)
 			continue;
 		Print_Error(error);
@@ -1009,7 +1008,7 @@ void GetAllToDo(int argc UNUSED, char *argv[]UNUSED)
 	GSM_Init(true);
 
 	while (!gshutdown) {
-		error = GSM_GetNextToDo(s, &ToDo, start);
+		error = GSM_GetNextToDo(gsm, &ToDo, start);
 		if (error == ERR_EMPTY)
 			break;
 		Print_Error(error);
@@ -1032,7 +1031,7 @@ void GetAllNotes(int argc UNUSED, char *argv[]UNUSED)
 	GSM_Init(true);
 
 	while (!gshutdown) {
-		error = GSM_GetNextNote(s, &Note, start);
+		error = GSM_GetNextNote(gsm, &Note, start);
 		if (error == ERR_EMPTY)
 			break;
 		Print_Error(error);
