@@ -19,24 +19,24 @@ extern GSM_Reply_Function UserReplyFunctions3[];
 
 GSM_Error CheckDCT3Only()
 {
-	bool found = false;
+bool found = false;
 
-	/* Checking if phone is DCT3 */
+/* Checking if phone is DCT3 */
 #ifdef GSM_ENABLE_NOKIA6110
-	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
+if (strstr(N6110Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA7110
-	if (strstr(N7110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
+	if (strstr(N7110Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA9210
-	if (strstr(N9210Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
+	if (strstr(N9210Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 	if (!found) return ERR_NOTSUPPORTED;
 
-	if (s->ConnectionType!=GCT_MBUS2	    && s->ConnectionType!=GCT_FBUS2      &&
-	    s->ConnectionType!=GCT_FBUS2DLR3 && s->ConnectionType!=GCT_FBUS2BLUE  &&
-	    s->ConnectionType!=GCT_FBUS2IRDA && s->ConnectionType!=GCT_IRDAPHONET &&
-	    s->ConnectionType!=GCT_BLUEFBUS2) {
+	if (gsm->ConnectionType!=GCT_MBUS2	    && gsm->ConnectionType!=GCT_FBUS2      &&
+	    gsm->ConnectionType!=GCT_FBUS2DLR3 && gsm->ConnectionType!=GCT_FBUS2BLUE  &&
+	    gsm->ConnectionType!=GCT_FBUS2IRDA && gsm->ConnectionType!=GCT_IRDAPHONET &&
+	    gsm->ConnectionType!=GCT_BLUEFBUS2) {
 		return ERR_OTHERCONNECTIONREQUIRED;
 	}
 	return ERR_NONE;
@@ -53,7 +53,7 @@ static void CheckDCT3()
 		break;
 	case ERR_OTHERCONNECTIONREQUIRED:
 		printf("Can't do it with current phone protocol\n");
-		GSM_TerminateConnection(s);
+		GSM_TerminateConnection(gsm);
 		exit(-1);
 	default:
 		break;
@@ -78,13 +78,15 @@ static bool answer_yes3(char *text)
 
 static FILE *DCT3T9File;
 
-static GSM_Error DCT3_ReplyGetT9(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetT9(GSM_Protocol_Message msg, GSM_StateMachine *sm UNUSED)
 {
-	int DCT3T9Size;
+	size_t DCT3T9Size;
+	size_t written;
 
 	DCT3T9Size = msg.Length - 6;
-	fwrite(msg.Buffer+6,1,DCT3T9Size,DCT3T9File);
-	return ERR_NONE;
+	written = fwrite(msg.Buffer+6,1,DCT3T9Size,DCT3T9File);
+	if (written == DCT3T9Size) return ERR_NONE;
+	return ERR_UNKNOWN;
 }
 
 void DCT3GetT9(int argc, char *argv[])
@@ -94,18 +96,18 @@ void DCT3GetT9(int argc, char *argv[])
 	unsigned char req[] = {0x00, 0x01, 0xAE, 0x02, 0x00,
 			       0x00};	/* Part number */
 
-//"00 01 AE 00" gets some control values
+/* "00 01 AE 00" gets some control values */
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
 	DCT3T9File = fopen("T9", "w");
 	if (DCT3T9File == NULL) return;
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
 	for (i=0;i<5;i++) {
 		req[5] = i;
-		error=GSM_WaitFor (s, req, 6, 0x40, 4, ID_User3);
+		error=GSM_WaitFor (gsm, req, 6, 0x40, 4, ID_User3);
 		Print_Error(error);
 	}
 
@@ -121,22 +123,22 @@ void DCT3VibraTest(int argc, char *argv[])
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	error=GSM_WaitFor (s, SetLevel, 4, 0x40, 4, ID_User3);
+	error=GSM_WaitFor (gsm, SetLevel, 4, 0x40, 4, ID_User3);
 	Print_Error(error);
 
 	printf("Press any key to continue...\n");
 	GetLine(stdin, ans, 99);
 
 	SetLevel[3] = 0x00;
-	error=GSM_WaitFor (s, SetLevel, 4, 0x40, 4, ID_User3);
+	error=GSM_WaitFor (gsm, SetLevel, 4, 0x40, 4, ID_User3);
 }
 
-static GSM_Error DCT3_ReplyPhoneTests(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyPhoneTests(GSM_Protocol_Message msg, GSM_StateMachine *sm UNUSED)
 {
 	int i;
 
@@ -188,12 +190,12 @@ void DCT3SelfTests(int argc, char *argv[])
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
 	if (answer_yes3("Run all tests now ?")) {
 		/* make almost all tests */
-		error = s->Protocol.Functions->WriteMessage(s, buffer3, 8, 0x40);
+		error = gsm->Protocol.Functions->WriteMessage(gsm, buffer3, 8, 0x40);
 		Print_Error(error);
 
 		GSM_Terminate();
@@ -207,10 +209,10 @@ void DCT3SelfTests(int argc, char *argv[])
 		my_sleep(400);
 	}
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
 	for (i=0;i<10;i++) {
-		error=GSM_WaitFor (s, buffer, 3, 0x40, 4, ID_User1);
+		error=GSM_WaitFor (gsm, buffer, 3, 0x40, 4, ID_User1);
 		if (error == ERR_NONE) break;
 	}
 }
@@ -225,14 +227,14 @@ struct DCT3ADCInfo {
 
 static struct DCT3ADCInfo DCT3ADC[] = {
 	{"Battery voltage:",			"mV",  1,  3,  2},
-//	{"Charger voltage:",			"mV",  1, -1,  7},
-//	{"Charger current:",			"mA",  1, -1,  5},
+/* 	{"Charger voltage:",			"mV",  1, -1,  7}, */
+/* 	{"Charger current:",			"mA",  1, -1,  5}, */
 	{"Battery type:",			"mAh", 1,  4,  3},
 	{"Battery temperature:",		"mK", 10,  5,  4},
-//	{"Accessory detection:",		"mV",  1, -1, -1},
+/* 	{"Accessory detection:",		"mV",  1, -1, -1}, */
 	{"RSSI:",				"",    1,  2, -1},
-//	{"VCXO temperature:",			"mV",  1, -1, -1},
-//	{"Hook information:",			"mV",  1, -1, -1},
+/* 	{"VCXO temperature:",			"mV",  1, -1, -1}, */
+/* 	{"Hook information:",			"mV",  1, -1, -1}, */
 
 	{"", "", 1, -1, -1}
 };
@@ -240,7 +242,7 @@ static struct DCT3ADCInfo DCT3ADC[] = {
 unsigned char 	DCT3ADCBuf[200];
 int		DCT3ADCInt;
 
-static GSM_Error DCT3_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	switch (msg.Buffer[2]) {
 	case 0x68:
@@ -263,12 +265,12 @@ void DCT3GetADC(int argc, char *argv[])
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=DCT3_EnableSecurity (s, 0x02);
+	error=DCT3_EnableSecurity (gsm, 0x02);
 	Print_Error(error);
 
-	error=GSM_WaitFor (s, GetRaw, 3, 0x40, 6, ID_User3);
+	error=GSM_WaitFor (gsm, GetRaw, 3, 0x40, 6, ID_User3);
 	Print_Error(error);
 
 	while (1) {
@@ -282,7 +284,7 @@ void DCT3GetADC(int argc, char *argv[])
 		if (DCT3ADC[i].pos2 != -1) {
 			printf("unit result ");
 			GetUnit[3] = DCT3ADC[i].pos2;
-			error=GSM_WaitFor (s, GetUnit, 6, 0x40, 4, ID_User3);
+			error=GSM_WaitFor (gsm, GetUnit, 6, 0x40, 4, ID_User3);
 			Print_Error(error);
 			printf("%10i ",DCT3ADCInt*DCT3ADC[i].x);
 			printf("%s\n",DCT3ADC[i].unit);
@@ -291,7 +293,7 @@ void DCT3GetADC(int argc, char *argv[])
 		if (DCT3ADC[i].name[0] == 0x00) break;
 	}
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 }
 
@@ -309,19 +311,19 @@ void DCT3DisplayTest(int argc, char *argv[])
 		printf("Give 1 or 2 as test number\n");
 	}
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
 	req[4] = atoi(argv[2]);
-	s->Protocol.Functions->WriteMessage(s, req, 5, 0x40);
+	gsm->Protocol.Functions->WriteMessage(gsm, req, 5, 0x40);
 
 	printf("Press any key to continue...\n");
 	GetLine(stdin, ans, 99);
 
 	req[3] = 0x02;
 	req[4] = 0x03;
-	s->Protocol.Functions->WriteMessage(s, req, 5, 0x40);
+	gsm->Protocol.Functions->WriteMessage(gsm, req, 5, 0x40);
 
-	error=DCT3_EnableSecurity (s, 0x03);
+	error=DCT3_EnableSecurity (gsm, 0x03);
 	Print_Error(error);
 }
 
@@ -334,7 +336,7 @@ void DCT3netmonitor(int argc, char *argv[])
 
         CheckDCT3();
 
-	error=DCT3_Netmonitor(s, atoi(argv[2]), value);
+	error=DCT3_Netmonitor(gsm, atoi(argv[2]), value);
 	Print_Error(error);
 
 	printf("%s\n",value);
@@ -344,7 +346,7 @@ void DCT3netmonitor(int argc, char *argv[])
 	GSM_Terminate();
 }
 
-static GSM_Error DCT3_ReplyGetMSID(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetMSID(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int i;
 
@@ -354,13 +356,13 @@ static GSM_Error DCT3_ReplyGetMSID(GSM_Protocol_Message msg, GSM_StateMachine *s
 	return ERR_NONE;
 }
 
-static GSM_Error DCT3_ReplyGetDSPROM(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetDSPROM(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("DSP ROM       : %c\n",msg.Buffer[5]);
 	return ERR_NONE;
 }
 
-static GSM_Error DCT3_ReplySimlockInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplySimlockInfo(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int	i, j;
 	char	uni[100], buffer[50];
@@ -414,7 +416,7 @@ static GSM_Error DCT3_ReplySimlockInfo(GSM_Protocol_Message msg, GSM_StateMachin
 	return ERR_NONE;
 }
 
-static GSM_Error DCT3_ReplyGetMCUchkSum(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetMCUchkSum(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int i;
 
@@ -429,9 +431,9 @@ static GSM_Error DCT3_ReplyGetMCUchkSum(GSM_Protocol_Message msg, GSM_StateMachi
 
 static unsigned char MSID1;
 
-GSM_Error DCT3_ReplyEnableSecurity2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplyEnableSecurity2(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
-	smprintf(s, "State of security commands set\n");
+	smprintf(sm, "State of security commands set\n");
 	MSID1 = msg.Buffer[5];
 	return ERR_NONE;
 }
@@ -446,27 +448,27 @@ void DCT3Info(int argc, char *argv[])
 
         if (CheckDCT3Only()!=ERR_NONE) return;
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	error=GSM_WaitFor (s, req, 4, 0x40, 4, ID_User3);
+	error=GSM_WaitFor (gsm, req, 4, 0x40, 4, ID_User3);
 	Print_Error(error);
 
 	req2[3] = MSID1;
 	req2[4] = req2[2] + req2[3];
- 	error=GSM_WaitFor (s, req2, 5, 0x40, 4, ID_User8);
+ 	error=GSM_WaitFor (gsm, req2, 5, 0x40, 4, ID_User8);
 	Print_Error(error);
 
-	error=GSM_WaitFor (s, req3, 4, 0x40, 4, ID_User9);
+	error=GSM_WaitFor (gsm, req3, 4, 0x40, 4, ID_User9);
 	Print_Error(error);
 
-	error=GSM_WaitFor (s, req4, 4, 0x40, 4, ID_User10);
+	error=GSM_WaitFor (gsm, req4, 4, 0x40, 4, ID_User10);
 	Print_Error(error);
 }
 
-static GSM_Error DCT3_ReplyResetTest36(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyResetTest36(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("Netmonitor test 36 cleaned OK\n");
 	return ERR_NONE;
@@ -481,12 +483,12 @@ void DCT3ResetTest36(int argc, char *argv[])
 
         CheckDCT3();
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (s, req, 5, 0x40, 4, ID_User2);
+	error=GSM_WaitFor (gsm, req, 5, 0x40, 4, ID_User2);
 	Print_Error(error);
 
 #ifdef GSM_ENABLE_BEEP
@@ -497,7 +499,7 @@ void DCT3ResetTest36(int argc, char *argv[])
 
 static unsigned char PPS[32]; /* Product Profile Settings */
 
-static GSM_Error DCT3_ReplyGetPPS(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetPPS(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int i,j,z;
 
@@ -523,7 +525,7 @@ static GSM_Error DCT3_ReplyGetPPS(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	return ERR_NONE;
 }
 
-static GSM_Error DCT3_ReplySetPPS(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplySetPPS(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("Setting done OK\n");
 	return ERR_NONE;
@@ -541,36 +543,36 @@ void DCT3SetPhoneMenus(int argc, char *argv[])
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (s, reqGet, 3, 0x40, 4, ID_User4);
+	error=GSM_WaitFor (gsm, reqGet, 3, 0x40, 4, ID_User4);
 	Print_Error(error);
 
 	printf("ALS : enabling menu\n");
 	PPS[10] = '1';
 
-	if (!strcmp(s->Phone.Data.ModelInfo->model,"3310") && s->Phone.Data.VerNum>5.87) {
+	if (!strcmp(gsm->Phone.Data.ModelInfo->model,"3310") && gsm->Phone.Data.VerNum>5.87) {
 		printf("3310: enabling control of SMS charsets\n");
-		PPS[11] = '0';//0 = ON, 1 = OFF
+		PPS[11] = '0';/* 0 = ON, 1 = OFF */
 	}
-	if (!strcmp(s->Phone.Data.ModelInfo->model,"6150")) {
+	if (!strcmp(gsm->Phone.Data.ModelInfo->model,"6150")) {
 		printf("6150: enabling WellMate menu\n");
 		PPS[18] = '1';
 	}
 	/* FIXME */
-	if (!strcmp(s->Phone.Data.ModelInfo->model,"3210")) {
+	if (!strcmp(gsm->Phone.Data.ModelInfo->model,"3210")) {
 		printf("3210: enabling vibra menu\n");
 		PPS[24] = '1';
 	}
-	if (!strcmp(s->Phone.Data.ModelInfo->model,"3310") && s->Phone.Data.VerNum>5.13) {
+	if (!strcmp(gsm->Phone.Data.ModelInfo->model,"3310") && gsm->Phone.Data.VerNum>5.13) {
 		printf("3310: enabling 3315 features\n");
 		PPS[25] = '1';
 	}
 	/* FIXME */
-	if (!strcmp(s->Phone.Data.ModelInfo->model,"3210") && s->Phone.Data.VerNum>=5.31) {
+	if (!strcmp(gsm->Phone.Data.ModelInfo->model,"3210") && gsm->Phone.Data.VerNum>=5.31) {
 		printf("3210: enabling React and Logic game\n");
 		PPS[26] = '1';
 	}
@@ -590,26 +592,26 @@ void DCT3SetPhoneMenus(int argc, char *argv[])
 		} else j=j/2;
 	}
 
-//	reqSet[3]=0xe7;
-//	reqSet[4]=0x25;
-//	reqSet[5]=0x00;
-//	reqSet[6]=0xe0;
+/* 	reqSet[3]=0xe7; */
+/* 	reqSet[4]=0x25; */
+/* 	reqSet[5]=0x00; */
+/* 	reqSet[6]=0xe0; */
 
-	error=GSM_WaitFor (s, reqSet, 7, 0x40, 4, ID_User4);
+	error=GSM_WaitFor (gsm, reqSet, 7, 0x40, 4, ID_User4);
 	Print_Error(error);
 
 	printf("Enabling netmonitor\n");
-	error=DCT3_Netmonitor(s, 243, value);
+	error=DCT3_Netmonitor(gsm, 243, value);
 	Print_Error(error);
 }
 
-static GSM_Error DCT3_Reply61GetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_Reply61GetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("Security Code is \"%s\"\n",msg.Buffer+5);
 	return ERR_NONE;
 }
 
-static GSM_Error DCT3_Reply7191GetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_Reply7191GetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("Security Code is \"%s\"\n",msg.Buffer+6);
 	return ERR_NONE;
@@ -629,24 +631,24 @@ void DCT3GetSecurityCode(int argc, char *argv[])
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
 #ifdef GSM_ENABLE_NOKIA6110
-	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) {
-		error=GSM_WaitFor (s, req6110, 4, 0x40, 4, ID_User6);
+	if (strstr(N6110Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) {
+		error=GSM_WaitFor (gsm, req6110, 4, 0x40, 4, ID_User6);
 	}
 #endif
 #ifdef GSM_ENABLE_NOKIA7110
-	if (strstr(N7110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) {
-		error=GSM_WaitFor (s, req71_91, 5, 0x7a, 4, ID_User6);
+	if (strstr(N7110Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) {
+		error=GSM_WaitFor (gsm, req71_91, 5, 0x7a, 4, ID_User6);
 	}
 #endif
 #ifdef GSM_ENABLE_NOKIA9210
-	if (strstr(N9210Phone.models, s->Phone.Data.ModelInfo->model) != NULL) {
-		error=GSM_WaitFor (s, req71_91, 5, 0x7a, 4, ID_User6);
+	if (strstr(N9210Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) {
+		error=GSM_WaitFor (gsm, req71_91, 5, 0x7a, 4, ID_User6);
 	}
 #endif
 	Print_Error(error);
@@ -654,7 +656,7 @@ void DCT3GetSecurityCode(int argc, char *argv[])
 
 #ifdef GSM_ENABLE_NOKIA6110
 
-static GSM_Error DCT3_ReplyGetOperatorName(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyGetOperatorName(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	unsigned char buffer[10];
 
@@ -674,21 +676,21 @@ void DCT3GetOperatorName(int argc, char *argv[])
 
 	GSM_Init(true);
 
-	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
+	if (strstr(N6110Phone.models, gsm->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
 	CheckDCT3();
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (s, req, 4, 0x40, 4, ID_User5);
+	error=GSM_WaitFor (gsm, req, 4, 0x40, 4, ID_User5);
 	Print_Error(error);
 
 	GSM_Terminate();
 }
 
-static GSM_Error DCT3_ReplySetOperatorName(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplySetOperatorName(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("Operator name set OK\n");
 	return ERR_NONE;
@@ -704,13 +706,13 @@ void DCT3SetOperatorName(int argc, char *argv[])
 
 	GSM_Init(true);
 
-	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
+	if (strstr(N6110Phone.models, gsm->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
 	CheckDCT3();
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
 	switch (argc) {
 	case 2:
@@ -723,13 +725,13 @@ void DCT3SetOperatorName(int argc, char *argv[])
 		i = strlen(argv[3]);
 	}
 
-	error=GSM_WaitFor (s, req, 8+i, 0x40, 4, ID_User7);
+	error=GSM_WaitFor (gsm, req, 8+i, 0x40, 4, ID_User7);
 	Print_Error(error);
 
 	GSM_Terminate();
 }
 
-static GSM_Error DCT3_ReplyDisplayOutput(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyDisplayOutput(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	unsigned char buf[100];
 
@@ -751,17 +753,17 @@ static GSM_Error DCT3_ReplyDisplayOutput(GSM_Protocol_Message msg, GSM_StateMach
 void DCT3DisplayOutput(int argc, char *argv[])
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x53,
-			       0x01}; //1 = enable, 2 = disable
+			       0x01}; /* 1 = enable, 2 = disable */
 	GSM_Error error;
 
 	GSM_Init(true);
 
-	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
+	if (strstr(N6110Phone.models, gsm->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
 	CheckDCT3();
 
-	s->User.UserReplyFunctions=UserReplyFunctions3;
+	gsm->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (s, req, 5, 0x0d, 4, ID_User7);
+	error=GSM_WaitFor (gsm, req, 5, 0x0d, 4, ID_User7);
 	Print_Error(error);
 
 	signal(SIGINT, interrupt);
@@ -769,12 +771,12 @@ void DCT3DisplayOutput(int argc, char *argv[])
 	printf("Entering monitor mode...\n\n");
 
 	while (!gshutdown) {
-		GSM_ReadDevice(s,true);
+		GSM_ReadDevice(gsm,true);
 		my_sleep(10);
 	}
 
 	req[4] = 0x02;
-	error=GSM_WaitFor (s, req, 5, 0x0d, 4, ID_User7);
+	error=GSM_WaitFor (gsm, req, 5, 0x0d, 4, ID_User7);
 	Print_Error(error);
 
 	GSM_Terminate();
