@@ -14,42 +14,43 @@
 #include "dsiemens.h"
 #include "chiffre.h"
 
-extern GSM_Error  ATGEN_GetSIMIMSI (GSM_StateMachine *s, char *IMSI);
-extern GSM_Error  ATGEN_GetMemoryStatus (GSM_StateMachine *s, GSM_MemoryStatus *status);
-extern GSM_Error  ATGEN_SetMemory (GSM_StateMachine *s, GSM_MemoryEntry *pbk);
+extern GSM_Error  ATGEN_GetSIMIMSI (GSM_StateMachine *sm, char *IMSI);
+extern GSM_Error  ATGEN_GetMemoryStatus (GSM_StateMachine *sm, GSM_MemoryStatus *status);
+extern GSM_Error  ATGEN_SetMemory (GSM_StateMachine *sm, GSM_MemoryEntry *pbk);
 extern GSM_Reply_Function UserReplyFunctionsAtS[];
 
 bool 	new_variable;
 GSM_Error CheckSiemens()
 {
-	if (s->Phone.Data.Priv.ATGEN.Manufacturer != AT_Siemens) return ERR_NOTSUPPORTED;
+	if (gsm->Phone.Data.Priv.ATGEN.Manufacturer != AT_Siemens) return ERR_NOTSUPPORTED;
 	return ERR_NONE;
 }
 
-GSM_Error ATSIEMENS_Reply_GetSAT(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATSIEMENS_Reply_GetSAT(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
-	GSM_Phone_ATGENData		*Priv = &(s->Phone.Data.Priv.ATGEN);
+	GSM_Phone_ATGENData		*Priv = &(sm->Phone.Data.Priv.ATGEN);
 	GSM_SAT_Measure_results		MeasureResult;
 	unsigned char 			buf[256];
-        int                             length,i,rep,ChNo=1,j=0,result=0,origARFCN=0;
+        int                             length,i,rep,ChNo=1,result=0,origARFCN=0;
+	size_t j=0;
     	int				freq_tmp,frequency[24];
    	GSM_NetworkInfo			Network;
 
     	if (Priv->ReplyState!=AT_Reply_OK) return ERR_UNKNOWN;
-    	if (s->Protocol.Data.AT.EditMode) s->Protocol.Data.AT.EditMode = false;
-	if (strstr(GetLineString(msg.Buffer,Priv->Lines,2),"SSTK")) {
-    	    length = strlen(GetLineString(msg.Buffer,Priv->Lines,2))-7;
-    	    DecodeHexBin(buf, GetLineString(msg.Buffer,Priv->Lines,2)+7,length);
+    	if (sm->Protocol.Data.AT.EditMode) sm->Protocol.Data.AT.EditMode = false;
+	if (strstr(GetLineString(msg.Buffer,&Priv->Lines,2),"SSTK")) {
+    	    length = strlen(GetLineString(msg.Buffer,&Priv->Lines,2))-7;
+    	    DecodeHexBin(buf, GetLineString(msg.Buffer,&Priv->Lines,2)+7,length);
 	    if (buf[0]==0x7f) {
 		new_variable=true;
 		return ERR_NONE;
 	    }
 	    else return ERR_UNKNOWN;
 	}
-    	if (!strstr(GetLineString(msg.Buffer,Priv->Lines,3),"SSTK")) return ERR_UNKNOWN;
+    	if (!strstr(GetLineString(msg.Buffer,&Priv->Lines,3),"SSTK")) return ERR_UNKNOWN;
 
-    	length = strlen(GetLineString(msg.Buffer,Priv->Lines,3))-7;
-    	DecodeHexBin(buf, GetLineString(msg.Buffer,Priv->Lines,3)+7,length);
+    	length = strlen(GetLineString(msg.Buffer,&Priv->Lines,3))-7;
+    	DecodeHexBin(buf, GetLineString(msg.Buffer,&Priv->Lines,3)+7,length);
 
     	if (buf[3]!=0x26) return ERR_UNKNOWN;
 
@@ -215,21 +216,21 @@ GSM_Error ATSIEMENS_Reply_GetSAT(GSM_Protocol_Message msg, GSM_StateMachine *s)
     	return ERR_NONE;
 }
 
-GSM_Error ATSIEMENS_Reply_GetNetmon(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATSIEMENS_Reply_GetNetmon(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
-    	GSM_Phone_ATGENData	*Priv = &(s->Phone.Data.Priv.ATGEN);
+    	GSM_Phone_ATGENData	*Priv = &(sm->Phone.Data.Priv.ATGEN);
     	int 			i=2;
 
-    	if (!strstr(GetLineString(msg.Buffer,Priv->Lines,1),"AT^S^MI")) return ERR_UNKNOWN;
-    	while (strlen(GetLineString(msg.Buffer,Priv->Lines,i+1)))
-		printf("%s\n",GetLineString(msg.Buffer,Priv->Lines,i++));
+    	if (!strstr(GetLineString(msg.Buffer,&Priv->Lines,1),"AT^S^MI")) return ERR_UNKNOWN;
+    	while (strlen(GetLineString(msg.Buffer,&Priv->Lines,i+1)))
+		printf("%s\n",GetLineString(msg.Buffer,&Priv->Lines,i++));
     	printf("\n");
     	return ERR_NONE;
 }
 
-GSM_Error ATSIEMENS_GetSAT(GSM_StateMachine *s)
+GSM_Error ATSIEMENS_GetSAT(GSM_StateMachine *sm)
 {
-    	GSM_Phone_ATGENData	*Priv = &(s->Phone.Data.Priv.ATGEN);
+    	GSM_Phone_ATGENData	*Priv = &(sm->Phone.Data.Priv.ATGEN);
     	GSM_Error		error;
     	unsigned char		*reqSAT[]= {"D009810301260082028182",
 				    	    "D009810301260282028182",
@@ -239,39 +240,39 @@ GSM_Error ATSIEMENS_GetSAT(GSM_StateMachine *s)
     	if (Priv->Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
 
         sprintf(req, "AT^SSTK=?\r");
-        error = GSM_WaitFor (s, req, strlen(req), 0x00, 3, ID_User1);
+        error = GSM_WaitFor (sm, req, strlen(req), 0x00, 3, ID_User1);
 
     	for (i=0;i<3;i++){
 		len				= strlen(reqSAT[i]);
-		s->Protocol.Data.AT.EditMode 	= true;
+		sm->Protocol.Data.AT.EditMode 	= true;
         	sprintf(req, "AT^SSTK=%i,1\r",len/2);
-        	error = GSM_WaitFor (s, req, strlen(req), 0x00, 3, ID_User1);
-		s->Phone.Data.DispatchError	= ERR_TIMEOUT;
-		s->Phone.Data.RequestID		= ID_User1;
-    		error = s->Protocol.Functions->WriteMessage(s, reqSAT[i], len, 0x00);
+        	error = GSM_WaitFor (sm, req, strlen(req), 0x00, 3, ID_User1);
+		sm->Phone.Data.DispatchError	= ERR_TIMEOUT;
+		sm->Phone.Data.RequestID		= ID_User1;
+    		error = sm->Protocol.Functions->WriteMessage(sm, reqSAT[i], len, 0x00);
     		if (error!=ERR_NONE) return error;
-    		error = s->Protocol.Functions->WriteMessage(s, "\x1A", 1, 0x00);
+    		error = sm->Protocol.Functions->WriteMessage(sm, "\x1A", 1, 0x00);
 		if (error!=ERR_NONE) return error;
-        	error = GSM_WaitForOnce (s, NULL,0x00, 0x00, 4);
+        	error = GSM_WaitForOnce (sm, NULL,0x00, 0x00, 4);
     		if (error!=ERR_NONE) return error;
     	}
     	return ERR_NONE;
 }
 
-GSM_Error ATSIEMENS_GetNetmon(GSM_StateMachine *s,int test_no)
+GSM_Error ATSIEMENS_GetNetmon(GSM_StateMachine *sm,int test_no)
 {
-	GSM_Phone_ATGENData	*Priv = &(s->Phone.Data.Priv.ATGEN);
+	GSM_Phone_ATGENData	*Priv = &(sm->Phone.Data.Priv.ATGEN);
 	unsigned char		req[32];
 
 	if (Priv->Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
 	sprintf(req, "AT^S^MI=%d\r",test_no);
 	printf ("Siemens NetMonitor test #%i\n",test_no);
-	return GSM_WaitFor(s, req, strlen(req), 0x00, 3, ID_User2);
+	return GSM_WaitFor(sm, req, strlen(req), 0x00, 3, ID_User2);
 }
 
-GSM_Error ATSIEMENS_ActivateNetmon (GSM_StateMachine *s,int netmon_type)
+GSM_Error ATSIEMENS_ActivateNetmon (GSM_StateMachine *sm,int netmon_type)
 {
-	GSM_Phone_ATGENData	*Priv = &(s->Phone.Data.Priv.ATGEN);
+	GSM_Phone_ATGENData	*Priv = &(sm->Phone.Data.Priv.ATGEN);
 	unsigned char		req[32];
 
 	if (Priv->Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
@@ -280,7 +281,7 @@ GSM_Error ATSIEMENS_ActivateNetmon (GSM_StateMachine *s,int netmon_type)
 	printf ("Activate Siemens NetMonitor\n");
 	siemens_code (req,req,2);
 
-	return GSM_WaitFor(s, req, strlen(req), 0x00, 3, ID_User2);
+	return GSM_WaitFor(sm, req, strlen(req), 0x00, 3, ID_User2);
 }
 
 void ATSIEMENSActivateNetmon(int argc, char *argv[])
@@ -293,18 +294,18 @@ void ATSIEMENSActivateNetmon(int argc, char *argv[])
 
 	GSM_Init(true);
 	if (CheckSiemens()==ERR_NOTSUPPORTED) Print_Error(ERR_NOTSUPPORTED);
-	s->User.UserReplyFunctions=UserReplyFunctionsAtS;
+	gsm->User.UserReplyFunctions=UserReplyFunctionsAtS;
 
 	printf ("Activate NetMonitor...\n");
 	netmon_type = atoi(argv[2]);
 
 	if ((netmon_type==1) || (netmon_type==2)) {
-	    error   = ATGEN_GetSIMIMSI (s,imsi);
+	    error   = ATGEN_GetSIMIMSI (gsm,imsi);
 	    Print_Error(error);
 	    siemens_code(imsi,NetMonCode,netmon_type);
 
 	    status.MemoryType = MEM_SM;
-	    error = ATGEN_GetMemoryStatus (s,&status);
+	    error = ATGEN_GetMemoryStatus (gsm,&status);
 	    Print_Error(error);
 
 	    pbk_maxlocation = status.MemoryUsed+status.MemoryFree;
@@ -316,7 +317,7 @@ void ATSIEMENSActivateNetmon(int argc, char *argv[])
 	    pbk.Entries[1].EntryType = PBK_Text_Name;
 	    sprintf (NetMonCode,"Net Monitor");
 	    EncodeUnicode (pbk.Entries[1].Text,NetMonCode,strlen(NetMonCode));
-	    error = ATGEN_SetMemory (s, &pbk);
+	    error = ATGEN_SetMemory (gsm, &pbk);
 	    Print_Error(error);
 	}
 	else printf ("NetMonitor type should be:\n1 - full Netmon\n2 - simple NetMon\n");
@@ -329,11 +330,11 @@ void ATSIEMENSSATNetmon(int argc, char *argv[])
 	GSM_Error error;
 	GSM_Init(true);
 	if (CheckSiemens()==ERR_NOTSUPPORTED) Print_Error(ERR_NOTSUPPORTED);
-	s->User.UserReplyFunctions=UserReplyFunctionsAtS;
+	gsm->User.UserReplyFunctions=UserReplyFunctionsAtS;
 
 	printf ("Getting Siemens Sim Aplication Toolkit NetMonitor...\n");
 
-	error=ATSIEMENS_GetSAT(s);
+	error=ATSIEMENS_GetSAT(gsm);
 	Print_Error(error);
 	GSM_Terminate();
 }
@@ -345,11 +346,11 @@ void ATSIEMENSNetmonitor(int argc, char *argv[])
 
 	GSM_Init(true);
 	if (CheckSiemens()==ERR_NOTSUPPORTED) Print_Error(ERR_NOTSUPPORTED);
-	s->User.UserReplyFunctions=UserReplyFunctionsAtS;
+	gsm->User.UserReplyFunctions=UserReplyFunctionsAtS;
 
 	printf ("Getting Siemens NetMonitor...\n");
 	test_no = atoi(argv[2]);
-	error   = ATSIEMENS_GetNetmon (s,test_no+1);
+	error   = ATSIEMENS_GetNetmon (gsm,test_no+1);
 	Print_Error(error);
 	GSM_Terminate();
 }
