@@ -4,7 +4,11 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include <unistd.h>
+#ifdef WIN32
+#  include <io.h>
+#else
+#  include <unistd.h>
+#endif
 
 #include "../common/misc/locales.h"
 
@@ -21,14 +25,48 @@ volatile bool 			wasincomingsms 		= false;
 
 GSM_MultiSMSMessage		IncomingSMSData;
 
-void PrintSMSCNumber(unsigned char *number,GSM_Backup *Info)
+/**
+ * Prints location information about message.
+ */
+void PrintSMSLocation(const GSM_SMSMessage *sms, const GSM_SMSFolders *folders) 
+{
+	printf(_("Location %i, folder \"%s\""),
+			sms->Location,
+			DecodeUnicodeConsole(folders->Folder[sms->Folder - 1].Name)
+			);
+
+	switch (sms->Memory) {
+		case MEM_SM: 
+			printf(", %s", _("SIM memory"));
+			break;
+		case MEM_ME: 
+			printf(", %s", _("phone memory"));
+			break;
+		case MEM_MT: 
+			printf(", %s", _("phone or SIM memory")); 
+			break;
+		default: 
+			break;
+	}
+
+	if (sms->InboxFolder) {
+		printf(", %s", _("Inbox folder"));
+	}
+
+	printf("\n");
+}
+
+/**
+ * Prints single phone number optionally showing name of contact from backup data.
+ */
+void PrintPhoneNumber(unsigned char *number, const GSM_Backup *Info)
 {
 	bool 	found=false,found2=false;
 	int 	i,j,z;
 
 	printf("\"%s\"",DecodeUnicodeConsole(number));
 
-	if (Info==NULL) return;
+	if (Info == NULL) return;
 
 	i=0;
 	while (Info->PhonePhonebook[i]!=NULL) {
@@ -45,19 +83,19 @@ void PrintSMSCNumber(unsigned char *number,GSM_Backup *Info)
 					found2=true;
 					switch (Info->PhonePhonebook[i]->Entries[j].EntryType) {
 					case PBK_Number_Mobile:
-						printf("(%s", _("mobile"));
+						printf(" (%s", _("mobile"));
 						break;
 					case PBK_Number_Work:
-						printf("(%s", _("work"));
+						printf(" (%s", _("work"));
 						break;
 					case PBK_Number_Fax:
-						printf("(%s", _("fax"));
+						printf(" (%s", _("fax"));
 						break;
 					case PBK_Number_Home:
-						printf("(%s", _("home"));
+						printf(" (%s", _("home"));
 						break;
 					case PBK_Number_Pager:
-						printf("(%s", _("pager"));
+						printf(" (%s", _("pager"));
 						break;
 					default:
 						found2=false;
@@ -116,7 +154,7 @@ void PrintSMSCNumber(unsigned char *number,GSM_Backup *Info)
 	}
 }
 
-void DisplaySingleSMSInfo(GSM_SMSMessage sms, bool displaytext, bool displayudh, GSM_Backup *Info)
+void DisplaySingleSMSInfo(GSM_SMSMessage sms, bool displaytext, bool displayudh, const GSM_Backup *Info)
 {
 	GSM_SiemensOTASMSInfo 	SiemensOTA;
 	int			i;
@@ -135,7 +173,7 @@ void DisplaySingleSMSInfo(GSM_SMSMessage sms, bool displaytext, bool displayudh,
 		printf("\n");
 
 		printf(LISTFORMAT, _("Remote number"));
-		PrintSMSCNumber(sms.Number, Info);
+		PrintPhoneNumber(sms.Number, Info);
 		printf("\n");
 
 		printf(LISTFORMAT "%d\n", _("Reference number"),sms.MessageReference);
@@ -224,10 +262,10 @@ void DisplaySingleSMSInfo(GSM_SMSMessage sms, bool displaytext, bool displayudh,
 		if (sms.State==SMS_UnSent && sms.Memory==MEM_ME) {
 		} else {
 			printf(LISTFORMAT, ngettext("Remote number", "Remote numbers", sms.OtherNumbersNum + 1));
-			PrintSMSCNumber(sms.Number, Info);
+			PrintPhoneNumber(sms.Number, Info);
 			for (i=0;i<sms.OtherNumbersNum;i++) {
 				printf(", ");
-				PrintSMSCNumber(sms.OtherNumbers[i], Info);
+				PrintPhoneNumber(sms.OtherNumbers[i], Info);
 			}
 			printf("\n");
 		}
@@ -290,10 +328,15 @@ void DisplaySingleSMSInfo(GSM_SMSMessage sms, bool displaytext, bool displayudh,
 			}
 		}
 		break;
+#ifndef CHECK_CASES
+	default:
+		printf(_("Unknown PDU type: 0x%x\n"), sms.PDU);
+		break;
+#endif
 	}
 }
 
-void DisplayMultiSMSInfo (GSM_MultiSMSMessage sms, bool eachsms, bool ems, GSM_Backup *Info)
+void DisplayMultiSMSInfo (GSM_MultiSMSMessage *sms, bool eachsms, bool ems, const GSM_Backup *Info)
 {
 	GSM_SiemensOTASMSInfo 	SiemensOTA;
 	GSM_MultiPartSMSInfo	SMSInfo;
@@ -303,23 +346,23 @@ void DisplayMultiSMSInfo (GSM_MultiSMSMessage sms, bool eachsms, bool ems, GSM_B
 	GSM_Error error;
 
 	/* GSM_DecodeMultiPartSMS returns if decoded SMS content correctly */
-	RetVal = GSM_DecodeMultiPartSMS(&SMSInfo,&sms,ems);
+	RetVal = GSM_DecodeMultiPartSMS(&SMSInfo,sms,ems);
 
 	if (eachsms) {
-		if (GSM_DecodeSiemensOTASMS(&SiemensOTA,&sms.SMS[0])) udhinfo = false;
-		if (sms.SMS[0].UDH.Type != UDH_NoUDH && sms.SMS[0].UDH.AllParts == sms.Number) udhinfo = false;
+		if (GSM_DecodeSiemensOTASMS(&SiemensOTA,&sms->SMS[0])) udhinfo = false;
+		if (sms->SMS[0].UDH.Type != UDH_NoUDH && sms->SMS[0].UDH.AllParts == sms->Number) udhinfo = false;
 		if (RetVal && !udhinfo) {
-			DisplaySingleSMSInfo(sms.SMS[0],false,false,Info);
+			DisplaySingleSMSInfo(sms->SMS[0],false,false,Info);
 			printf("\n");
 		} else {
-			for (j=0;j<sms.Number;j++) {
-				DisplaySingleSMSInfo(sms.SMS[j],!RetVal,udhinfo,Info);
+			for (j=0;j<sms->Number;j++) {
+				DisplaySingleSMSInfo(sms->SMS[j],!RetVal,udhinfo,Info);
 				printf("\n");
 			}
 		}
 	} else {
-		for (j=0;j<sms.Number;j++) {
-			DisplaySingleSMSInfo(sms.SMS[j],!RetVal,true,Info);
+		for (j=0;j<sms->Number;j++) {
+			DisplaySingleSMSInfo(sms->SMS[j],!RetVal,true,Info);
 			printf("\n");
 		}
 	}
@@ -350,7 +393,8 @@ void DisplayMultiSMSInfo (GSM_MultiSMSMessage sms, bool eachsms, bool ems, GSM_B
 			printf(_("Ringtone \"%s\"\n"),DecodeUnicodeConsole(SMSInfo.Entries[i].Ringtone->Name));
 			GSM_SaveRingtoneRttl(stdout,SMSInfo.Entries[i].Ringtone);
 			printf("\n");
-			if (answer_yes(_("Do you want to play it"))) GSM_PlayRingtone(*SMSInfo.Entries[i].Ringtone);
+			if (answer_yes(_("Do you want to play it?")))
+				GSM_PlayRingtone(*SMSInfo.Entries[i].Ringtone);
 			break;
 		case SMS_NokiaCallerLogo:
 			printf("%s\n\n", _("Caller logo"));
@@ -438,7 +482,7 @@ void DisplayMultiSMSInfo (GSM_MultiSMSMessage sms, bool eachsms, bool ems, GSM_B
 	GSM_FreeMultiPartSMSInfo(&SMSInfo);
 }
 
-void IncomingSMS(GSM_StateMachine *s UNUSED, GSM_SMSMessage sms)
+void IncomingSMS(GSM_StateMachine *sm UNUSED, GSM_SMSMessage sms)
 {
 	printf("%s\n", _("SMS message received"));
  	if (wasincomingsms) {
@@ -456,10 +500,10 @@ void DisplayIncomingSMS()
 	GSM_Error error;
 
  	if (IncomingSMSData.SMS[0].State == 0) {
- 		error=GSM_GetSMSFolders(s, &folders);
+ 		error=GSM_GetSMSFolders(gsm, &folders);
  		Print_Error(error);
 
- 		error=GSM_GetSMS(s, &IncomingSMSData);
+ 		error=GSM_GetSMS(gsm, &IncomingSMSData);
  		switch (error) {
  		case ERR_EMPTY:
  			printf(_("Location %i\n"),IncomingSMSData.SMS[0].Location);
@@ -467,28 +511,20 @@ void DisplayIncomingSMS()
  			break;
  		default:
  			Print_Error(error);
- 			printf(_("Location %i, folder \"%s\""),IncomingSMSData.SMS[0].Location,DecodeUnicodeConsole(folders.Folder[IncomingSMSData.SMS[0].Folder-1].Name));
- 			switch(IncomingSMSData.SMS[0].Memory) {
- 				case MEM_SM: printf(_(", SIM memory")); 		break;
- 				case MEM_ME: printf(_(", phone memory")); 	break;
- 				case MEM_MT: printf(_(", phone or SIM memory")); break;
- 				default    : break;
- 			}
- 			if (IncomingSMSData.SMS[0].InboxFolder) printf(_(", Inbox folder"));
- 			printf("\n");
+			PrintSMSLocation(&IncomingSMSData.SMS[0], &folders);
  		}
  	}
- 	DisplayMultiSMSInfo(IncomingSMSData,false,false,NULL);
+ 	DisplayMultiSMSInfo(&IncomingSMSData,false,false,NULL);
  	wasincomingsms = false;
 }
 
-void IncomingCB(GSM_StateMachine *s UNUSED, GSM_CBMessage CB)
+void IncomingCB(GSM_StateMachine *sm UNUSED, GSM_CBMessage CB)
 {
 	printf("%s\n", _("CB message received"));
 	printf(_("Channel %i, text \"%s\"\n"),CB.Channel,DecodeUnicodeConsole(CB.Text));
 }
 
-void IncomingUSSD(GSM_StateMachine *s UNUSED, GSM_USSDMessage ussd)
+void IncomingUSSD(GSM_StateMachine *sm UNUSED, GSM_USSDMessage ussd)
 {
 	printf("%s\n", _("USSD received"));
 	printf(LISTFORMAT, _("Status"));
@@ -521,9 +557,9 @@ void IncomingUSSD(GSM_StateMachine *s UNUSED, GSM_USSDMessage ussd)
 	printf(LISTFORMAT "\"%s\"\n", _("Service reply"), DecodeUnicodeConsole(ussd.Text));
 }
 
-void IncomingUSSD2(GSM_StateMachine *s, GSM_USSDMessage ussd)
+void IncomingUSSD2(GSM_StateMachine *sm, GSM_USSDMessage ussd)
 {
-	IncomingUSSD(s, ussd);
+	IncomingUSSD(sm, ussd);
 
 	gshutdown = true;
 }
@@ -536,21 +572,21 @@ void GetUSSD(int argc UNUSED, char *argv[])
 	signal(SIGINT, interrupt);
 	fprintf(stderr, "%s\n", _("Press Ctrl+C to break..."));
 
-	GSM_SetIncomingUSSDCallback(s, IncomingUSSD2);
+	GSM_SetIncomingUSSDCallback(gsm, IncomingUSSD2);
 
-	error=GSM_SetIncomingUSSD(s,true);
+	error=GSM_SetIncomingUSSD(gsm,true);
 	Print_Error(error);
 
-	error=GSM_DialService(s, argv[2]);
+	error=GSM_DialService(gsm, argv[2]);
 	/* Fallback to voice call, it can work with some phones */
 	if (error == ERR_NOTIMPLEMENTED || error == ERR_NOTSUPPORTED) {
-		error=GSM_DialVoice(s, argv[2], GSM_CALL_DefaultNumberPresence);
+		error=GSM_DialVoice(gsm, argv[2], GSM_CALL_DefaultNumberPresence);
 	}
 	Print_Error(error);
 
-	while (!gshutdown) GSM_ReadDevice(s, false);
+	while (!gshutdown) GSM_ReadDevice(gsm, false);
 
-	error=GSM_SetIncomingUSSD(s, false);
+	error=GSM_SetIncomingUSSD(gsm, false);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -574,7 +610,7 @@ void GetSMSC(int argc, char *argv[])
 	for (i=start;i<=stop;i++) {
 		smsc.Location=i;
 
-		error=GSM_GetSMSC(s, &smsc);
+		error=GSM_GetSMSC(gsm, &smsc);
 		Print_Error(error);
 
 		if (!strcmp(DecodeUnicodeConsole(smsc.Name),"")) {
@@ -615,8 +651,10 @@ void GetSMSC(int argc, char *argv[])
 			case SMS_VALID_Max_Time	:
 				printf(_("Maximum time"));
 				break;
-			default           	:
-				if (smsc.Validity.Relative >= 0 && smsc.Validity.Relative <= 143) {
+			default:
+				/* Typecasting is here needed to silent GCC warning.
+				 * Validity usually fits in some unsigned type so it is always >= 0 */
+				if ((int)smsc.Validity.Relative >= 0 && smsc.Validity.Relative <= 143) {
 					PRINTMINUTES((smsc.Validity.Relative + 1) * 5);
 				} else if (smsc.Validity.Relative >= 144 && smsc.Validity.Relative <= 167) {
 					PRINTMINUTES(12 * 60 + (smsc.Validity.Relative - 143) * 30);
@@ -644,13 +682,13 @@ void GetSMS(int argc, char *argv[])
 
 	GSM_Init(true);
 
-	error=GSM_GetSMSFolders(s, &folders);
+	error=GSM_GetSMSFolders(gsm, &folders);
 	Print_Error(error);
 
 	for (j = start; j <= stop; j++) {
 		sms.SMS[0].Folder	= atoi(argv[2]);
 		sms.SMS[0].Location	= j;
-		error=GSM_GetSMS(s, &sms);
+		error=GSM_GetSMS(gsm, &sms);
 		switch (error) {
 		case ERR_EMPTY:
 			printf(_("Location %i\n"),sms.SMS[0].Location);
@@ -658,16 +696,8 @@ void GetSMS(int argc, char *argv[])
 			break;
 		default:
 			Print_Error(error);
-			printf(_("Location %i, folder \"%s\""),sms.SMS[0].Location,DecodeUnicodeConsole(folders.Folder[sms.SMS[0].Folder-1].Name));
-			switch(sms.SMS[0].Memory) {
-				case MEM_SM: printf(_(", SIM memory")); 		break;
-				case MEM_ME: printf(_(", phone memory")); 	break;
-				case MEM_MT: printf(_(", phone or SIM memory")); break;
-				default    : break;
-			}
-			if (sms.SMS[0].InboxFolder) printf(_(", Inbox folder"));
-			printf("\n");
-			DisplayMultiSMSInfo(sms,false,false,NULL);
+			PrintSMSLocation(&sms.SMS[0], &folders);
+			DisplayMultiSMSInfo(&sms,false,false,NULL);
 		}
 	}
 
@@ -688,7 +718,7 @@ void DeleteSMS(int argc, char *argv[])
 
 	for (i=start;i<=stop;i++) {
 		sms.Location	= i;
-		error=GSM_DeleteSMS(s, &sms);
+		error=GSM_DeleteSMS(gsm, &sms);
 		Print_Error(error);
 	}
 #ifdef GSM_ENABLE_BEEP
@@ -704,6 +734,7 @@ void GetAllSMS(int argc, char *argv[])
 	GSM_SMSFolders		folders;
 	bool			start = true;
 	int			smsnum=0,smspos=0;
+	GSM_Backup		*BackupPtr = NULL;
 #ifdef GSM_ENABLE_BACKUP
 	int			used,i;
 	GSM_MemoryStatus	MemStatus;
@@ -711,6 +742,7 @@ void GetAllSMS(int argc, char *argv[])
 	GSM_Backup		Backup;
 
 	GSM_ClearBackup(&Backup);
+	BackupPtr = &Backup;
 #endif
 
 	GSM_Init(true);
@@ -718,14 +750,14 @@ void GetAllSMS(int argc, char *argv[])
 #ifdef GSM_ENABLE_BACKUP
 	if (argc == 3 && strcasecmp(argv[2],"-pbk") == 0) {
 		MemStatus.MemoryType = MEM_ME;
-		error=GSM_GetMemoryStatus(s, &MemStatus);
+		error=GSM_GetMemoryStatus(gsm, &MemStatus);
 		if (error==ERR_NONE && MemStatus.MemoryUsed != 0) {
 			Pbk.MemoryType  = MEM_ME;
 			i		= 1;
 			used 		= 0;
 			while (used != MemStatus.MemoryUsed) {
 				Pbk.Location = i;
-				error=GSM_GetMemory(s, &Pbk);
+				error=GSM_GetMemory(gsm, &Pbk);
 				if (error != ERR_EMPTY) {
 					Print_Error(error);
 					if (used < GSM_BACKUP_MAX_PHONEPHONEBOOK) {
@@ -751,37 +783,25 @@ void GetAllSMS(int argc, char *argv[])
 	}
 #endif
 
-	error=GSM_GetSMSFolders(s, &folders);
+	error=GSM_GetSMSFolders(gsm, &folders);
 	Print_Error(error);
 
 	while (error == ERR_NONE) {
 		sms.SMS[0].Folder=0x00;
-		error=GSM_GetNextSMS(s, &sms, start);
+		error=GSM_GetNextSMS(gsm, &sms, start);
 		switch (error) {
 		case ERR_EMPTY:
 			break;
+		case ERR_CORRUPTED:
+			fprintf(stderr, "\n%s\n", _("Corrupted message, skipping"));
+			error = ERR_NONE;
+			continue;
 		default:
 			Print_Error(error);
-			printf(_("Location %i, folder \"%s\""),sms.SMS[0].Location,DecodeUnicodeConsole(folders.Folder[sms.SMS[0].Folder-1].Name));
-			switch(sms.SMS[0].Memory) {
-				case MEM_SM: printf(_(", SIM memory")); 		break;
-				case MEM_ME: printf(_(", phone memory")); 	break;
-				case MEM_MT: printf(_(", phone or SIM memory")); break;
-				default    : break;
-			}
-			if (sms.SMS[0].InboxFolder) printf(_(", Inbox folder"));
-			printf("\n");
+			PrintSMSLocation(&sms.SMS[0], &folders);
 			smspos++;
 			smsnum+=sms.Number;
-#ifdef GSM_ENABLE_BACKUP
-			if (Backup.PhonePhonebook[0]!=NULL) {
-				DisplayMultiSMSInfo(sms,false,false,&Backup);
-			} else {
-#endif
-				DisplayMultiSMSInfo(sms,false,false,NULL);
-#ifdef GSM_ENABLE_BACKUP
-			}
-#endif
+			DisplayMultiSMSInfo(&sms, false, false, BackupPtr);
 		}
 		start=false;
 	}
@@ -796,11 +816,12 @@ void GetAllSMS(int argc, char *argv[])
 void GetEachSMS(int argc, char *argv[])
 {
 	GSM_Error error;
-	GSM_MultiSMSMessage	*GetSMS[GSM_PHONE_MAXSMSINFOLDER],*SortedSMS[GSM_PHONE_MAXSMSINFOLDER],sms;
+	GSM_MultiSMSMessage	*GetSMSData[GSM_PHONE_MAXSMSINFOLDER],*SortedSMS[GSM_PHONE_MAXSMSINFOLDER],sms;
 	int			GetSMSNumber = 0,i,j;
 	int			smsnum=0,smspos=0;
 	GSM_SMSFolders		folders;
 	bool			start = true, ems = true;
+	GSM_Backup		*BackupPtr = NULL;
 #ifdef GSM_ENABLE_BACKUP
 	GSM_MemoryStatus	MemStatus;
 	GSM_MemoryEntry		Pbk;
@@ -808,23 +829,24 @@ void GetEachSMS(int argc, char *argv[])
 	GSM_Backup		Backup;
 
 	GSM_ClearBackup(&Backup);
+	BackupPtr = &Backup;
 #endif
 
-	GetSMS[0] = NULL;
+	GetSMSData[0] = NULL;
 
 	GSM_Init(true);
 
 #ifdef GSM_ENABLE_BACKUP
 	if (argc == 3 && strcasecmp(argv[2],"-pbk") == 0) {
 		MemStatus.MemoryType = MEM_ME;
-		error=GSM_GetMemoryStatus(s, &MemStatus);
+		error=GSM_GetMemoryStatus(gsm, &MemStatus);
 		if (error==ERR_NONE && MemStatus.MemoryUsed != 0) {
 			Pbk.MemoryType  = MEM_ME;
 			i		= 1;
 			used 		= 0;
 			while (used != MemStatus.MemoryUsed) {
 				Pbk.Location = i;
-				error=GSM_GetMemory(s, &Pbk);
+				error=GSM_GetMemory(gsm, &Pbk);
 				if (error != ERR_EMPTY) {
 					Print_Error(error);
 					if (used < GSM_BACKUP_MAX_PHONEPHONEBOOK) {
@@ -850,7 +872,7 @@ void GetEachSMS(int argc, char *argv[])
 	}
 #endif
 
-	error=GSM_GetSMSFolders(s, &folders);
+	error=GSM_GetSMSFolders(gsm, &folders);
 	Print_Error(error);
 
 	fprintf(stderr, LISTFORMAT, _("Reading"));
@@ -860,16 +882,20 @@ void GetEachSMS(int argc, char *argv[])
 			break;
 		}
 		sms.SMS[0].Folder=0x00;
-		error=GSM_GetNextSMS(s, &sms, start);
+		error=GSM_GetNextSMS(gsm, &sms, start);
 		switch (error) {
 		case ERR_EMPTY:
 			break;
+		case ERR_CORRUPTED:
+			fprintf(stderr, "\n%s\n", _("Corrupted message, skipping"));
+			error = ERR_NONE;
+			continue;
 		default:
 			Print_Error(error);
-			GetSMS[GetSMSNumber] = malloc(sizeof(GSM_MultiSMSMessage));
-		        if (GetSMS[GetSMSNumber] == NULL) Print_Error(ERR_MOREMEMORY);
-			GetSMS[GetSMSNumber+1] = NULL;
-			memcpy(GetSMS[GetSMSNumber],&sms,sizeof(GSM_MultiSMSMessage));
+			GetSMSData[GetSMSNumber] = malloc(sizeof(GSM_MultiSMSMessage));
+		        if (GetSMSData[GetSMSNumber] == NULL) Print_Error(ERR_MOREMEMORY);
+			GetSMSData[GetSMSNumber+1] = NULL;
+			memcpy(GetSMSData[GetSMSNumber],&sms,sizeof(GSM_MultiSMSMessage));
 			GetSMSNumber++;
 		}
 		fprintf(stderr,"*");
@@ -881,13 +907,13 @@ void GetEachSMS(int argc, char *argv[])
 	GSM_PhoneBeep();
 #endif
 
-	error = GSM_LinkSMS(GetSMS, SortedSMS, ems);
+	error = GSM_LinkSMS(GetSMSData, SortedSMS, ems);
 	Print_Error(error);
 
 	i=0;
-	while(GetSMS[i] != NULL) {
-		free(GetSMS[i]);
-		GetSMS[i] = NULL;
+	while(GetSMSData[i] != NULL) {
+		free(GetSMSData[i]);
+		GetSMSData[i] = NULL;
 		i++;
 	}
 
@@ -897,26 +923,10 @@ void GetEachSMS(int argc, char *argv[])
 		for (j=0;j<SortedSMS[i]->Number;j++) {
 			smsnum++;
 			if ((j==0) || (j!=0 && SortedSMS[i]->SMS[j].Location != SortedSMS[i]->SMS[j-1].Location)) {
-				printf(_("Location %i, folder \"%s\""),SortedSMS[i]->SMS[j].Location,DecodeUnicodeConsole(folders.Folder[SortedSMS[i]->SMS[j].Folder-1].Name));
-				switch(SortedSMS[i]->SMS[j].Memory) {
-					case MEM_SM: printf(_(", SIM memory")); 		break;
-					case MEM_ME: printf(_(", phone memory")); 	break;
-					case MEM_MT: printf(_(", phone or SIM memory")); break;
-					default    : break;
-				}
-				if (SortedSMS[i]->SMS[j].InboxFolder) printf(_(", Inbox folder"));
-				printf("\n");
+				PrintSMSLocation(&SortedSMS[i]->SMS[j], &folders);
 			}
 		}
-#ifdef GSM_ENABLE_BACKUP
-		if (Backup.PhonePhonebook[0]!=NULL) {
-			DisplayMultiSMSInfo(*SortedSMS[i],true,ems,&Backup);
-		} else {
-#endif
-			DisplayMultiSMSInfo(*SortedSMS[i],true,ems,NULL);
-#ifdef GSM_ENABLE_BACKUP
-		}
-#endif
+		DisplayMultiSMSInfo(SortedSMS[i], true, ems, BackupPtr);
 
 		free(SortedSMS[i]);
 		SortedSMS[i] = NULL;
@@ -936,7 +946,7 @@ void GetSMSFolders(int argc UNUSED, char *argv[] UNUSED)
 
 	GSM_Init(true);
 
-	error=GSM_GetSMSFolders(s,&folders);
+	error=GSM_GetSMSFolders(gsm,&folders);
 	Print_Error(error);
 
 	for (i=0;i<folders.Number;i++) {
@@ -963,7 +973,7 @@ void GetMMSFolders(int argc UNUSED, char *argv[] UNUSED)
 
 	GSM_Init(true);
 
-	error=GSM_GetMMSFolders(s,&folders);
+	error=GSM_GetMMSFolders(gsm,&folders);
 	Print_Error(error);
 
 	for (i=0;i<folders.Number;i++) {
@@ -986,12 +996,15 @@ void DecodeMMSFile(GSM_File *file, int num)
 	GSM_CalendarEntry 		Calendar;
 	GSM_ToDoEntry 			ToDo;
 
-	if (num != -1 && answer_yes(_("Do you want to save this MMS file"))) {
+	if (num != -1 && answer_yes(_("Do you want to save this MMS file?"))) {
 		sprintf(buff,"%i_0",num);
 		file2 = fopen(buff,"wb");
-		fwrite(file->Buffer, 1, file->Used, file2);
+		if (fwrite(file->Buffer, 1, file->Used, file2) != file->Used) {
+			printf_err(_("Error while saving to file %s!\n"), buff);
+		} else {
+			printf(_("Saved to file %s\n"),buff);
+		}
 		fclose(file2);
-		printf(_("Saved to file %s\n"),buff);
 	}
 
 	for (i=0;i<GSM_MAX_MULTI_MMS;i++) info.Entries[i].File.Buffer = NULL;
@@ -1064,12 +1077,15 @@ void DecodeMMSFile(GSM_File *file, int num)
 			error = GSM_DecodeVCALENDAR_VTODO(info.Entries[i].File.Buffer, &Pos, &Calendar, &ToDo, Nokia_VCalendar, Nokia_VToDo);
 			if (error == ERR_NONE) PrintCalendar(&Calendar);
 		}
-		if (num != -1 && answer_yes(_("Do you want to save this attachment"))) {
+		if (num != -1 && answer_yes(_("Do you want to save this attachment?"))) {
 			sprintf(buff,"%i_%i_%s",num,i+1,DecodeUnicodeString(info.Entries[i].File.Name));
 			file2 = fopen(buff,"wb");
-			fwrite(info.Entries[i].File.Buffer, 1, info.Entries[i].File.Used, file2);
+			if (fwrite(info.Entries[i].File.Buffer, 1, info.Entries[i].File.Used, file2) != info.Entries[i].File.Used) {
+				printf_err(_("Error while saving to file %s!\n"), buff);
+			} else {
+				printf(_("Saved to file %s\n"),buff);
+			}
 			fclose(file2);
-			printf(_("Saved to file %s\n"),buff);
 		}
 
 	}
@@ -1090,36 +1106,33 @@ void GetEachMMS(int argc, char *argv[])
 
 	GSM_Init(true);
 
-	error=GSM_GetMMSFolders(s,&folders);
+	error=GSM_GetMMSFolders(gsm,&folders);
 	Print_Error(error);
 
 	File.Buffer = NULL;
 
 	while (1) {
-		error=GSM_GetNextMMSFileInfo(s,File.ID_FullName,&FileFolder,start);
-		if (error==ERR_EMPTY) break;
+		error = GSM_GetNextMMSFileInfo(gsm,File.ID_FullName,&FileFolder,start);
+		if (error == ERR_EMPTY) break;
 		Print_Error(error);
 		start = false;
 
 		printf(_("Folder %s\n"),DecodeUnicodeConsole(folders.Folder[FileFolder-1].Name));
 		printf(LISTFORMAT "\"%s\"\n", _("  File filesystem ID"),DecodeUnicodeConsole(File.ID_FullName));
-		if (!File.ModifiedEmpty) {
-			printf(_("  File last changed  : %s\n"),OSDateTime(File.Modified,0));
-		}
 		if (File.Buffer != NULL) {
 			free(File.Buffer);
 			File.Buffer = NULL;
 		}
 		File.Used = 0;
 		while (true) {
-			error = GSM_GetFilePart(s,&File,&Handle,&Size);
+			error = GSM_GetFilePart(gsm,&File,&Handle,&Size);
 			if (error == ERR_EMPTY) break;
 			Print_Error(error);
 			fprintf(stderr, _("%c  Reading: %i percent"),13, (int)(File.Used*100/Size));
 		}
 		fprintf(stderr, "%c",13);
 
-		if (GSM_IsPhoneFeatureAvailable(GSM_GetModelInfo(s), F_SERIES40_30)) {
+		if (GSM_IsPhoneFeatureAvailable(GSM_GetModelInfo(gsm), F_SERIES40_30)) {
 			memcpy(File.Buffer,File.Buffer+176,File.Used-176);
 			File.Used-=176;
 			File.Buffer = realloc(File.Buffer,File.Used);
@@ -1142,7 +1155,7 @@ void DisplaySMSFrame(GSM_SMSMessage *SMS)
 #ifdef OSCAR
         unsigned char           hexmsg[1000], hexudh[1000];
 #endif
-	error=PHONE_EncodeSMSFrame(s,SMS,buffer,PHONE_SMSSubmit,&length,true);
+	error=PHONE_EncodeSMSFrame(gsm,SMS,buffer,PHONE_SMSSubmit,&length,true);
 	if (error != ERR_NONE) {
 		printf("%s\n", _("Error"));
 		exit(-1);
@@ -1186,9 +1199,9 @@ void DisplaySMSFrame(GSM_SMSMessage *SMS)
 
 GSM_Error SMSStatus;
 
-void SendSMSStatus (GSM_StateMachine *s, int status, int MessageReference)
+void SendSMSStatus (GSM_StateMachine *sm, int status, int MessageReference)
 {
-	dbgprintf("Sent SMS on device: \"%s\"\n", GSM_GetConfig(s, -1)->Device);
+	dbgprintf("Sent SMS on device: \"%s\"\n", GSM_GetConfig(sm, -1)->Device);
 	if (status==0) {
 		printf(_("..OK"));
 		SMSStatus = ERR_NONE;
@@ -1425,7 +1438,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			printf("%s\n", _("Where is backup filename and location?"));
 			exit(-1);
 		}
-		error=GSM_ReadBackupFile(argv[3+startarg],&Backup);
+		error=GSM_ReadBackupFile(argv[3+startarg],&Backup,GSM_GuessBackupFormat(argv[3+startarg], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
 		i = 0;
 		while (Backup.WAPBookmark[i]!=NULL) {
@@ -1448,7 +1461,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			printf("%s\n", _("Where is backup filename and location?"));
 			exit(-1);
 		}
-		error=GSM_ReadBackupFile(argv[3+startarg],&Backup);
+		error=GSM_ReadBackupFile(argv[3+startarg],&Backup,GSM_GuessBackupFormat(argv[3+startarg], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
 		i = 0;
 		while (Backup.WAPSettings[i]!=NULL) {
@@ -1491,7 +1504,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			printf("%s\n", _("Where is backup filename and location?"));
 			exit(-1);
 		}
-		error=GSM_ReadBackupFile(argv[3+startarg],&Backup);
+		error=GSM_ReadBackupFile(argv[3+startarg],&Backup,GSM_GuessBackupFormat(argv[3+startarg], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
 		i = 0;
 		while (Backup.MMSSettings[i]!=NULL) {
@@ -1527,7 +1540,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			printf("%s\n", _("Where is backup filename and location?"));
 			exit(-1);
 		}
-		error=GSM_ReadBackupFile(argv[3+startarg],&Backup);
+		error=GSM_ReadBackupFile(argv[3+startarg],&Backup,GSM_GuessBackupFormat(argv[3+startarg], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
 		i = 0;
 		while (Backup.Calendar[i]!=NULL) {
@@ -1549,7 +1562,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			printf("%s\n", _("Where is backup filename and location?"));
 			exit(-1);
 		}
-		error=GSM_ReadBackupFile(argv[3+startarg],&Backup);
+		error=GSM_ReadBackupFile(argv[3+startarg],&Backup,GSM_GuessBackupFormat(argv[3+startarg], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
 		i = 0;
 		while (Backup.ToDo[i]!=NULL) {
@@ -1571,7 +1584,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			printf("%s\n", _("Where is backup filename and location and memory type?"));
 			exit(-1);
 		}
-		error=GSM_ReadBackupFile(argv[3+startarg],&Backup);
+		error=GSM_ReadBackupFile(argv[3+startarg],&Backup,GSM_GuessBackupFormat(argv[3+startarg], false));
 		if (error!=ERR_NOTIMPLEMENTED) Print_Error(error);
 		i = 0;
 		if (strcasecmp(argv[4+startarg],"SM") == 0) {
@@ -2114,7 +2127,9 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			ReplaceFile = fopen(argv[i], "rb");
 			if (ReplaceFile == NULL) Print_Error(ERR_CANTOPENFILE);
 			memset(ReplaceBuffer,0,sizeof(ReplaceBuffer));
-			fread(ReplaceBuffer,1,sizeof(ReplaceBuffer),ReplaceFile);
+			if (fread(ReplaceBuffer,1,sizeof(ReplaceBuffer),ReplaceFile) != sizeof(ReplaceBuffer)) {
+				printf_err(_("Error while writing file!\n"));
+			}
 			fclose(ReplaceFile);
 			ReadUnicodeFile(ReplaceBuffer2,ReplaceBuffer);
 			for(j=0;j<(int)(UnicodeLength(Buffer[0]));j++) {
@@ -2333,7 +2348,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 
 		if (strcasecmp(argv[2],"OPERATOR") == 0) {
 			if (bitmap[0].Bitmap[0].Type==GSM_OperatorLogo && strcmp(bitmap[0].Bitmap[0].NetworkCode,"000 00")==0) {
-				error=GSM_GetNetworkInfo(s,&NetInfo);
+				error=GSM_GetNetworkInfo(gsm,&NetInfo);
 				Print_Error(error);
 				strcpy(bitmap[0].Bitmap[0].NetworkCode,NetInfo.NetworkCode);
 				if (SaveSMS) {
@@ -2419,23 +2434,24 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			CopyUnicodeString(sms.SMS[i].SMSC.Number, SMSC);
 			if (Validity.Format != 0) memcpy(&sms.SMS[i].SMSC.Validity,&Validity,sizeof(GSM_SMSValidity));
 		}
-		SMSDaemonSendSMS(argv[4],argv[5],&sms);
+		error = SMSDaemonSendSMS(argv[4],argv[5],&sms);
+		Print_Error(error);
 		exit(0);
 	}
 	if (SaveSMS || SendSaved) {
-		error=GSM_GetSMSFolders(s, &folders);
+		error=GSM_GetSMSFolders(gsm, &folders);
 		Print_Error(error);
 
 		if (SendSaved)	{
 			if (Validity.Format != 0 && SMSCSet != 0) {
 				PhoneSMSC.Location = SMSCSet;
-				error=GSM_GetSMSC(s,&PhoneSMSC);
+				error=GSM_GetSMSC(gsm,&PhoneSMSC);
 				Print_Error(error);
 				CopyUnicodeString(SMSC,PhoneSMSC.Number);
 				SMSCSet = 0;
 			}
 
-			GSM_SetSendSMSStatusCallback(s, SendSMSStatus);
+			GSM_SetSendSMSStatusCallback(gsm, SendSMSStatus);
 
 			signal(SIGINT, interrupt);
 			fprintf(stderr, "%s\n", _("If you want break, press Ctrl+C..."));
@@ -2443,7 +2459,6 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 
 		for (i=0;i<sms.Number;i++) {
 			printf(_("Saving SMS %i/%i\n"),i+1,sms.Number);
-//			sms.SMS[i].Location		= 0;
 			sms.SMS[i].Folder		= Folder;
 			sms.SMS[i].State		= State;
 			sms.SMS[i].ReplyViaSameSMSC	= ReplyViaSameSMSC;
@@ -2460,7 +2475,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			CopyUnicodeString(sms.SMS[i].Number, Sender);
 			CopyUnicodeString(sms.SMS[i].Name, Name);
 			if (SMSCSet==0) CopyUnicodeString(sms.SMS[i].SMSC.Number, SMSC);
-			error=GSM_AddSMS(s, &sms.SMS[i]);
+			error=GSM_AddSMS(gsm, &sms.SMS[i]);
 			Print_Error(error);
 			printf(_("Saved in folder \"%s\", location %i"),
 				DecodeUnicodeConsole(folders.Folder[sms.SMS[i].Folder-1].Name),sms.SMS[i].Location);
@@ -2477,11 +2492,11 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 				printf(_("Sending sms from folder \"%s\", location %i\n"),
 					DecodeUnicodeString(folders.Folder[sms.SMS[i].Folder-1].Name),sms.SMS[i].Location);
 				SMSStatus = ERR_TIMEOUT;
-				error=GSM_SendSavedSMS(s, 0, sms.SMS[i].Location);
+				error=GSM_SendSavedSMS(gsm, 0, sms.SMS[i].Location);
 				Print_Error(error);
 				printf(_("....waiting for network answer"));
 				while (!gshutdown) {
-					GSM_ReadDevice(s,true);
+					GSM_ReadDevice(gsm,true);
 					if (SMSStatus == ERR_UNKNOWN) {
 						GSM_Terminate();
 						exit(-1);
@@ -2493,7 +2508,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 	} else {
 		if (Validity.Format != 0 && SMSCSet != 0) {
 			PhoneSMSC.Location = SMSCSet;
-			error=GSM_GetSMSC(s,&PhoneSMSC);
+			error=GSM_GetSMSC(gsm,&PhoneSMSC);
 			Print_Error(error);
 			CopyUnicodeString(SMSC,PhoneSMSC.Number);
 			SMSCSet = 0;
@@ -2502,7 +2517,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 		signal(SIGINT, interrupt);
 		fprintf(stderr, "%s\n", _("If you want break, press Ctrl+C..."));
 
-		GSM_SetSendSMSStatusCallback(s, SendSMSStatus);
+		GSM_SetSendSMSStatusCallback(gsm, SendSMSStatus);
 
 		for (i=0;i<sms.Number;i++) {
 			printf(_("Sending SMS %i/%i"),i+1,sms.Number);
@@ -2516,12 +2531,12 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 			if (SMSCSet==0) CopyUnicodeString(sms.SMS[i].SMSC.Number, SMSC);
 			if (Validity.Format != 0) memcpy(&sms.SMS[i].SMSC.Validity,&Validity,sizeof(GSM_SMSValidity));
 			SMSStatus = ERR_TIMEOUT;
-			error=GSM_SendSMS(s, &sms.SMS[i]);
+			error=GSM_SendSMS(gsm, &sms.SMS[i]);
 			Print_Error(error);
 			printf(_("....waiting for network answer"));
 			fflush(stdout);
 			while (!gshutdown) {
-				GSM_ReadDevice(s,true);
+				GSM_ReadDevice(gsm,true);
 				if (SMSStatus == ERR_UNKNOWN) {
 					GSM_Terminate();
 					exit(-1);
@@ -2559,7 +2574,7 @@ void AddSMSFolder(int argc UNUSED, char *argv[])
 	GSM_Init(true);
 
 	EncodeUnicode(buffer,argv[2],strlen(argv[2]));
-	error=GSM_AddSMSFolder(s,buffer);
+	error=GSM_AddSMSFolder(gsm,buffer);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -2575,7 +2590,7 @@ void DeleteAllSMS(int argc, char *argv[])
 
 	GSM_Init(true);
 
-	error=GSM_GetSMSFolders(s, &folders);
+	error=GSM_GetSMSFolders(gsm, &folders);
 	Print_Error(error);
 
 	GetStartStop(&foldernum, NULL, 2, argc, argv);
@@ -2589,15 +2604,19 @@ void DeleteAllSMS(int argc, char *argv[])
 
 	while (error == ERR_NONE) {
 		sms.SMS[0].Folder=0x00;
-		error=GSM_GetNextSMS(s, &sms, start);
+		error=GSM_GetNextSMS(gsm, &sms, start);
 		switch (error) {
 		case ERR_EMPTY:
 			break;
+		case ERR_CORRUPTED:
+			fprintf(stderr, "\n%s\n", _("Corrupted message, skipping"));
+			error = ERR_NONE;
+			continue;
 		default:
 			Print_Error(error);
 			if (sms.SMS[0].Folder == foldernum) {
 				sms.SMS[0].Folder=0x00;
-				error=GSM_DeleteSMS(s, &sms.SMS[0]);
+				error=GSM_DeleteSMS(gsm, &sms.SMS[0]);
 				Print_Error(error);
 				printf("*");
 			}
