@@ -22,8 +22,14 @@
 /* mywstrstr */
 #include "../../common/misc/coding/coding.h"
 
+/**
+ * Helper define to check error code from fwrite.
+ */
+#define chk_fwrite(data, size, count, file) \
+	if (fwrite(data, size, count, file) != count) goto fail;
+
 /* Save SMS from phone (called Inbox sms - it's in phone Inbox) somewhere */
-static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig *Config)
+static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config)
 {
 	GSM_Error	error = ERR_NONE;
 	int 		i,j;
@@ -36,24 +42,25 @@ static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 
 	j 	= 0;
 	done 	= false;
-	for (i=0;i<sms.Number && !done;i++) {
+	for (i=0;i<sms->Number && !done;i++) {
 		strcpy(ext, "txt");
-		if (sms.SMS[i].Coding == SMS_Coding_8bit) strcpy(ext, "bin");
-		DecodeUnicode(sms.SMS[i].Number,buffer2);
+		if (sms->SMS[i].Coding == SMS_Coding_8bit) strcpy(ext, "bin");
+		DecodeUnicode(sms->SMS[i].Number,buffer2);
 		/* we loop on yy for the first SMS assuming that if xxxx_yy_00.ext is absent,
 		   any xxxx_yy_01,02, must be garbage, that can be overwritten */
 		file = NULL;
 		do {
 			sprintf(FileName,
 				"IN%02d%02d%02d_%02d%02d%02d_%02i_%s_%02i.%s",
-				sms.SMS[i].DateTime.Year, sms.SMS[i].DateTime.Month,  sms.SMS[i].DateTime.Day,
-				sms.SMS[i].DateTime.Hour, sms.SMS[i].DateTime.Minute, sms.SMS[i].DateTime.Second,
+				sms->SMS[i].DateTime.Year, sms->SMS[i].DateTime.Month,  sms->SMS[i].DateTime.Day,
+				sms->SMS[i].DateTime.Hour, sms->SMS[i].DateTime.Minute, sms->SMS[i].DateTime.Second,
 				j, buffer2, i, ext);
 			strcpy(FullName, Config->inboxpath);
 			strcat(FullName, FileName);
 			if (file) fclose(file);
 			file = fopen(FullName, "r");
-		} while ((i == 0) && (file && (++j < 100)));
+		} while ((i == 0) && file != NULL && (++j < 100));
+
 		if (file) {
 			fclose(file);
 			if (i == 0) {
@@ -63,36 +70,36 @@ static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 		}
 		errno = 0;
 
-		if ((sms.SMS[i].PDU == SMS_Status_Report) && strncasecmp(Config->deliveryreport, "log", 3) == 0) {
-			strcpy(buffer, DecodeUnicodeString(sms.SMS[i].Number));
-			WriteSMSDLog(_("Delivery report: %s to %s"), DecodeUnicodeString(sms.SMS[i].Text), buffer);
+		if ((sms->SMS[i].PDU == SMS_Status_Report) && strncasecmp(Config->deliveryreport, "log", 3) == 0) {
+			strcpy(buffer, DecodeUnicodeString(sms->SMS[i].Number));
+			WriteSMSDLog(_("Delivery report: %s to %s"), DecodeUnicodeString(sms->SMS[i].Text), buffer);
 		} else {
 #ifdef GSM_ENABLE_BACKUP
 			if (strcasecmp(Config->inboxformat, "detail") == 0) {
-				for (j=0;j<sms.Number;j++) backup.SMS[j] = &sms.SMS[j];
-				backup.SMS[sms.Number] = NULL;
+				for (j=0;j<sms->Number;j++) backup.SMS[j] = &sms->SMS[j];
+				backup.SMS[sms->Number] = NULL;
 				error = GSM_AddSMSBackupFile(FullName, &backup);
 				done = true;
 			}
 #endif
 			if (!strcasecmp(Config->inboxformat, "detail") == 0) {
 				file = fopen(FullName, "wb");
-				if (file) {
-					switch (sms.SMS[i].Coding) {
+				if (file != NULL) {
+					switch (sms->SMS[i].Coding) {
 					case SMS_Coding_Unicode_No_Compression:
 	    				case SMS_Coding_Default_No_Compression:
-					    DecodeUnicode(sms.SMS[i].Text,buffer2);
+					    DecodeUnicode(sms->SMS[i].Text,buffer2);
 					    if (strcasecmp(Config->inboxformat, "unicode") == 0) {
 						buffer[0] = 0xFE;
 	    					buffer[1] = 0xFF;
-						fwrite(buffer,1,2,file);
-						fwrite(sms.SMS[i].Text,1,strlen(buffer2)*2,file);
+						chk_fwrite(buffer,1,2,file);
+						chk_fwrite(sms->SMS[i].Text,1,strlen(buffer2)*2,file);
 					    } else {
-						fwrite(buffer2,1,strlen(buffer2),file);
+						chk_fwrite(buffer2,1,strlen(buffer2),file);
 					    }
 					    break;
 					case SMS_Coding_8bit:
-					    fwrite(sms.SMS[i].Text,1,sms.SMS[i].Length,file);
+					    chk_fwrite(sms->SMS[i].Text,1,(size_t)sms->SMS[i].Length,file);
 					default:
 					    break;
 					}
@@ -100,7 +107,7 @@ static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 				} else error = ERR_CANTOPENFILE;
 			}
 			if (error == ERR_NONE) {
-				WriteSMSDLog("%s %s", (sms.SMS[i].PDU == SMS_Status_Report ? _("Delivery report"): _("Received")), FileName);
+				WriteSMSDLog("%s %s", (sms->SMS[i].PDU == SMS_Status_Report ? _("Delivery report"): _("Received")), FileName);
 			} else {
 				WriteSMSDLog(_("Cannot save %s (%i)"), FileName, errno);
 				return ERR_CANTOPENFILE;
@@ -108,6 +115,8 @@ static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 		}
 	}
 	return ERR_NONE;
+fail:
+	return ERR_WRITING_FILE;
 }
 
 /* Find one multi SMS to sending and return it (or return ERR_EMPTY)
@@ -175,6 +184,9 @@ static GSM_Error SMSDFiles_FindOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfi
   	strcat(FullName, FileName);
 
   	File = fopen(FullName, "rb");
+	if (File == NULL) {
+		return ERR_CANTOPENFILE;
+	}
  	len  = fread(Buffer, 1, sizeof(Buffer)-2, File);
   	fclose(File);
 
@@ -295,7 +307,9 @@ static GSM_Error SMSDFiles_FindOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 }
 
 /* After sending SMS is moved to Sent Items or Error Items. */
-static GSM_Error SMSDFiles_MoveSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config, unsigned char *ID, bool alwaysDelete, bool sent)
+static GSM_Error SMSDFiles_MoveSMS(GSM_MultiSMSMessage *sms UNUSED,
+		GSM_SMSDConfig *Config, unsigned char *ID,
+		bool alwaysDelete, bool sent)
 {
 	FILE 	*oFile,*iFile;
 	int 	ilen = 0, olen = 0;
@@ -316,9 +330,15 @@ static GSM_Error SMSDFiles_MoveSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Con
 
 	if (!strncasecmp(ifilename, ofilename, strlen(ofilename) == 0)) {
 		iFile = fopen(ifilename, "r");
+		if (iFile == NULL) {
+			return ERR_CANTOPENFILE;
+		}
 		ilen = fread(Buffer, 1, sizeof(Buffer), iFile);
 		fclose(iFile);
 		oFile = fopen(ofilename, "w");
+		if (oFile == NULL) {
+			return ERR_CANTOPENFILE;
+		}
 		olen = fwrite(Buffer, 1, ilen, oFile);
 		fclose(oFile);
 	}
@@ -338,7 +358,9 @@ static GSM_Error SMSDFiles_MoveSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Con
 	}
 }
 
-static GSM_Error SMSDFiles_AddSentSMSInfo(GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config, unsigned char *ID, int Part, GSM_SMSDSendingError err, int TPMR)
+static GSM_Error SMSDFiles_AddSentSMSInfo(GSM_MultiSMSMessage *sms UNUSED,
+		GSM_SMSDConfig *Config, unsigned char *ID UNUSED,
+		int Part, GSM_SMSDSendingError err, int TPMR UNUSED)
 {
 	if (err == SMSD_SEND_OK) WriteSMSDLog(_("Transmitted %s (%s: %i) to %s"), Config->SMSID, (Part == sms->Number?"total":"part"),Part,DecodeUnicodeString(sms->SMS[0].Number));
 
