@@ -26,7 +26,7 @@ static GSM_Error SMSDPgSQL_Init(GSM_SMSDConfig * Config)
 	unsigned char buf[400];
 	PGresult *Res;
 
-	unsigned int port = 0;
+	unsigned int port = 5432;
 	char *pport;
 
 	pport = strstr(Config->PC, ":");
@@ -117,7 +117,6 @@ static GSM_Error SMSDPgSQL_Init(GSM_SMSDConfig * Config)
 		return ERR_UNKNOWN;
 	}
 	if (atoi(PQgetvalue(Res, 0, 0)) < 7) {
-		//    if (atoi(Row[0]) < 7) {
 		WriteSMSDLog(_
 			     ("DataBase structures are from older Gammu version"));
 		WriteSMSDLog(_
@@ -140,7 +139,7 @@ static GSM_Error SMSDPgSQL_InitAfterConnect(GSM_SMSDConfig * Config)
 
 	PGresult *Res;
 
-	GSM_GetIMEI(s, imei);
+	GSM_GetIMEI(gsm, imei);
 
 	sprintf(buf, "DELETE FROM phones WHERE IMEI = '%s'", imei);
 	dbgprintf("%s\n", buf);
@@ -182,7 +181,7 @@ static GSM_Error SMSDPgSQL_InitAfterConnect(GSM_SMSDConfig * Config)
 }
 
 /* Save SMS from phone (called Inbox sms - it's in phone Inbox) somewhere */
-static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
+static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage *sms,
 					GSM_SMSDConfig * Config)
 {
 	PGresult *Res = NULL;
@@ -194,12 +193,12 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 	bool found;
 	long diff;
 
-	for (i = 0; i < sms.Number; i++) {
-		if (sms.SMS[i].PDU == SMS_Status_Report) {
-			strcpy(buffer2, DecodeUnicodeString(sms.SMS[i].Number));
+	for (i = 0; i < sms->Number; i++) {
+		if (sms->SMS[i].PDU == SMS_Status_Report) {
+			strcpy(buffer2, DecodeUnicodeString(sms->SMS[i].Number));
 			if (strncasecmp(Config->deliveryreport, "log", 3) == 0) {
 				WriteSMSDLog(_("Delivery report: %s to %s"),
-					     DecodeUnicodeString(sms.SMS[i].
+					     DecodeUnicodeString(sms->SMS[i].
 								 Text),
 					     buffer2);
 			}
@@ -209,7 +208,7 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
                                         FROM sentitems WHERE \
 					DeliveryDateTime = 'epoch' AND \
 					SenderID = '%s' AND TPMR = '%i' AND DestinationNumber = '%s'",
-				Config->PhoneID, sms.SMS[i].MessageReference, buffer2);
+				Config->PhoneID, sms->SMS[i].MessageReference, buffer2);
 			dbgprintf("%s\n", buffer);
 
 			Res = PQexec(Config->DBConnPgSQL, buffer);
@@ -228,7 +227,7 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 
 				if (strcmp
 				    (PQgetvalue(Res, j, 4),
-				     DecodeUnicodeString(sms.SMS[i].SMSC.
+				     DecodeUnicodeString(sms->SMS[i].SMSC.
 							 Number))) {
 					if (Config->skipsmscnumber[0] == 0)
 						continue;
@@ -242,7 +241,7 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 				    || !strcmp(PQgetvalue(Res, j, 1),
 					       "DeliveryPending")) {
 					t_time1 = atoi(PQgetvalue(Res, j, 2));
-					t_time2 = Fill_Time_T(sms.SMS[i].DateTime);
+					t_time2 = Fill_Time_T(sms->SMS[i].DateTime);
 					diff = t_time2 - t_time1;
 
 					if (diff > -Config->deliveryreportdelay && diff < Config->deliveryreportdelay) {
@@ -255,11 +254,11 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 			if (found) {
 				sprintf(buffer, "UPDATE sentitems SET DeliveryDateTime = '%04i%02i%02i%02i%02i%02i', \
                                           Status = '",
-					sms.SMS[i].SMSCTime.Year, sms.SMS[i].SMSCTime.Month, sms.SMS[i].SMSCTime.Day, sms.SMS[i].SMSCTime.Hour, sms.SMS[i].SMSCTime.Minute,
-					sms.SMS[i].SMSCTime.Second);
+					sms->SMS[i].SMSCTime.Year, sms->SMS[i].SMSCTime.Month, sms->SMS[i].SMSCTime.Day, sms->SMS[i].SMSCTime.Hour, sms->SMS[i].SMSCTime.Minute,
+					sms->SMS[i].SMSCTime.Second);
 
 				sprintf(buffer3, "%s",
-					DecodeUnicodeString(sms.SMS[i].Text));
+					DecodeUnicodeString(sms->SMS[i].Text));
 				if (!strcmp(buffer3, "Delivered")) {
 					sprintf(buffer + strlen(buffer),
 						"DeliveryOK");
@@ -276,11 +275,11 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 
 				sprintf(buffer + strlen(buffer),
 					"', StatusError = '%i'",
-					sms.SMS[i].DeliveryStatus);
+					sms->SMS[i].DeliveryStatus);
 				sprintf(buffer + strlen(buffer),
 					" WHERE ID = '%s' AND `TPMR` = '%i'",
 					PQgetvalue(Res, j, 0),
-					sms.SMS[i].MessageReference);
+					sms->SMS[i].MessageReference);
 				dbgprintf("%s\n", buffer);
 
 				Res = PQexec(Config->DBConnPgSQL, buffer);
@@ -298,36 +297,36 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 			continue;
 		}
 
-		if (sms.SMS[i].PDU != SMS_Deliver)
+		if (sms->SMS[i].PDU != SMS_Deliver)
 			continue;
 		buffer[0] = 0;
 		sprintf(buffer + strlen(buffer), "INSERT INTO inbox \
 			(ReceivingDateTime, Text, SenderNumber, Coding, SMSCNumber, UDH, \
-			Class, TextDecoded, RecipientID) VALUES ('%04d-%02d-%02d %02d:%02d:%02d', '", sms.SMS[i].DateTime.Year, sms.SMS[i].DateTime.Month, sms.SMS[i].DateTime.Day, sms.SMS[i].DateTime.Hour, sms.SMS[i].DateTime.Minute, sms.SMS[i].DateTime.Second);
+			Class, TextDecoded, RecipientID) VALUES ('%04d-%02d-%02d %02d:%02d:%02d', '", sms->SMS[i].DateTime.Year, sms->SMS[i].DateTime.Month, sms->SMS[i].DateTime.Day, sms->SMS[i].DateTime.Hour, sms->SMS[i].DateTime.Minute, sms->SMS[i].DateTime.Second);
 
-		switch (sms.SMS[i].Coding) {
+		switch (sms->SMS[i].Coding) {
 			case SMS_Coding_Unicode_No_Compression:
 
 			case SMS_Coding_Default_No_Compression:
 				EncodeHexUnicode(buffer + strlen(buffer),
-						 sms.SMS[i].Text,
-						 UnicodeLength(sms.SMS[i].
+						 sms->SMS[i].Text,
+						 UnicodeLength(sms->SMS[i].
 							       Text));
 				break;
 
 			case SMS_Coding_8bit:
 				EncodeHexBin(buffer + strlen(buffer),
-					     sms.SMS[i].Text,
-					     sms.SMS[i].Length);
+					     sms->SMS[i].Text,
+					     sms->SMS[i].Length);
 
 			default:
 				break;
 		}
 
 		sprintf(buffer + strlen(buffer), "','%s','",
-			DecodeUnicodeString(sms.SMS[i].Number));
+			DecodeUnicodeString(sms->SMS[i].Number));
 
-		switch (sms.SMS[i].Coding) {
+		switch (sms->SMS[i].Coding) {
 			case SMS_Coding_Unicode_No_Compression:
 				sprintf(buffer + strlen(buffer),
 					"Unicode_No_Compression");
@@ -354,30 +353,33 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage sms,
 		}
 
 		sprintf(buffer + strlen(buffer), "','%s'",
-			DecodeUnicodeString(sms.SMS[i].SMSC.Number));
+			DecodeUnicodeString(sms->SMS[i].SMSC.Number));
 
-		if (sms.SMS[i].UDH.Type == UDH_NoUDH) {
+		if (sms->SMS[i].UDH.Type == UDH_NoUDH) {
 			sprintf(buffer + strlen(buffer), ",''");
 		} else {
 			sprintf(buffer + strlen(buffer), ",'");
 			EncodeHexBin(buffer + strlen(buffer),
-				     sms.SMS[i].UDH.Text,
-				     sms.SMS[i].UDH.Length);
+				     sms->SMS[i].UDH.Text,
+				     sms->SMS[i].UDH.Length);
 			sprintf(buffer + strlen(buffer), "'");
 		}
 
-		sprintf(buffer + strlen(buffer), ",'%i','", sms.SMS[i].Class);
+		sprintf(buffer + strlen(buffer), ",'%i','", sms->SMS[i].Class);
 
-		switch (sms.SMS[i].Coding) {
-				int error;
+		switch (sms->SMS[i].Coding) {
 
 			case SMS_Coding_Unicode_No_Compression:
 
 			case SMS_Coding_Default_No_Compression:
-				EncodeUTF8(buffer2, sms.SMS[i].Text);
+				EncodeUTF8(buffer2, sms->SMS[i].Text);
+#ifdef HAVE_PQESCAPESTRINGCONN
 				PQescapeStringConn(Config->DBConnPgSQL, buffer4,
 						   buffer2, strlen(buffer2),
-						   &error);
+						   NULL);
+#else
+				PQescapeString(buffer4, buffer2, strlen(buffer2));
+#endif
 				memcpy(buffer + strlen(buffer), buffer4,
 				       strlen(buffer4) + 1);
 				break;
@@ -613,9 +615,9 @@ static GSM_Error SMSDPgSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms,
 }
 
 /* After sending SMS is moved to Sent Items or Error Items. */
-static GSM_Error SMSDPgSQL_MoveSMS(GSM_MultiSMSMessage * sms,
+static GSM_Error SMSDPgSQL_MoveSMS(GSM_MultiSMSMessage * sms UNUSED,
 				   GSM_SMSDConfig * Config, unsigned char *ID,
-				   bool alwaysDelete, bool sent)
+				   bool alwaysDelete UNUSED, bool sent UNUSED)
 {
 	unsigned char buffer[10000];
 	PGresult *Res;
@@ -656,7 +658,6 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 	unsigned char buffer[10000], buffer2[400], buffer4[10000], buffer5[400];
 	int i, ID;
 	int numb_rows, numb_tuples;
-	int error;
 	PGresult *Res;
 
 	sprintf(buffer, "SELECT ID FROM outbox ORDER BY ID DESC LIMIT 1");
@@ -798,9 +799,13 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 
 			case SMS_Coding_Default_No_Compression:
 				EncodeUTF8(buffer2, sms->SMS[i].Text);
+#ifdef HAVE_PQESCAPESTRINGCONN
 				PQescapeStringConn(Config->DBConnPgSQL, buffer5,
 						   buffer2, strlen(buffer2),
-						   &error);
+						   NULL);
+#else
+				PQescapeString(buffer5, buffer2, strlen(buffer2));
+#endif
 				memcpy(buffer + strlen(buffer), buffer5,
 				       strlen(buffer5) + 1);
 				break;
@@ -914,7 +919,6 @@ static GSM_Error SMSDPgSQL_AddSentSMSInfo(GSM_MultiSMSMessage * sms,
 	PGresult *Res;
 
 	unsigned char buffer[10000], buffer2[400], buff[50], buffer5[400];
-	int escape_error;
 
 	if (err == SMSD_SEND_OK) {
 		WriteSMSDLog(_("Transmitted %s (%s: %i) to %s"), Config->SMSID,
@@ -1009,9 +1013,13 @@ static GSM_Error SMSDPgSQL_AddSentSMSInfo(GSM_MultiSMSMessage * sms,
 
 		case SMS_Coding_Default_No_Compression:
 			EncodeUTF8(buffer2, sms->SMS[Part - 1].Text);
+#ifdef HAVE_PQESCAPESTRINGCONN
 			PQescapeStringConn(Config->DBConnPgSQL, buffer5,
 					   buffer2, strlen(buffer2),
-					   &escape_error);
+					   NULL);
+#else
+			PQescapeString(buffer5, buffer2, strlen(buffer2));
+#endif
 			memcpy(buffer + strlen(buffer), buffer5,
 			       strlen(buffer5) + 1);
 			break;
@@ -1048,13 +1056,11 @@ static GSM_Error SMSDPgSQL_RefreshPhoneStatus(GSM_SMSDConfig * Config)
 {
 	PGresult *Res;
 
-	unsigned char buffer[500], imei[100];
-
-	GSM_GetIMEI(s, imei);
+	unsigned char buffer[500];
 
 	sprintf(buffer,
 		"UPDATE phones SET TimeOut= now() + INTERVAL '10 seconds'");
-	sprintf(buffer + strlen(buffer), " WHERE IMEI = '%s'", imei);
+	sprintf(buffer + strlen(buffer), " WHERE IMEI = '%s'", Config->IMEI);
 	dbgprintf("%s\n", buffer);
 
 	Res = PQexec(Config->DBConnPgSQL, buffer);
