@@ -5,12 +5,9 @@
 #ifdef HAVE_MYSQL_MYSQL_H
 #ifdef WIN32
 #  include <winsock2.h>
-#  include <mysql.h>
-#  include <mysqld_error.h>
-#else
-#  include <mysql/mysql.h>
-#  include <mysql/mysqld_error.h>
 #endif
+#include <mysql.h>
+#include <mysqld_error.h>
 #endif
 
 #ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
@@ -30,6 +27,8 @@ typedef struct {
 	unsigned int	resetfrequency;
 	unsigned char   *deliveryreport, *logfilename,  *PINCode;
 	unsigned char	*PhoneID;
+	unsigned char   *RunOnReceive;
+	bool checksecurity;
 
 	/* options for FILES */
 	unsigned char   *inboxpath, 	 *outboxpath, 	*sentsmspath;
@@ -41,6 +40,7 @@ typedef struct {
 	int		currdeliveryreport;
 	unsigned char 	SMSID[200],	 prevSMSID[200];
 	GSM_SMSC	SMSC;
+	char		IMEI[GSM_MAX_IMEI_LENGTH];
 
 #if defined(HAVE_MYSQL_MYSQL_H) || defined(HAVE_POSTGRESQL_LIBPQ_FE_H)
 	/* options for SQL database */
@@ -74,7 +74,7 @@ typedef enum {
 typedef struct {
 	GSM_Error	(*Init) 	      (GSM_SMSDConfig *Config);
 	GSM_Error	(*InitAfterConnect)   (GSM_SMSDConfig *Config);
-	GSM_Error	(*SaveInboxSMS)       (GSM_MultiSMSMessage  sms, GSM_SMSDConfig *Config);
+	GSM_Error	(*SaveInboxSMS)       (GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config);
 	GSM_Error	(*FindOutboxSMS)      (GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config, unsigned char *ID);
 	GSM_Error	(*MoveSMS)  	      (GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config, unsigned char *ID, bool alwaysDelete, bool sent);
 	GSM_Error	(*CreateOutboxSMS)    (GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Config);
