@@ -243,6 +243,41 @@ typedef enum {
 	 * Gammu is not configured.
 	 */
 	ERR_UNCONFIGURED,
+	/**
+	 * Wrong folder selected (eg. for SMS).
+	 */
+	ERR_WRONGFOLDER,
+	/**
+	 * Internal phone error (phone got crazy).
+	 */
+	ERR_PHONE_INTERNAL,
+	/**
+	 * Could not write to a file (on local filesystem).
+	 */
+	ERR_WRITING_FILE,
+	/**
+	 * No such section exists.
+	 */
+	ERR_NONE_SECTION,
+	/**
+	 * Using default values.
+	 */
+	ERR_USING_DEFAULTS,
+
+	/** 60 **/
+	/**
+	 * Corrupted data returned by phone.
+	 */
+	ERR_CORRUPTED,
+	/**
+	 * Bad feature string.
+	 */
+	ERR_BADFEATURE,
+
+	/**
+	 * Just marker of highest error code, should not be used.
+	 */
+	ERR_LAST_VALUE,
 } GSM_Error;
 
 /**
@@ -254,6 +289,16 @@ typedef enum {
  * \ingroup Error
  */
 const char *GSM_ErrorString(GSM_Error e);
+
+/**
+ * Returns name for error.
+ *
+ * \param e Error code.
+ * \return Text with error name
+ *
+ * \ingroup Error
+ */
+const char *GSM_ErrorName(GSM_Error e);
 #endif
 
 /* Editor configuration
