@@ -1,5 +1,5 @@
 /**
- * \info gammu-info.h
+ * \file gammu-info.h
  * \author Michal Čihař
  *
  * Phone information.
@@ -14,7 +14,7 @@
 
 #include <gammu-types.h>
 #include <gammu-error.h>
-#include <gammu-statemachine.h>
+#include <gammu-limits.h>
 
 /**
  * Find network name from given network code.
@@ -248,6 +248,8 @@ typedef struct {
  * be determined on run time.
  *
  * \ingroup Info
+ *
+ * \todo API change: should contain GSM prefix.
  */
 typedef enum {
 	/* n6110.c */
@@ -518,15 +520,83 @@ typedef enum {
 	 */
 	F_SMS_ME,
 	/**
-	 * Switching to OBEX mode is done using AT+XLNK
+	 * Switching to OBEX mode is done using AT+XLNK.
 	 */
 	F_XLNK,
+	/**
+	 * Submit messages can be saved on SM memory only.
+	 */
+	F_SUBMIT_SIM_ONLY,
+	/**
+	 * Prefer UCS-2 for phone book manipulations.
+	 */
+	F_PBK_UCS2,
+	/**
+	 * Switching to OBEX mode using AT^SQWE=3.
+	 */
+	F_SQWE,
+	/**
+	 * Do not use OBEX/AT switching even if available.
+	 */
+	F_NO_ATOBEX,
+	/**
+	 * Length of text for contact is in bytes and not chars.
+	 */
+	F_LENGTH_BYTES,
+	/**
+	 * CMGL does not list real locations for CMGR, these should be
+	 * sequential.
+	 */
+	F_BROKEN_CMGL,
+	/**
+	 * Phonebook has extra numeric field at the end.
+	 */
+	F_EXTRA_PBK_FIELD,
+	/**
+	 * Key presses can not be in unicode.
+	 */
+	F_CKPD_NO_UNICODE,
+	/**
+	 * OBEX switching using AT+CPROT even if phone does not report
+	 * it properly.
+	 */
+	F_CPROT,
+
+	/**
+	 * Just marker of highest feature code, should not be used.
+	 */
+	F_LAST_VALUE,
 } Feature;
 
+/**
+ * Converts feature value to string.
+ *
+ * \ingroup Info
+ *
+ * \param feature Feature to convert.
+ *
+ * \return Pointer to static string with string for specified feature,
+ * NULL on failure.
+ */
+const char *GSM_FeatureToString(Feature feature);
+
+/**
+ * Converts feature string to value.
+ *
+ * \ingroup Info
+ *
+ * \param feature Feature string to convert.
+ *
+ * \return Feature value, 0 on failure.
+ */
+Feature GSM_FeatureFromString(const char * feature);
+
 /**
  * Model identification, used for finding phone features.
  *
  * \ingroup Info
+ *
+ * \todo API change: should contain GSM prefix.
  */
 typedef struct {
 	/**
@@ -544,7 +614,7 @@ typedef struct {
 	/**
 	 * List of supported features
 	 */
-	Feature features[14];
+	Feature features[GSM_MAX_PHONE_FEATURES + 1];
 } OnePhoneModel;
 
 /**
@@ -559,6 +629,20 @@ typedef struct {
  */
 bool GSM_IsPhoneFeatureAvailable(OnePhoneModel * model, Feature feature);
 
+/**
+ * Adds feature to phone configuration.
+ *
+ * \param model Model information (you can get it using \ref GSM_GetModelInfo).
+ * \param feature Feature to check for.
+ *
+ * \return True if phone has defined this feature.
+ *
+ * \ingroup Info
+ */
+bool GSM_AddPhoneFeature(OnePhoneModel *model, Feature feature);
+
+#include <gammu-statemachine.h>
+
 /**
  * Reads manufacturer from phone.
  *
