@@ -551,6 +551,12 @@ typedef struct {
 GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage * SMS, unsigned char *buffer,
 			     GSM_SMSMessageLayout Layout);
 
+/**
+ * Finds out coding type based on TPDCS header byte as defined by GSM
+ * 03.38.
+ */
+GSM_Coding_Type GSM_GetMessageCoding(const char TPDCS);
+
 /**
  * Encodes SMS frame.
  *
@@ -591,6 +597,17 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader * UDH);
  */
 void GSM_EncodeUDHHeader(GSM_UDHHeader * UDH);
 
+/**
+ * Sets default content for SMS except for changing locations. 
+ * Use this for clearing structure while keeping location of message.
+ *
+ * \param SMS Pointer to structure which should be cleaned up.
+ *
+ * \ingroup SMS
+ */
+void GSM_SetDefaultReceivedSMSData(GSM_SMSMessage * SMS);
+
+
 /**
  * Sets default content for SMS. Use this for clearing structure.
  *
@@ -990,8 +1007,14 @@ typedef enum {
  */
 typedef struct {
 	GSM_File File;
-	unsigned char ContentType[400];	// CT in Unicode
-	unsigned char SMIL[400];	// Smil ID in Unicode
+	/**
+	 * CT in Unicode
+	 */
+	unsigned char ContentType[400];
+	/**
+	 * Smil ID in Unicode
+	 */
+	unsigned char SMIL[400];
 } GSM_EncodedMultiPartMMSEntry;
 
 /**
@@ -1002,16 +1025,34 @@ typedef struct {
 typedef struct {
 	int EntriesNum;
 
-	unsigned char Source[200];	// in Unicode
+	/**
+	 * in Unicode
+	 */
+	unsigned char Source[200];
 	MMSAddressType SourceType;
-	unsigned char Destination[200];	// in Unicode
+	/**
+	 * in Unicode
+	 */
+	unsigned char Destination[200];
 	MMSAddressType DestinationType;
-	unsigned char CC[200];	// in Unicode
+	/**
+	 * in Unicode
+	 */
+	unsigned char CC[200];
 	MMSAddressType CCType;
 
-	unsigned char Subject[200];	// in Unicode
-	unsigned char ContentType[400];	// CT in Unicode
-	unsigned char MSGType[50];	// no Unicode
+	/**
+	 * in Unicode
+	 */
+	unsigned char Subject[200];
+	/**
+	 * CT in Unicode
+	 */
+	unsigned char ContentType[400];
+	/**
+	 * no Unicode
+	 */
+	unsigned char MSGType[50];
 
 	bool DateTimeAvailable;
 	GSM_DateTime DateTime;
