@@ -17,12 +17,18 @@ int main(int argc, char **argv)
 	int pos = 0;
 	GSM_MemoryEntry pbk;
 	GSM_Error error;
-	char buffer[2048];
+	char buffer[65536];
 	FILE *f;
 	size_t len;
 	bool generate = false;
 	GSM_Backup backup;
 	int i;
+	GSM_Debug_Info *debug_info;
+
+	/* Configure debugging */
+	debug_info = GSM_GetGlobalDebug();
+	GSM_SetDebugFileDescriptor(stderr, debug_info);
+	GSM_SetDebugLevel("textall", debug_info);
 
 	/* Check parameters */
 	if (argc != 3 && argc != 4) {
@@ -76,7 +82,7 @@ int main(int argc, char **argv)
 
 	/* Read file content */
 	GSM_ClearBackup(&backup);
-	if (GSM_ReadBackupFile(argv[2], &backup) != ERR_NONE) {
+	if (GSM_ReadBackupFile(argv[2], &backup, GSM_Backup_GammuUCS2) != ERR_NONE) {
 		printf("Error reading backup from %s\n", argv[2]);
 		return 1;
 	}
